/** Implements a very color cycle filter. 
 *
 * Authors: Taneli Korhonen, ttkorhon@cc.hut.fi
 *
 */


#include <SDL/SDL.h>
#include <tcl.h>
#include "filters.h"
#include "oil.h"
#include "debug.h"

/* Function prototypes. */

static Uint32 getpixel16(SDL_Surface *surface, int x, int y);
static void putpixel16(SDL_Surface *surface, int x, int y, Uint32 colour);
SDL_Surface *colorCycleFilterCreate(SDL_Surface *originalSurface, 
			       Tcl_HashTable *options,
			       void **filterData);
static void colorCycleFilterRender(SDL_Surface *originalSurface, 
			    SDL_Surface *renderSurface,
			    int offsetX, int offsetY, 
			    void *filterData);


static Uint32 getpixel16(SDL_Surface *surface, int x, int y) {
  Uint8 *p;
  
  int bpp = surface->format->BytesPerPixel;
  p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

  return *(Uint16 *)p;   
}


static void putpixel16(SDL_Surface *surface, int x, int y, Uint32 colour) {
  Uint16 *bufp;

  bufp = (Uint16 *)surface->pixels + y*surface->pitch/2 + x;
  *bufp = colour;
}


int colorCycleFilterInit() {
  Oil_AddFilter("colorCycle", colorCycleFilterCreate, colorCycleFilterRender);
  return TCL_OK;
}


SDL_Surface *colorCycleFilterCreate(SDL_Surface *originalSurface, 
			       Tcl_HashTable *options,
			       void **filterData) {
  return Oil_CreateDisplaySurface16(originalSurface->w, originalSurface->h);
}


/* Maximum size for the surface */
#define SIZE_X 1280
#define SIZE_Y 960
#define CYCLE_AMOUNT 10
static void colorCycleFilterRender(SDL_Surface *originalSurface, 
				   SDL_Surface *renderSurface, int offsetX, 
				   int offsetY,
				   void *filterData) {
  int x, y, addition;
  static int speedcounter = 0, speed = 1;
  static int init_done = 0;
  static Uint8 r[SIZE_X][SIZE_Y];
  static Uint8 g[SIZE_X][SIZE_Y];
  static Uint8 b[SIZE_X][SIZE_Y];

  Uint32 srcPixel, newPixel;
  Uint8 r_pix, g_pix, b_pix;

  /* :NOTE: This routine is buggy: doesn't account for big surfaces and
     wont work with many colour cycling objects. */

  if( init_done == 0 ) {
    /* Copy the source surface into three arrays (one for each color component) */
    int i,j;

    for( i = 0; i < originalSurface->w; i++ ) {
      for( j = 0; j < originalSurface->h; j++ ) {
	srcPixel = getpixel16( originalSurface, i+offsetX, j+offsetY );
	SDL_GetRGB( srcPixel, renderSurface->format, &r_pix, &g_pix, &b_pix );
	r[i][j] = r_pix;
	g[i][j] = g_pix;
	b[i][j] = b_pix;
      }
    }
    ++init_done;
    //	  D1PRINTF(("colorCycleFilter: init done"));
  } else if (speedcounter % speed == 0) {
    //	  D1PRINTF(("colorCycleFilter: cycling"));

    SDL_LockSurface(originalSurface);
    SDL_LockSurface(renderSurface);
	  
    addition = speedcounter << 3;
    for(x=0; x < originalSurface->w; x++) {      
      for(y=0; y < originalSurface->h; y++) {
	/* Cycle the colors */
	/*
	r[x][y] += CYCLE_AMOUNT;
	g[x][y] += CYCLE_AMOUNT;
	b[x][y] += CYCLE_AMOUNT;

	newPixel = SDL_MapRGB( renderSurface->format, r[x][y], g[x][y], b[x][y] ); */

	newPixel = SDL_MapRGB(renderSurface->format, 
			      r[x][y] + addition,
			      g[x][y] + addition,
			      b[x][y] + addition);
	putpixel16(renderSurface, x+offsetX, y+offsetY, newPixel);
      }
    }

    SDL_UnlockSurface(originalSurface);
    SDL_UnlockSurface(renderSurface);
  }
  
  if( ++speedcounter > 100000)
    speedcounter = 0;
  
  return;
}
