/** Implements a very basic noise filter. 
 *
 * Authors: Kristoffer Lawson, setok@fishpool.com
 *
 * $Header: /home/sleber/cvs/liboil/colorWobbleFilter.c,v 1.3 2003/01/30 01:55:47 sleber Exp $
 */


#include <SDL/SDL.h>
#include <tcl.h>
#include "filters.h"
#include "oil.h"
#include "debug.h"

/* Function prototypes. */

static Uint32 getpixel16(SDL_Surface *surface, int x, int y);
static void putpixel16(SDL_Surface *surface, int x, int y, Uint32 colour);
SDL_Surface *colorWobbleFilterCreate(SDL_Surface *originalSurface, 
			       Tcl_HashTable *options,
			       void **filterData);
static void colorWobbleFilterRender(SDL_Surface *originalSurface, 
			    SDL_Surface *renderSurface,
			    int offsetX, int offsetY, 
			    void *filterData);


static Uint32 getpixel16(SDL_Surface *surface, int x, int y) {
  Uint8 *p;
  
  int bpp = surface->format->BytesPerPixel;
  p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

  return *(Uint16 *)p;   
}


static void putpixel16(SDL_Surface *surface, int x, int y, Uint32 colour) {
  Uint16 *bufp;

  bufp = (Uint16 *)surface->pixels + y*surface->pitch/2 + x;
  *bufp = colour;
}


int colorWobbleFilterInit() {
  Oil_AddFilter("colorWobble", colorWobbleFilterCreate, colorWobbleFilterRender);
  return TCL_OK;
}


SDL_Surface *colorWobbleFilterCreate(SDL_Surface *originalSurface, 
			       Tcl_HashTable *options,
			       void **filterData) {
  return Oil_CreateDisplaySurface16(originalSurface->w, originalSurface->h);
}


static void colorWobbleFilterRender(SDL_Surface *originalSurface, 
			    SDL_Surface *renderSurface, int offsetX, int offsetY,
			    void *filterData) {

  int x, y, rnd;
  static int speedcounter = 0;
  static int speed = 5;         /* ColorWobble speed; 1 = fastest */
  static int rand_tbl_counter = 0;
  Uint32 srcPixel, newPixel;
  Uint8 r, g, b;
  //  SDL_Rect dstrect;
  
  int rand[10000] = { 15, 41, 29, 22, 1, 8, 47, 14, 18, 10, 35, 31, 15, 10, 3, 31, 38, 43, 6, 23, 
		      45, 20, 12, 32, 22, 4, 29, 38, 48, 24, 16, 13, 46, 22, 35, 9, 34, 1, 18, 2, 
		      4, 41, 12, 9, 10, 50, 49, 17, 32, 1, 36, 37, 4, 28, 14, 36, 18, 48, 35, 25, 
		      34, 41, 48, 31, 44, 8, 40, 23, 32, 32, 35, 15, 7, 9, 37, 24, 10, 29, 21, 29, 
		      11, 8, 11, 49, 49, 23, 27, 21, 32, 40, 3, 10, 49, 25, 25, 31, 11, 29, 9, 9, 
		      48, 19, 27, 32, 36, 26, 44, 34, 50, 19, 39, 38, 3, 34, 31, 40, 13, 26, 17, 34, 
		      48, 2, 34, 6, 40, 16, 11, 46, 26, 25, 46, 43, 50, 5, 33, 2, 27, 47, 35, 29, 
		      42, 31, 3, 43, 17, 49, 31, 9, 31, 50, 3, 14, 46, 27, 25, 44, 17, 38, 25, 32, 
		      37, 3, 26, 27, 36, 31, 32, 39, 20, 39, 40, 9, 6, 15, 23, 43, 11, 3, 12, 11, 
		      12, 15, 33, 41, 44, 37, 5, 23, 46, 29, 32, 10, 2, 35, 15, 11, 35, 42, 46, 45, 
		      5, 34, 9, 30, 4, 5, 44, 6, 20, 10, 26, 9, 19, 21, 8, 2, 31, 4, 5, 9, 
		      40, 15, 4, 21, 33, 35, 49, 42, 49, 30, 9, 15, 7, 14, 37, 49, 14, 43, 27, 17, 
		      13, 5, 45, 33, 44, 48, 15, 35, 45, 5, 31, 34, 15, 41, 32, 14, 32, 18, 16, 37, 
		      50, 12, 7, 25, 4, 3, 25, 34, 8, 13, 29, 16, 36, 24, 20, 17, 3, 27, 10, 18, 
		      44, 43, 28, 26, 4, 8, 15, 1, 46, 33, 35, 35, 42, 34, 27, 40, 23, 19, 25, 29, 
		      26, 44, 22, 19, 2, 48, 30, 33, 15, 37, 19, 3, 31, 5, 3, 4, 37, 6, 45, 41, 
		      4, 43, 46, 26, 48, 26, 41, 17, 22, 43, 11, 26, 44, 25, 28, 30, 12, 17, 25, 7, 
		      5, 15, 30, 42, 3, 30, 22, 49, 19, 9, 14, 44, 17, 21, 45, 6, 8, 20, 41, 19, 
		      4, 48, 50, 48, 1, 43, 39, 32, 19, 16, 48, 33, 43, 1, 3, 2, 24, 19, 26, 40, 
		      36, 16, 29, 49, 22, 36, 24, 10, 39, 29, 4, 30, 8, 40, 38, 39, 7, 22, 45, 22, 
		      29, 26, 18, 16, 6, 39, 22, 28, 13, 6, 29, 16, 4, 1, 45, 27, 36, 19, 17, 17, 
		      26, 32, 15, 32, 43, 16, 35, 7, 15, 1, 7, 26, 36, 46, 25, 16, 24, 20, 22, 38, 
		      49, 16, 19, 50, 43, 10, 45, 8, 47, 8, 40, 39, 12, 17, 21, 19, 38, 29, 22, 34, 
		      30, 2, 33, 3, 6, 32, 42, 17, 48, 25, 1, 30, 8, 34, 1, 6, 48, 21, 2, 29, 
		      22, 25, 47, 34, 22, 45, 39, 45, 44, 20, 5, 38, 23, 36, 49, 34, 40, 11, 6, 1, 
		      6, 8, 3, 6, 3, 14, 39, 2, 30, 45, 37, 9, 2, 41, 23, 39, 34, 4, 47, 45, 
		      50, 12, 35, 45, 36, 7, 24, 1, 27, 1, 6, 48, 32, 41, 47, 8, 9, 6, 46, 10, 
		      15, 41, 47, 43, 8, 9, 42, 42, 40, 44, 25, 38, 46, 14, 33, 15, 43, 48, 40, 42, 
		      40, 20, 44, 33, 27, 38, 20, 39, 45, 7, 15, 7, 2, 28, 39, 27, 43, 46, 20, 21, 
		      50, 6, 37, 11, 3, 39, 27, 34, 48, 8, 17, 8, 19, 40, 5, 10, 45, 42, 15, 27, 
		      37, 38, 48, 18, 30, 3, 35, 45, 44, 25, 25, 17, 30, 18, 5, 34, 3, 40, 44, 6, 
		      47, 18, 28, 37, 35, 33, 49, 34, 16, 45, 36, 29, 41, 3, 38, 8, 10, 12, 29, 49, 
		      38, 24, 36, 22, 47, 44, 26, 2, 34, 48, 15, 11, 15, 8, 30, 35, 5, 17, 6, 24, 
		      37, 17, 1, 34, 46, 7, 38, 15, 12, 24, 14, 35, 3, 31, 9, 12, 40, 6, 41, 7, 
		      30, 32, 33, 8, 17, 7, 21, 22, 34, 35, 3, 17, 6, 24, 6, 49, 31, 24, 29, 34, 
		      19, 6, 12, 20, 14, 48, 38, 38, 40, 48, 28, 43, 37, 19, 28, 14, 45, 45, 20, 33, 
		      8, 42, 36, 3, 19, 49, 37, 39, 12, 11, 37, 40, 30, 9, 27, 27, 13, 21, 30, 7, 
		      31, 23, 9, 37, 12, 23, 43, 25, 15, 28, 28, 41, 6, 21, 31, 13, 12, 5, 48, 46, 
		      42, 25, 48, 14, 41, 45, 49, 44, 8, 50, 19, 32, 5, 32, 36, 15, 3, 43, 23, 5, 
		      7, 25, 3, 26, 25, 8, 43, 32, 17, 47, 33, 14, 40, 40, 8, 8, 16, 30, 30, 16, 
		      40, 28, 47, 17, 49, 29, 30, 39, 20, 18, 9, 26, 8, 24, 21, 19, 24, 47, 45, 11, 
		      14, 32, 18, 43, 34, 13, 36, 8, 36, 20, 11, 50, 39, 24, 30, 4, 39, 29, 50, 37, 
		      21, 49, 2, 23, 44, 2, 50, 1, 14, 49, 12, 15, 42, 28, 39, 48, 30, 5, 18, 43, 
		      44, 11, 24, 49, 40, 26, 48, 18, 7, 29, 24, 47, 50, 31, 15, 2, 17, 11, 8, 11, 
		      38, 41, 38, 8, 11, 10, 1, 1, 43, 41, 8, 43, 13, 29, 41, 22, 39, 42, 17, 16, 
		      21, 38, 3, 7, 40, 48, 6, 7, 41, 30, 37, 20, 36, 9, 49, 33, 23, 21, 16, 20, 
		      6, 37, 16, 30, 15, 37, 49, 8, 18, 6, 26, 33, 35, 19, 26, 12, 12, 32, 5, 45, 
		      18, 34, 13, 16, 48, 32, 50, 24, 37, 8, 19, 50, 41, 8, 6, 23, 7, 33, 26, 39, 
		      46, 34, 25, 37, 47, 33, 11, 10, 46, 34, 8, 23, 19, 23, 12, 40, 14, 19, 22, 14, 
		      1, 28, 25, 34, 8, 43, 23, 18, 17, 12, 37, 9, 8, 6, 11, 29, 39, 6, 7, 19, 
		      42, 9, 21, 33, 11, 32, 24, 2, 47, 30, 44, 29, 29, 29, 47, 28, 14, 20, 30, 37, 
		      9, 24, 36, 32, 9, 25, 18, 48, 15, 37, 23, 35, 4, 7, 25, 10, 28, 31, 19, 39, 
		      11, 47, 22, 16, 36, 50, 22, 12, 49, 25, 32, 45, 33, 39, 34, 12, 5, 12, 17, 28, 
		      24, 36, 16, 6, 19, 39, 38, 30, 50, 22, 4, 48, 24, 33, 41, 29, 23, 46, 38, 33, 
		      41, 20, 9, 43, 14, 12, 25, 47, 10, 37, 2, 7, 33, 3, 1, 4, 5, 44, 44, 36, 
		      13, 23, 42, 18, 8, 47, 26, 23, 24, 28, 20, 5, 29, 14, 25, 24, 21, 9, 27, 45, 
		      21, 16, 2, 33, 30, 34, 30, 4, 1, 26, 25, 2, 38, 40, 8, 39, 26, 34, 22, 22, 
		      48, 39, 19, 36, 1, 26, 7, 6, 27, 22, 36, 27, 39, 10, 19, 31, 34, 18, 41, 18, 
		      33, 48, 6, 19, 35, 38, 6, 2, 3, 31, 34, 50, 33, 36, 11, 13, 15, 13, 47, 36, 
		      2, 4, 39, 39, 40, 35, 43, 10, 28, 10, 18, 34, 17, 47, 8, 36, 29, 43, 22, 33, 
		      41, 47, 40, 12, 46, 7, 44, 13, 24, 29, 24, 3, 38, 45, 35, 2, 46, 22, 49, 37, 
		      3, 5, 28, 10, 40, 49, 7, 4, 10, 46, 41, 14, 20, 1, 10, 17, 45, 1, 1, 21, 
		      35, 6, 47, 42, 16, 10, 33, 23, 50, 45, 15, 41, 28, 23, 17, 21, 27, 13, 3, 33, 
		      49, 32, 3, 37, 35, 46, 48, 32, 40, 37, 10, 30, 23, 11, 20, 36, 50, 18, 41, 41, 
		      39, 37, 26, 47, 21, 2, 35, 14, 36, 21, 5, 9, 4, 22, 12, 4, 30, 40, 40, 30, 
		      11, 27, 10, 41, 31, 28, 29, 7, 31, 44, 35, 36, 23, 33, 44, 31, 6, 8, 19, 16, 
		      41, 44, 7, 7, 24, 29, 43, 45, 7, 11, 31, 33, 28, 33, 34, 47, 24, 18, 47, 12, 
		      31, 50, 28, 22, 7, 2, 18, 5, 18, 43, 49, 45, 39, 12, 48, 32, 1, 33, 6, 45, 
		      39, 17, 49, 28, 26, 30, 30, 23, 26, 47, 17, 27, 25, 45, 9, 1, 23, 49, 12, 32, 
		      38, 22, 24, 34, 50, 3, 3, 45, 24, 20, 41, 36, 4, 45, 12, 41, 33, 11, 2, 36, 
		      17, 50, 46, 33, 17, 22, 36, 9, 2, 13, 16, 9, 38, 6, 50, 44, 39, 36, 9, 18, 
		      22, 36, 38, 42, 28, 11, 34, 37, 5, 41, 49, 27, 5, 9, 31, 37, 26, 24, 2, 10, 
		      12, 32, 31, 34, 45, 43, 2, 3, 27, 42, 27, 33, 17, 10, 31, 43, 46, 45, 11, 16, 
		      12, 7, 15, 4, 15, 15, 1, 1, 37, 42, 48, 44, 29, 48, 37, 48, 18, 23, 5, 30, 
		      27, 28, 33, 44, 11, 38, 13, 46, 19, 33, 8, 45, 31, 18, 37, 6, 4, 46, 28, 9, 
		      8, 31, 3, 18, 27, 42, 29, 6, 3, 38, 22, 28, 26, 19, 50, 46, 38, 41, 33, 21, 
		      22, 38, 23, 31, 37, 47, 9, 35, 31, 36, 15, 31, 42, 19, 24, 50, 3, 50, 43, 14, 
		      33, 3, 28, 22, 30, 6, 1, 7, 40, 24, 7, 42, 42, 29, 6, 32, 9, 23, 41, 1, 
		      17, 45, 1, 31, 12, 30, 7, 8, 18, 9, 45, 12, 20, 41, 35, 11, 11, 13, 16, 46, 
		      20, 21, 13, 44, 39, 37, 4, 10, 28, 23, 35, 42, 7, 33, 38, 29, 6, 45, 1, 50, 
		      22, 35, 15, 12, 42, 17, 48, 15, 16, 10, 49, 35, 14, 38, 2, 40, 21, 28, 6, 17, 
		      40, 7, 1, 1, 14, 31, 45, 42, 22, 38, 46, 11, 48, 37, 39, 33, 15, 45, 31, 40, 
		      32, 14, 2, 44, 35, 21, 47, 50, 37, 46, 21, 29, 9, 46, 48, 32, 21, 10, 19, 37, 
		      33, 40, 7, 42, 41, 5, 47, 44, 13, 43, 43, 28, 5, 32, 11, 5, 48, 36, 45, 35, 
		      23, 17, 5, 13, 46, 12, 15, 13, 18, 11, 35, 22, 3, 30, 35, 42, 3, 33, 15, 2, 
		      22, 5, 48, 39, 23, 24, 34, 25, 1, 22, 16, 6, 21, 38, 15, 34, 17, 46, 16, 34, 
		      41, 7, 5, 32, 25, 10, 32, 44, 22, 26, 50, 1, 46, 27, 33, 6, 48, 30, 11, 21, 
		      3, 47, 8, 24, 11, 16, 19, 1, 23, 36, 34, 46, 48, 34, 13, 9, 3, 49, 27, 47, 
		      20, 34, 3, 7, 43, 2, 1, 28, 10, 11, 15, 40, 49, 43, 42, 33, 48, 42, 29, 11, 
		      28, 41, 32, 37, 49, 10, 20, 40, 17, 19, 18, 46, 35, 19, 13, 16, 14, 8, 35, 39, 
		      12, 42, 29, 13, 39, 45, 17, 37, 22, 24, 38, 12, 40, 5, 2, 9, 23, 10, 41, 18, 
		      21, 40, 28, 15, 21, 6, 12, 43, 25, 26, 20, 1, 31, 36, 18, 47, 3, 10, 2, 47, 
		      29, 44, 40, 32, 47, 25, 42, 44, 32, 33, 43, 22, 26, 23, 9, 28, 31, 12, 37, 32, 
		      23, 49, 41, 42, 37, 11, 21, 40, 17, 35, 43, 35, 49, 28, 42, 20, 12, 13, 18, 16, 
		      31, 38, 15, 25, 29, 30, 31, 47, 18, 6, 34, 47, 44, 15, 48, 6, 16, 38, 42, 35, 
		      22, 8, 50, 6, 50, 46, 15, 2, 9, 4, 44, 38, 24, 3, 7, 7, 6, 14, 9, 37, 
		      20, 5, 23, 34, 19, 36, 40, 49, 47, 33, 9, 8, 34, 6, 4, 17, 38, 15, 37, 2, 
		      32, 20, 12, 11, 7, 10, 43, 11, 16, 38, 10, 47, 17, 48, 16, 23, 5, 4, 9, 1, 
		      38, 32, 31, 38, 9, 31, 46, 27, 43, 41, 22, 24, 39, 11, 34, 6, 20, 11, 2, 47, 
		      18, 27, 10, 29, 15, 1, 27, 37, 27, 37, 19, 19, 24, 9, 26, 4, 2, 37, 30, 37, 
		      45, 8, 45, 7, 16, 20, 28, 48, 24, 10, 30, 31, 40, 36, 22, 13, 3, 42, 49, 48, 
		      46, 1, 32, 13, 13, 21, 48, 30, 25, 50, 9, 9, 33, 5, 1, 29, 31, 18, 27, 32, 
		      48, 27, 3, 35, 18, 15, 27, 24, 41, 40, 22, 44, 32, 27, 21, 3, 50, 10, 30, 40, 
		      1, 25, 41, 35, 11, 8, 11, 42, 7, 10, 23, 41, 1, 37, 37, 23, 39, 28, 28, 4, 
		      19, 39, 12, 31, 15, 4, 39, 10, 27, 16, 26, 22, 21, 46, 21, 22, 34, 41, 9, 8, 
		      34, 20, 36, 48, 7, 28, 46, 33, 49, 28, 50, 24, 49, 8, 33, 11, 40, 17, 11, 24, 
		      10, 8, 26, 4, 39, 19, 37, 37, 44, 48, 27, 12, 47, 34, 25, 19, 15, 39, 39, 4, 
		      13, 34, 22, 6, 21, 37, 32, 35, 5, 3, 45, 44, 14, 24, 15, 27, 29, 49, 50, 11, 
		      23, 43, 15, 23, 2, 33, 13, 48, 14, 30, 26, 7, 50, 11, 1, 35, 1, 49, 20, 30, 
		      43, 10, 10, 19, 50, 20, 2, 37, 25, 20, 49, 7, 16, 15, 6, 34, 8, 30, 10, 49, 
		      35, 8, 4, 9, 10, 1, 27, 24, 37, 8, 8, 12, 49, 26, 26, 43, 6, 45, 2, 29, 
		      21, 47, 32, 3, 10, 21, 14, 10, 11, 45, 2, 19, 12, 50, 8, 40, 30, 13, 16, 19, 
		      10, 43, 39, 16, 45, 2, 38, 50, 10, 8, 30, 6, 9, 42, 7, 21, 26, 27, 32, 2, 
		      37, 41, 34, 20, 14, 33, 35, 4, 30, 10, 47, 22, 44, 26, 40, 8, 34, 6, 40, 27, 
		      33, 33, 46, 30, 24, 48, 21, 38, 33, 38, 4, 18, 30, 38, 8, 26, 48, 8, 42, 38, 
		      45, 5, 25, 37, 2, 49, 2, 19, 6, 10, 28, 15, 9, 49, 24, 5, 28, 35, 15, 42, 
		      14, 30, 12, 23, 10, 45, 30, 38, 38, 26, 16, 8, 39, 6, 39, 40, 4, 20, 35, 16, 
		      19, 14, 18, 32, 3, 20, 12, 42, 50, 17, 46, 8, 8, 12, 22, 37, 13, 20, 22, 17, 
		      7, 13, 3, 15, 3, 16, 4, 24, 26, 3, 9, 34, 43, 5, 35, 33, 47, 13, 16, 6, 
		      18, 25, 43, 24, 2, 1, 49, 11, 7, 42, 24, 35, 50, 49, 31, 27, 47, 37, 39, 15, 
		      16, 27, 9, 2, 22, 46, 15, 45, 25, 49, 31, 1, 42, 12, 30, 1, 49, 23, 42, 6, 
		      46, 19, 20, 40, 9, 44, 29, 30, 14, 26, 37, 14, 45, 37, 36, 33, 50, 8, 29, 36, 
		      20, 4, 41, 30, 40, 7, 28, 12, 39, 20, 21, 46, 38, 2, 3, 42, 27, 15, 39, 49, 
		      46, 6, 12, 13, 26, 18, 13, 47, 19, 26, 23, 7, 12, 26, 17, 46, 23, 46, 28, 23, 
		      42, 42, 43, 26, 12, 39, 9, 16, 19, 9, 33, 41, 16, 27, 43, 32, 19, 6, 41, 13, 
		      25, 2, 30, 24, 44, 17, 37, 30, 13, 12, 40, 12, 9, 44, 25, 44, 42, 29, 49, 18, 
		      18, 38, 15, 6, 15, 36, 50, 30, 2, 29, 14, 34, 2, 27, 46, 19, 33, 27, 47, 4, 
		      22, 48, 20, 17, 37, 15, 32, 36, 19, 27, 49, 30, 27, 50, 27, 21, 6, 43, 8, 46, 
		      22, 27, 28, 40, 9, 4, 25, 32, 48, 4, 4, 30, 18, 27, 10, 28, 2, 10, 34, 38, 
		      40, 38, 44, 2, 7, 10, 48, 45, 32, 47, 45, 2, 23, 9, 49, 20, 1, 44, 43, 22, 
		      38, 22, 2, 38, 49, 48, 21, 7, 49, 18, 29, 50, 34, 12, 31, 37, 23, 26, 46, 38, 
		      38, 11, 36, 39, 1, 34, 28, 25, 44, 50, 30, 15, 22, 2, 31, 37, 5, 34, 19, 2, 
		      1, 33, 19, 27, 8, 40, 23, 17, 20, 30, 11, 13, 2, 45, 20, 25, 28, 29, 26, 3, 
		      21, 37, 22, 29, 35, 13, 22, 13, 38, 15, 4, 43, 28, 11, 21, 15, 35, 39, 15, 23, 
		      6, 1, 3, 42, 29, 31, 17, 33, 13, 40, 35, 2, 35, 38, 26, 33, 31, 24, 27, 5, 
		      20, 5, 1, 23, 28, 28, 24, 30, 5, 45, 50, 44, 15, 49, 31, 1, 48, 22, 20, 44, 
		      28, 5, 3, 10, 40, 36, 8, 28, 42, 27, 26, 20, 5, 17, 32, 18, 32, 21, 23, 17, 
		      24, 36, 42, 33, 36, 39, 41, 19, 32, 20, 40, 30, 18, 20, 15, 15, 32, 45, 2, 16, 
		      45, 16, 25, 20, 39, 21, 7, 45, 2, 40, 14, 9, 22, 25, 5, 3, 14, 27, 35, 8, 
		      11, 26, 18, 48, 31, 27, 20, 46, 15, 7, 21, 42, 37, 34, 38, 11, 13, 37, 2, 45, 
		      19, 24, 4, 14, 6, 25, 41, 12, 14, 28, 6, 10, 18, 34, 41, 21, 32, 44, 17, 32, 
		      20, 45, 14, 25, 21, 25, 42, 41, 19, 35, 37, 21, 10, 38, 33, 1, 34, 36, 28, 1, 
		      15, 6, 47, 11, 18, 33, 34, 31, 30, 12, 44, 3, 8, 28, 9, 4, 36, 12, 34, 4, 
		      4, 1, 9, 42, 4, 44, 23, 21, 22, 7, 18, 8, 14, 46, 29, 35, 24, 4, 46, 48, 
		      48, 38, 6, 2, 23, 44, 4, 34, 13, 22, 50, 8, 12, 19, 23, 48, 44, 21, 21, 14, 
		      24, 12, 34, 43, 6, 39, 4, 48, 3, 37, 16, 16, 1, 8, 32, 36, 22, 32, 3, 42, 
		      20, 39, 41, 41, 41, 23, 15, 33, 18, 21, 31, 14, 2, 9, 38, 35, 31, 7, 23, 37, 
		      8, 29, 35, 33, 18, 42, 1, 43, 19, 27, 34, 27, 30, 2, 13, 9, 31, 7, 38, 20, 
		      46, 34, 43, 33, 11, 9, 32, 46, 44, 37, 33, 14, 5, 5, 32, 36, 19, 18, 2, 47, 
		      45, 10, 28, 31, 32, 36, 12, 16, 14, 9, 32, 15, 40, 28, 35, 43, 28, 33, 37, 1, 
		      46, 8, 39, 32, 25, 32, 16, 23, 44, 43, 26, 29, 22, 20, 25, 47, 13, 30, 38, 15, 
		      46, 6, 8, 3, 17, 2, 38, 42, 46, 22, 11, 32, 49, 25, 41, 27, 13, 39, 28, 38, 
		      29, 35, 18, 35, 11, 28, 32, 1, 35, 34, 49, 10, 7, 26, 18, 29, 36, 15, 19, 10, 
		      11, 35, 35, 48, 44, 45, 40, 28, 4, 38, 13, 40, 36, 12, 31, 49, 18, 11, 21, 20, 
		      27, 16, 36, 19, 28, 23, 39, 13, 40, 24, 22, 7, 22, 9, 42, 14, 7, 14, 16, 42, 
		      36, 4, 43, 37, 34, 35, 35, 6, 24, 3, 23, 18, 31, 39, 49, 43, 31, 28, 39, 3, 
		      15, 4, 35, 6, 27, 48, 15, 4, 3, 39, 28, 1, 50, 50, 13, 50, 29, 8, 20, 19, 
		      25, 34, 7, 31, 8, 39, 42, 45, 10, 50, 10, 13, 12, 23, 9, 20, 35, 28, 31, 14, 
		      36, 31, 38, 40, 26, 15, 42, 44, 48, 38, 12, 46, 43, 49, 18, 11, 45, 25, 9, 36, 
		      12, 6, 36, 49, 37, 2, 4, 15, 17, 32, 15, 1, 41, 23, 47, 35, 49, 47, 33, 41, 
		      18, 44, 36, 18, 4, 18, 39, 20, 31, 39, 4, 19, 13, 11, 22, 19, 44, 24, 40, 9, 
		      26, 42, 49, 29, 20, 16, 14, 23, 19, 45, 8, 47, 5, 32, 31, 9, 28, 25, 11, 31, 
		      22, 28, 41, 37, 49, 36, 12, 28, 2, 30, 25, 24, 5, 27, 34, 46, 28, 46, 36, 28, 
		      39, 46, 37, 14, 45, 14, 20, 24, 41, 45, 39, 36, 34, 25, 32, 43, 33, 27, 16, 19, 
		      17, 24, 30, 6, 18, 25, 13, 14, 45, 28, 9, 24, 41, 32, 49, 12, 35, 2, 33, 14, 
		      50, 4, 44, 7, 28, 6, 6, 6, 34, 50, 28, 28, 29, 3, 3, 6, 17, 25, 46, 18, 
		      7, 32, 50, 5, 46, 17, 31, 39, 12, 33, 26, 23, 36, 24, 5, 7, 8, 39, 9, 48, 
		      39, 38, 50, 6, 6, 45, 23, 31, 27, 43, 38, 32, 10, 17, 1, 31, 16, 11, 15, 40, 
		      1, 30, 15, 5, 15, 1, 50, 19, 27, 16, 50, 18, 28, 16, 9, 46, 19, 34, 39, 2, 
		      24, 50, 30, 16, 33, 13, 45, 47, 28, 18, 4, 17, 44, 40, 48, 24, 10, 24, 30, 16, 
		      49, 34, 13, 43, 33, 37, 36, 23, 14, 33, 10, 37, 28, 40, 41, 10, 37, 5, 48, 17, 
		      48, 3, 29, 43, 49, 5, 41, 27, 49, 15, 50, 42, 13, 46, 18, 23, 7, 31, 21, 47, 
		      12, 46, 23, 30, 40, 2, 13, 23, 19, 12, 22, 33, 31, 45, 49, 15, 24, 14, 11, 42, 
		      18, 23, 31, 49, 18, 19, 5, 39, 29, 23, 47, 21, 15, 31, 11, 19, 41, 37, 19, 9, 
		      6, 29, 10, 41, 9, 15, 32, 25, 22, 40, 12, 10, 43, 32, 22, 38, 49, 14, 13, 22, 
		      17, 22, 48, 20, 22, 47, 4, 43, 38, 15, 22, 38, 48, 48, 37, 41, 15, 34, 31, 4, 
		      34, 1, 25, 35, 27, 19, 49, 49, 23, 13, 27, 25, 32, 12, 31, 23, 19, 7, 22, 33, 
		      21, 47, 36, 11, 47, 3, 42, 6, 43, 6, 35, 37, 32, 10, 50, 18, 15, 38, 19, 21, 
		      32, 29, 29, 6, 38, 22, 43, 9, 23, 14, 12, 6, 20, 27, 3, 12, 44, 45, 19, 16, 
		      36, 35, 25, 11, 14, 4, 50, 12, 12, 47, 1, 33, 4, 9, 12, 18, 46, 6, 37, 13, 
		      41, 50, 18, 30, 19, 50, 14, 12, 39, 34, 42, 28, 26, 14, 41, 27, 50, 48, 49, 11, 
		      34, 4, 16, 30, 10, 16, 46, 4, 21, 41, 36, 48, 2, 48, 46, 42, 14, 6, 47, 2, 
		      12, 14, 3, 46, 39, 46, 17, 40, 27, 36, 27, 3, 7, 19, 4, 22, 37, 8, 11, 12, 
		      27, 36, 4, 28, 35, 22, 21, 49, 47, 31, 49, 34, 33, 8, 21, 27, 50, 44, 23, 22, 
		      2, 3, 30, 17, 2, 20, 4, 18, 14, 46, 31, 48, 26, 14, 4, 15, 29, 29, 25, 23, 
		      16, 2, 35, 36, 30, 39, 29, 2, 18, 7, 17, 49, 44, 38, 15, 31, 19, 20, 5, 37, 
		      46, 4, 25, 6, 15, 5, 19, 14, 40, 32, 36, 28, 15, 31, 32, 42, 30, 30, 25, 18, 
		      6, 23, 18, 8, 38, 11, 17, 19, 2, 5, 10, 49, 4, 42, 7, 43, 27, 6, 7, 42, 
		      29, 49, 49, 44, 7, 49, 40, 39, 18, 26, 47, 47, 39, 8, 29, 38, 33, 50, 48, 12, 
		      9, 18, 27, 50, 28, 46, 7, 24, 12, 5, 25, 46, 41, 10, 42, 23, 20, 5, 36, 31, 
		      21, 37, 24, 23, 43, 12, 35, 30, 48, 30, 18, 2, 10, 19, 17, 10, 27, 31, 35, 11, 
		      33, 27, 31, 42, 26, 41, 27, 43, 29, 36, 46, 24, 4, 20, 38, 43, 21, 12, 44, 11, 
		      19, 10, 46, 5, 37, 35, 23, 7, 47, 49, 22, 21, 41, 37, 5, 10, 41, 7, 48, 40, 
		      9, 8, 11, 20, 43, 15, 19, 42, 24, 32, 18, 49, 8, 38, 27, 17, 17, 41, 6, 37, 
		      38, 24, 2, 37, 17, 15, 2, 50, 46, 36, 11, 43, 23, 36, 23, 44, 13, 31, 49, 35, 
		      35, 36, 34, 33, 3, 50, 50, 32, 7, 31, 3, 6, 32, 29, 19, 30, 48, 46, 18, 1, 
		      41, 48, 14, 33, 41, 20, 47, 14, 48, 31, 50, 38, 13, 7, 33, 20, 25, 39, 27, 2, 
		      11, 13, 6, 50, 28, 49, 29, 32, 7, 25, 50, 20, 31, 19, 1, 4, 28, 4, 22, 8, 
		      16, 20, 34, 42, 5, 48, 15, 16, 37, 14, 7, 6, 42, 5, 8, 39, 38, 37, 21, 33, 
		      32, 25, 3, 23, 43, 48, 30, 7, 25, 7, 45, 20, 27, 30, 28, 34, 7, 13, 6, 22, 
		      50, 36, 17, 16, 28, 44, 10, 3, 28, 47, 39, 10, 45, 39, 1, 39, 18, 7, 3, 35, 
		      46, 31, 29, 7, 39, 5, 18, 7, 40, 23, 26, 9, 27, 36, 28, 22, 7, 2, 10, 48, 
		      30, 20, 21, 45, 30, 23, 44, 16, 5, 48, 14, 44, 12, 29, 4, 35, 8, 41, 20, 22, 
		      42, 46, 32, 10, 2, 15, 48, 44, 12, 20, 45, 44, 6, 38, 35, 23, 50, 28, 17, 5, 
		      30, 37, 9, 21, 16, 14, 31, 5, 37, 18, 13, 14, 9, 20, 11, 48, 30, 29, 50, 16, 
		      3, 36, 16, 47, 16, 36, 41, 7, 15, 20, 47, 33, 15, 4, 13, 32, 3, 16, 17, 39, 
		      9, 46, 4, 37, 6, 43, 43, 21, 8, 44, 14, 2, 34, 41, 16, 45, 46, 38, 42, 11, 
		      3, 39, 8, 13, 26, 13, 28, 15, 3, 38, 43, 39, 29, 39, 4, 16, 38, 5, 19, 28, 
		      29, 40, 15, 30, 20, 26, 42, 45, 48, 34, 12, 39, 12, 11, 31, 38, 22, 47, 28, 11, 
		      15, 6, 35, 7, 1, 15, 32, 36, 20, 29, 34, 50, 3, 27, 17, 13, 32, 6, 8, 29, 
		      25, 48, 16, 20, 36, 30, 3, 9, 22, 19, 44, 36, 49, 19, 5, 38, 50, 26, 41, 28, 
		      23, 9, 19, 27, 12, 11, 14, 38, 3, 39, 42, 47, 40, 41, 5, 14, 14, 7, 30, 9, 
		      35, 45, 40, 34, 18, 46, 35, 26, 43, 1, 7, 1, 20, 12, 13, 34, 37, 27, 35, 41, 
		      3, 18, 41, 31, 13, 2, 5, 18, 42, 32, 33, 49, 11, 44, 25, 35, 37, 13, 30, 46, 
		      5, 26, 1, 25, 47, 23, 22, 3, 13, 13, 37, 45, 31, 6, 2, 3, 43, 25, 30, 34, 
		      25, 49, 9, 12, 35, 43, 8, 33, 26, 4, 10, 34, 21, 43, 22, 45, 18, 2, 5, 50, 
		      6, 6, 13, 43, 19, 2, 8, 48, 24, 27, 35, 39, 36, 11, 28, 27, 12, 20, 46, 21, 
		      13, 34, 25, 15, 49, 30, 7, 13, 13, 32, 17, 42, 23, 31, 41, 38, 8, 41, 1, 30, 
		      31, 3, 50, 27, 9, 27, 40, 31, 41, 48, 23, 3, 2, 46, 12, 11, 4, 44, 12, 36, 
		      9, 37, 15, 24, 37, 11, 1, 28, 43, 31, 3, 23, 45, 16, 50, 34, 44, 1, 25, 14, 
		      18, 21, 31, 1, 28, 33, 18, 6, 18, 5, 10, 4, 42, 38, 23, 29, 4, 18, 27, 2, 
		      19, 25, 27, 40, 7, 27, 3, 3, 48, 17, 44, 11, 47, 39, 29, 49, 34, 22, 18, 41, 
		      10, 43, 19, 29, 6, 22, 15, 5, 38, 43, 23, 29, 19, 19, 30, 26, 11, 13, 14, 27, 
		      37, 48, 39, 10, 13, 23, 50, 30, 37, 46, 1, 5, 32, 29, 37, 9, 33, 10, 1, 23, 
		      25, 13, 30, 4, 42, 3, 11, 38, 14, 23, 37, 23, 19, 33, 40, 19, 20, 35, 1, 43, 
		      11, 19, 49, 29, 35, 20, 17, 15, 22, 31, 35, 50, 8, 22, 42, 41, 20, 23, 9, 5, 
		      1, 14, 34, 46, 42, 12, 1, 2, 47, 32, 36, 26, 12, 1, 34, 49, 50, 45, 45, 39, 
		      30, 43, 25, 4, 50, 14, 26, 29, 31, 49, 24, 18, 10, 43, 11, 28, 27, 36, 15, 10, 
		      7, 26, 44, 45, 20, 21, 7, 41, 46, 47, 3, 28, 26, 21, 17, 23, 17, 25, 48, 20, 
		      2, 11, 9, 48, 3, 23, 38, 8, 8, 36, 49, 45, 33, 21, 4, 48, 19, 34, 9, 17, 
		      48, 22, 46, 21, 5, 28, 24, 20, 48, 25, 44, 41, 3, 32, 28, 43, 12, 38, 4, 37, 
		      11, 10, 50, 11, 9, 15, 47, 34, 50, 37, 21, 45, 37, 11, 50, 21, 9, 11, 37, 1, 
		      39, 29, 4, 41, 32, 37, 32, 7, 24, 29, 13, 21, 38, 27, 38, 50, 42, 25, 44, 32, 
		      2, 21, 33, 31, 4, 36, 37, 27, 11, 32, 27, 11, 11, 1, 46, 48, 46, 3, 4, 14, 
		      42, 32, 42, 48, 46, 45, 6, 15, 13, 26, 50, 3, 16, 35, 9, 21, 43, 46, 16, 46, 
		      9, 19, 31, 1, 2, 24, 7, 25, 23, 37, 18, 36, 30, 40, 16, 43, 28, 46, 14, 30, 
		      25, 11, 19, 46, 40, 26, 17, 23, 43, 2, 47, 14, 31, 1, 27, 29, 34, 19, 19, 31, 
		      33, 44, 26, 22, 44, 15, 30, 11, 28, 38, 33, 37, 39, 21, 29, 23, 39, 48, 3, 35, 
		      20, 26, 28, 3, 50, 16, 42, 15, 4, 29, 8, 24, 28, 28, 2, 37, 39, 30, 39, 11, 
		      26, 18, 45, 21, 39, 45, 27, 1, 6, 2, 33, 9, 2, 31, 47, 30, 7, 9, 5, 40, 
		      9, 2, 26, 34, 50, 27, 12, 48, 10, 50, 3, 9, 49, 5, 31, 36, 1, 32, 36, 4, 
		      36, 34, 30, 16, 42, 24, 16, 2, 47, 40, 25, 4, 35, 34, 50, 16, 3, 28, 47, 25, 
		      25, 2, 39, 25, 48, 42, 19, 1, 35, 33, 29, 30, 42, 17, 27, 35, 50, 39, 14, 23, 
		      18, 38, 18, 19, 32, 15, 24, 24, 41, 33, 29, 34, 19, 35, 37, 48, 6, 3, 27, 46, 
		      27, 1, 12, 35, 35, 50, 12, 9, 47, 35, 16, 28, 19, 21, 12, 12, 12, 39, 11, 21, 
		      28, 32, 29, 50, 46, 10, 29, 1, 11, 30, 3, 32, 13, 20, 35, 26, 50, 30, 49, 21, 
		      28, 29, 22, 27, 28, 4, 47, 19, 21, 9, 40, 6, 6, 42, 12, 11, 37, 38, 22, 19, 
		      7, 19, 19, 18, 45, 42, 49, 27, 37, 42, 28, 30, 12, 1, 35, 32, 16, 24, 50, 36, 
		      36, 15, 15, 23, 41, 2, 39, 37, 19, 45, 35, 32, 19, 43, 12, 8, 39, 4, 2, 45, 
		      42, 35, 47, 34, 42, 24, 32, 47, 46, 7, 11, 5, 46, 46, 14, 50, 23, 27, 48, 45, 
		      33, 35, 50, 11, 47, 16, 25, 33, 26, 5, 24, 19, 4, 47, 32, 39, 27, 14, 30, 27, 
		      9, 48, 34, 14, 46, 30, 6, 4, 41, 25, 45, 31, 19, 30, 14, 14, 29, 49, 45, 23, 
		      20, 7, 2, 15, 50, 24, 21, 10, 28, 20, 3, 15, 43, 37, 37, 37, 23, 18, 3, 37, 
		      22, 15, 19, 41, 3, 13, 11, 48, 42, 46, 49, 15, 48, 5, 46, 2, 47, 50, 46, 22, 
		      40, 19, 41, 43, 41, 43, 22, 22, 1, 28, 31, 45, 39, 23, 20, 13, 37, 37, 31, 32, 
		      7, 39, 33, 42, 47, 14, 43, 22, 5, 50, 35, 46, 22, 3, 10, 4, 39, 47, 3, 30, 
		      25, 43, 43, 12, 29, 10, 10, 9, 11, 44, 6, 27, 33, 22, 10, 16, 16, 23, 6, 50, 
		      39, 17, 1, 36, 12, 33, 50, 3, 45, 34, 48, 30, 12, 25, 38, 33, 39, 39, 29, 19, 
		      49, 3, 3, 11, 18, 10, 42, 41, 45, 24, 7, 40, 26, 11, 3, 1, 49, 21, 31, 10, 
		      49, 33, 33, 20, 41, 17, 31, 2, 22, 25, 31, 49, 40, 5, 21, 28, 43, 9, 30, 14, 
		      43, 4, 4, 13, 29, 27, 50, 23, 30, 32, 41, 7, 19, 14, 5, 31, 46, 24, 33, 41, 
		      19, 20, 33, 33, 48, 21, 5, 29, 15, 5, 50, 11, 42, 10, 4, 37, 15, 42, 24, 15, 
		      19, 23, 38, 36, 37, 34, 11, 40, 15, 27, 16, 45, 40, 33, 44, 39, 4, 26, 12, 34, 
		      4, 17, 39, 13, 3, 29, 24, 32, 30, 25, 42, 25, 42, 41, 27, 28, 15, 42, 37, 22, 
		      35, 29, 6, 9, 27, 28, 48, 33, 39, 35, 36, 42, 40, 19, 29, 14, 45, 44, 30, 38, 
		      28, 27, 41, 30, 39, 13, 7, 50, 44, 35, 5, 14, 37, 31, 50, 31, 19, 22, 14, 45, 
		      30, 45, 2, 9, 41, 17, 36, 19, 33, 47, 31, 49, 7, 36, 4, 21, 46, 45, 41, 22, 
		      42, 12, 44, 5, 2, 36, 13, 42, 46, 49, 22, 50, 1, 2, 50, 21, 2, 14, 40, 18, 
		      30, 36, 45, 3, 43, 13, 6, 35, 36, 13, 41, 2, 15, 42, 4, 8, 50, 45, 4, 10, 
		      29, 24, 10, 37, 38, 18, 48, 41, 24, 13, 23, 16, 12, 15, 25, 50, 38, 39, 45, 23, 
		      43, 9, 37, 48, 30, 27, 3, 42, 50, 23, 24, 25, 10, 28, 29, 49, 50, 14, 47, 50, 
		      48, 7, 45, 27, 47, 31, 9, 15, 5, 44, 8, 16, 41, 36, 50, 46, 48, 16, 16, 1, 
		      15, 3, 49, 7, 31, 44, 32, 17, 50, 8, 8, 4, 27, 26, 38, 19, 38, 8, 16, 2, 
		      31, 2, 10, 32, 7, 7, 4, 7, 10, 37, 46, 35, 10, 16, 33, 44, 47, 7, 13, 9, 
		      28, 36, 28, 44, 9, 2, 24, 37, 32, 24, 34, 32, 20, 3, 17, 22, 41, 30, 45, 37, 
		      1, 13, 33, 12, 48, 12, 45, 2, 40, 10, 35, 18, 1, 17, 32, 21, 2, 47, 12, 21, 
		      33, 34, 47, 19, 9, 39, 5, 14, 15, 19, 43, 33, 1, 47, 4, 2, 3, 3, 29, 20, 
		      18, 27, 5, 27, 46, 50, 16, 18, 6, 3, 13, 47, 40, 32, 47, 48, 24, 23, 37, 42, 
		      37, 24, 1, 32, 17, 49, 4, 37, 8, 40, 44, 22, 6, 32, 32, 5, 15, 49, 8, 47, 
		      20, 48, 34, 20, 42, 15, 46, 24, 2, 18, 35, 28, 36, 40, 6, 6, 11, 26, 15, 35, 
		      17, 24, 30, 14, 33, 46, 47, 28, 39, 19, 29, 31, 19, 30, 47, 37, 31, 32, 14, 26, 
		      5, 2, 20, 44, 17, 45, 25, 3, 1, 15, 19, 12, 7, 48, 6, 48, 4, 25, 44, 42, 
		      46, 20, 23, 16, 13, 3, 27, 50, 16, 34, 25, 39, 2, 38, 47, 27, 37, 50, 50, 25, 
		      6, 33, 37, 2, 42, 15, 34, 29, 20, 20, 35, 39, 40, 11, 31, 27, 15, 45, 22, 4, 
		      24, 12, 26, 33, 42, 35, 2, 29, 45, 43, 38, 18, 21, 15, 14, 20, 26, 48, 29, 28, 
		      38, 42, 26, 28, 27, 43, 40, 41, 42, 25, 38, 22, 37, 36, 6, 25, 38, 16, 47, 14, 
		      1, 41, 47, 39, 33, 15, 34, 24, 21, 18, 25, 42, 15, 39, 14, 25, 10, 29, 29, 8, 
		      39, 50, 12, 14, 15, 31, 3, 39, 5, 14, 36, 45, 4, 31, 22, 34, 15, 8, 29, 27, 
		      49, 17, 39, 11, 9, 26, 45, 34, 35, 14, 31, 40, 18, 18, 42, 34, 18, 2, 48, 43, 
		      3, 39, 16, 14, 4, 20, 16, 48, 13, 47, 16, 8, 13, 1, 45, 33, 3, 34, 46, 41, 
		      38, 16, 46, 1, 25, 41, 9, 14, 2, 47, 5, 39, 23, 24, 3, 14, 14, 33, 26, 44, 
		      11, 19, 34, 20, 3, 23, 41, 35, 40, 12, 25, 12, 16, 23, 24, 11, 37, 34, 7, 41, 
		      16, 24, 43, 25, 49, 27, 50, 6, 12, 11, 7, 6, 39, 8, 48, 47, 17, 43, 30, 39, 
		      31, 41, 34, 42, 39, 11, 46, 49, 11, 22, 14, 11, 27, 49, 6, 20, 50, 24, 26, 22, 
		      47, 3, 42, 46, 10, 9, 15, 11, 18, 19, 39, 37, 21, 6, 7, 13, 28, 14, 21, 29, 
		      32, 2, 20, 34, 46, 2, 20, 2, 46, 6, 4, 21, 28, 25, 6, 50, 46, 49, 13, 27, 
		      9, 31, 8, 12, 33, 32, 11, 6, 35, 22, 34, 12, 15, 38, 47, 7, 28, 12, 36, 9, 
		      13, 18, 49, 24, 19, 47, 30, 24, 36, 6, 25, 5, 41, 40, 12, 16, 12, 20, 18, 15, 
		      38, 21, 14, 3, 36, 7, 28, 39, 23, 50, 19, 7, 2, 48, 9, 2, 16, 47, 1, 32, 
		      35, 38, 47, 3, 42, 18, 6, 43, 10, 5, 42, 23, 32, 17, 1, 27, 36, 19, 20, 1, 
		      6, 5, 24, 18, 6, 8, 28, 4, 36, 11, 21, 34, 39, 6, 44, 28, 23, 24, 4, 13, 
		      8, 12, 1, 20, 38, 4, 27, 42, 50, 48, 40, 20, 3, 12, 15, 24, 39, 22, 38, 22, 
		      29, 26, 4, 13, 11, 10, 31, 33, 9, 46, 25, 48, 32, 35, 35, 44, 13, 14, 1, 33, 
		      3, 33, 21, 2, 1, 18, 18, 13, 7, 46, 36, 6, 6, 36, 21, 8, 20, 24, 5, 18, 
		      5, 38, 30, 7, 16, 6, 10, 6, 21, 7, 10, 39, 36, 41, 45, 50, 46, 46, 21, 17, 
		      46, 4, 43, 18, 46, 35, 43, 27, 40, 32, 48, 25, 19, 28, 22, 17, 45, 45, 20, 40, 
		      3, 48, 23, 12, 46, 2, 7, 7, 33, 49, 6, 36, 2, 2, 33, 24, 9, 20, 31, 23, 
		      13, 21, 31, 31, 32, 17, 39, 14, 2, 4, 23, 44, 19, 23, 5, 13, 12, 49, 36, 46, 
		      34, 15, 35, 6, 28, 6, 7, 39, 33, 40, 15, 49, 11, 28, 46, 47, 30, 47, 9, 24, 
		      45, 31, 35, 32, 31, 41, 40, 36, 25, 37, 15, 36, 8, 4, 6, 48, 22, 1, 38, 2, 
		      37, 43, 32, 25, 28, 34, 32, 5, 17, 17, 46, 9, 39, 30, 2, 36, 7, 25, 15, 17, 
		      14, 15, 1, 8, 10, 44, 19, 16, 30, 12, 50, 4, 8, 33, 48, 32, 11, 22, 27, 2, 
		      4, 13, 11, 2, 14, 2, 28, 22, 19, 39, 8, 29, 22, 26, 36, 36, 28, 6, 14, 28, 
		      46, 11, 26, 1, 41, 9, 33, 22, 17, 25, 22, 39, 23, 32, 17, 4, 28, 21, 35, 48, 
		      14, 6, 43, 4, 24, 12, 26, 29, 20, 37, 20, 35, 19, 29, 2, 23, 18, 26, 29, 3, 
		      20, 39, 34, 10, 16, 33, 40, 27, 14, 40, 23, 43, 24, 9, 40, 45, 50, 42, 45, 39, 
		      25, 23, 19, 39, 12, 5, 32, 24, 39, 41, 22, 37, 37, 33, 37, 36, 5, 20, 30, 41, 
		      22, 40, 23, 22, 1, 35, 40, 12, 50, 3, 11, 18, 27, 11, 24, 33, 24, 47, 42, 29, 
		      35, 13, 44, 22, 8, 16, 7, 8, 31, 2, 29, 19, 49, 7, 41, 25, 45, 1, 28, 3, 
		      14, 8, 46, 47, 45, 6, 26, 3, 47, 25, 13, 11, 43, 3, 42, 24, 38, 1, 26, 9, 
		      22, 44, 35, 43, 20, 46, 43, 43, 50, 23, 17, 5, 31, 18, 46, 24, 26, 36, 3, 45, 
		      49, 22, 43, 28, 7, 31, 26, 4, 1, 33, 5, 9, 38, 40, 27, 49, 41, 30, 41, 16, 
		      47, 19, 42, 39, 1, 31, 38, 11, 7, 35, 46, 42, 47, 21, 35, 35, 42, 19, 25, 44, 
		      6, 8, 3, 40, 46, 25, 33, 37, 21, 32, 14, 19, 14, 2, 17, 30, 45, 37, 2, 40, 
		      37, 44, 44, 15, 38, 4, 30, 3, 27, 1, 26, 7, 14, 3, 32, 27, 6, 9, 11, 19, 
		      8, 42, 25, 18, 38, 8, 35, 16, 13, 40, 28, 37, 15, 33, 45, 27, 35, 35, 6, 20, 
		      4, 16, 31, 8, 26, 10, 1, 18, 3, 7, 21, 31, 40, 20, 32, 21, 34, 18, 1, 35, 
		      35, 41, 33, 1, 43, 43, 20, 8, 8, 46, 12, 39, 13, 19, 34, 21, 29, 20, 36, 9, 
		      38, 30, 10, 40, 17, 31, 43, 39, 20, 4, 2, 1, 29, 45, 20, 48, 37, 42, 6, 44, 
		      23, 18, 36, 19, 32, 24, 15, 31, 48, 7, 5, 23, 9, 34, 41, 5, 35, 19, 12, 46, 
		      10, 33, 1, 4, 45, 23, 42, 10, 6, 4, 44, 28, 45, 13, 5, 5, 48, 14, 19, 35, 
		      10, 26, 9, 34, 4, 19, 34, 37, 42, 10, 28, 2, 44, 39, 23, 4, 19, 2, 10, 9, 
		      45, 8, 6, 22, 33, 44, 3, 3, 40, 38, 39, 42, 23, 4, 32, 36, 32, 42, 47, 17, 
		      2, 3, 48, 26, 46, 47, 24, 46, 17, 10, 7, 19, 35, 49, 17, 47, 6, 6, 40, 49, 
		      37, 19, 34, 6, 8, 39, 7, 23, 23, 32, 25, 7, 37, 44, 27, 22, 27, 21, 43, 23, 
		      28, 45, 44, 18, 17, 30, 4, 18, 41, 39, 30, 13, 40, 49, 10, 21, 48, 1, 36, 1, 
		      6, 11, 34, 21, 24, 40, 31, 15, 48, 19, 32, 4, 7, 3, 24, 21, 21, 9, 3, 9, 
		      47, 2, 18, 9, 22, 2, 29, 43, 42, 39, 33, 14, 14, 26, 16, 42, 20, 44, 7, 40, 
		      41, 14, 34, 12, 15, 28, 42, 4, 8, 26, 38, 22, 24, 32, 27, 27, 38, 26, 27, 31, 
		      26, 37, 17, 6, 15, 29, 29, 45, 40, 43, 31, 30, 39, 18, 26, 22, 5, 23, 6, 9, 
		      34, 43, 7, 30, 45, 8, 36, 32, 13, 10, 25, 5, 42, 25, 28, 43, 11, 28, 5, 17, 
		      21, 14, 16, 24, 5, 38, 13, 3, 11, 29, 28, 41, 19, 5, 41, 20, 24, 7, 47, 33, 
		      44, 24, 33, 27, 35, 47, 45, 24, 33, 23, 45, 35, 26, 28, 45, 7, 33, 21, 43, 30, 
		      24, 22, 49, 45, 33, 19, 20, 21, 1, 15, 20, 11, 28, 31, 6, 8, 43, 26, 27, 46, 
		      34, 22, 34, 37, 43, 24, 12, 31, 40, 32, 17, 44, 34, 20, 29, 24, 46, 4, 15, 20, 
		      23, 22, 1, 24, 49, 41, 42, 21, 13, 34, 50, 16, 36, 44, 42, 8, 30, 36, 49, 17, 
		      28, 32, 19, 33, 47, 10, 14, 43, 36, 5, 28, 22, 39, 31, 23, 25, 49, 10, 23, 2, 
		      9, 26, 26, 15, 29, 44, 42, 49, 35, 35, 47, 30, 44, 24, 48, 1, 23, 43, 32, 10, 
		      25, 37, 19, 43, 34, 45, 39, 47, 7, 50, 16, 6, 2, 4, 16, 50, 15, 27, 15, 20, 
		      32, 21, 21, 48, 5, 11, 14, 19, 41, 38, 9, 12, 7, 29, 26, 47, 34, 19, 38, 5, 
		      42, 31, 2, 49, 1, 10, 17, 24, 16, 40, 23, 14, 23, 40, 21, 40, 22, 17, 32, 15, 
		      29, 9, 9, 14, 7, 10, 19, 23, 49, 43, 39, 19, 36, 36, 17, 2, 9, 5, 50, 29, 
		      11, 48, 23, 48, 6, 43, 26, 49, 41, 17, 43, 32, 29, 50, 50, 15, 47, 43, 4, 42, 
		      35, 40, 17, 50, 29, 16, 27, 25, 26, 11, 26, 13, 2, 39, 44, 45, 13, 20, 46, 29, 
		      31, 38, 27, 36, 20, 47, 46, 25, 26, 43, 2, 19, 50, 41, 12, 5, 27, 45, 10, 5, 
		      26, 45, 2, 28, 5, 10, 3, 28, 3, 30, 4, 35, 38, 11, 32, 46, 28, 41, 19, 37, 
		      11, 6, 37, 1, 17, 24, 11, 12, 38, 27, 25, 28, 31, 24, 22, 37, 10, 37, 38, 4, 
		      45, 6, 21, 24, 39, 49, 12, 2, 48, 24, 41, 32, 47, 9, 48, 49, 41, 41, 20, 7, 
		      19, 50, 35, 6, 10, 4, 1, 7, 28, 27, 30, 12, 37, 1, 33, 2, 10, 23, 24, 14, 
		      47, 33, 46, 18, 24, 29, 47, 8, 32, 44, 13, 42, 3, 42, 20, 47, 23, 37, 35, 22, 
		      7, 44, 43, 12, 22, 13, 18, 37, 45, 17, 26, 44, 31, 27, 17, 11, 15, 32, 21, 20, 
		      30, 6, 5, 15, 33, 27, 42, 17, 37, 23, 9, 19, 26, 29, 46, 36, 44, 49, 45, 13, 
		      36, 31, 2, 22, 48, 12, 49, 46, 17, 28, 16, 22, 34, 47, 22, 2, 30, 32, 48, 1, 
		      45, 18, 34, 44, 13, 39, 14, 26, 41, 24, 39, 4, 9, 6, 21, 46, 47, 2, 36, 28, 
		      21, 49, 18, 35, 33, 48, 29, 26, 20, 43, 33, 12, 2, 46, 27, 43, 44, 29, 2, 47, 
		      42, 40, 13, 37, 21, 5, 30, 10, 25, 24, 33, 17, 26, 43, 21, 43, 3, 16, 14, 29, 
		      33, 10, 32, 35, 13, 16, 23, 9, 9, 49, 49, 23, 40, 2, 23, 28, 3, 17, 33, 23, 
		      36, 11, 39, 50, 33, 38, 33, 8, 37, 43, 29, 18, 25, 7, 24, 31, 43, 33, 27, 24, 
		      17, 47, 11, 50, 18, 21, 39, 5, 26, 9, 48, 50, 32, 40, 9, 2, 45, 23, 28, 27, 
		      50, 22, 31, 25, 18, 36, 21, 24, 22, 28, 48, 3, 12, 26, 18, 45, 11, 48, 28, 48, 
		      10, 5, 9, 4, 18, 2, 39, 23, 31, 1, 29, 34, 23, 43, 35, 39, 7, 35, 12, 44, 
		      27, 24, 30, 29, 29, 6, 14, 7, 12, 45, 18, 28, 4, 18, 34, 44, 39, 27, 21, 29, 
		      4, 11, 45, 24, 14, 6, 21, 50, 42, 38, 44, 4, 9, 24, 9, 1, 13, 47, 14, 17, 
		      19, 36, 15, 6, 4, 49, 50, 37, 28, 34, 31, 8, 15, 16, 30, 38, 7, 12, 46, 15, 
		      26, 26, 11, 33, 44, 29, 14, 11, 29, 27, 7, 8, 24, 2, 4, 30, 3, 49, 27, 25, 
		      42, 36, 33, 31, 3, 9, 47, 18, 36, 16, 7, 34, 31, 43, 23, 47, 30, 37, 49, 41, 
		      28, 23, 12, 15, 41, 49, 1, 42, 24, 40, 18, 48, 4, 24, 13, 33, 19, 47, 16, 17, 
		      40, 17, 24, 8, 41, 48, 44, 26, 13, 18, 21, 40, 13, 1, 35, 50, 31, 20, 42, 38, 
		      30, 1, 15, 31, 34, 1, 34, 48, 3, 40, 3, 37, 43, 5, 7, 43, 6, 26, 49, 18, 
		      50, 17, 41, 11, 8, 18, 37, 5, 10, 36, 21, 36, 26, 24, 3, 20, 42, 49, 20, 8, 
		      36, 23, 43, 12, 21, 39, 49, 46, 22, 44, 5, 9, 6, 35, 31, 42, 20, 15, 42, 30, 
		      12, 24, 15, 40, 28, 12, 33, 34, 48, 47, 5, 1, 41, 33, 20, 25, 37, 45, 15, 16, 
		      27, 13, 23, 23, 44, 41, 45, 41, 15, 37, 7, 39, 27, 27, 8, 22, 35, 19, 49, 23, 
		      20, 16, 21, 47, 14, 21, 11, 27, 20, 37, 34, 17, 21, 41, 28, 32, 33, 12, 1, 7, 
		      32, 39, 39, 45, 45, 5, 1, 49, 12, 22, 23, 32, 47, 20, 43, 21, 50, 14, 1, 23, 
		      5, 5, 5, 11, 49, 2, 7, 42, 14, 46, 28, 18, 10, 19, 21, 18, 32, 11, 25, 16, 
		      20, 33, 50, 23, 20, 42, 26, 9, 3, 16, 6, 25, 21, 34, 4, 14, 10, 31, 19, 45, 
		      32, 10, 17, 50, 50, 35, 31, 18, 5, 11, 39, 38, 22, 4, 7, 9, 45, 17, 1, 47, 
		      41, 10, 15, 15, 40, 35, 40, 13, 35, 48, 37, 4, 16, 13, 45, 7, 1, 49, 24, 15, 
		      40, 29, 24, 46, 23, 5, 7, 39, 26, 44, 9, 50, 27, 45, 49, 26, 20, 9, 37, 6, 
		      29, 4, 2, 36, 34, 8, 49, 44, 20, 14, 9, 19, 34, 38, 17, 31, 40, 8, 29, 1, 
		      24, 42, 45, 28, 4, 24, 26, 35, 18, 20, 42, 6, 40, 4, 4, 42, 46, 21, 28, 38, 
		      9, 26, 30, 9, 40, 23, 1, 7, 7, 46, 40, 20, 30, 48, 42, 33, 4, 32, 8, 21, 
		      37, 4, 21, 12, 44, 5, 29, 33, 29, 14, 32, 11, 29, 23, 2, 13, 18, 42, 14, 10, 
		      10, 14, 44, 29, 24, 8, 14, 47, 12, 10, 39, 39, 47, 39, 31, 3, 50, 34, 37, 46, 
		      48, 14, 46, 1, 37, 1, 1, 38, 2, 8, 29, 21, 10, 5, 21, 32, 45, 2, 39, 5, 
		      46, 22, 24, 1, 44, 44, 19, 42, 25, 42, 48, 18, 4, 37, 50, 26, 8, 27, 46, 10, 
		      30, 15, 5, 29, 35, 45, 40, 41, 42, 20, 1, 3, 49, 18, 4, 36, 23, 44, 18, 50, 
		      39, 25, 11, 39, 24, 24, 22, 7, 15, 20, 50, 4, 1, 46, 44, 10, 20, 42, 17, 7, 
		      41, 13, 19, 39, 1, 34, 2, 44, 1, 17, 48, 50, 27, 19, 36, 13, 33, 19, 34, 29, 
		      6, 20, 8, 38, 32, 15, 34, 50, 19, 3, 1, 21, 43, 6, 41, 1, 37, 50, 37, 35, 
		      47, 47, 23, 25, 31, 27, 7, 9, 12, 13, 15, 11, 41, 36, 48, 4, 2, 40, 44, 34, 
		      4, 10, 43, 9, 34, 30, 23, 27, 31, 22, 3, 16, 9, 20, 19, 47, 43, 18, 49, 11, 
		      41, 29, 28, 13, 15, 31, 29, 18, 41, 13, 47, 30, 40, 7, 2, 3, 32, 17, 16, 14, 
		      41, 30, 4, 3, 8, 20, 40, 17, 38, 45, 40, 8, 28, 24, 8, 50, 1, 27, 45, 8, 
		      7, 36, 6, 39, 3, 6, 32, 4, 41, 9, 50, 2, 27, 48, 16, 38, 21, 16, 46, 30, 
		      10, 4, 22, 35, 23, 25, 10, 5, 20, 9, 29, 9, 9, 9, 33, 34, 19, 47, 3, 10, 
		      45, 40, 17, 2, 19, 31, 7, 32, 31, 6, 28, 4, 44, 46, 50, 9, 17, 45, 21, 35, 
		      40, 36, 9, 11, 17, 40, 9, 22, 44, 31, 10, 2, 3, 24, 3, 7, 10, 22, 27, 9, 
		      20, 49, 5, 24, 37, 38, 48, 4, 26, 22, 32, 28, 49, 11, 19, 10, 16, 47, 34, 45, 
		      16, 3, 11, 39, 44, 19, 23, 41, 2, 24, 1, 26, 27, 50, 6, 4, 13, 29, 2, 30, 
		      15, 38, 24, 15, 1, 9, 16, 48, 1, 43, 26, 1, 5, 12, 47, 5, 8, 49, 48, 39, 
		      28, 50, 8, 48, 2, 33, 24, 27, 35, 5, 6, 6, 47, 24, 15, 15, 45, 4, 34, 29, 
		      18, 4, 14, 1, 25, 21, 4, 24, 31, 45, 42, 21, 24, 29, 3, 45, 18, 29, 14, 31, 
		      6, 8, 50, 19, 31, 28, 19, 50, 23, 12, 16, 10, 35, 20, 22, 4, 49, 40, 2, 20, 
		      39, 2, 36, 44, 25, 46, 21, 46, 34, 38, 12, 34, 22, 46, 8, 44, 41, 26, 16, 37, 
		      47, 32, 20, 15, 20, 36, 43, 33, 40, 10, 17, 1, 22, 1, 13, 49, 18, 35, 31, 13, 
		      17, 10, 37, 14, 12, 8, 16, 47, 34, 2, 36, 45, 4, 49, 4, 35, 46, 24, 16, 19, 
		      23, 15, 47, 33, 27, 48, 34, 35, 14, 8, 24, 25, 11, 4, 23, 10, 30, 20, 6, 38, 
		      41, 13, 49, 42, 33, 41, 39, 45, 50, 39, 37, 26, 34, 40, 43, 42, 31, 26, 20, 25, 
		      28, 33, 44, 35, 21, 9, 38, 36, 36, 19, 27, 22, 5, 27, 32, 46, 28, 44, 1, 5, 
		      23, 9, 14, 5, 10, 9, 45, 50, 21, 18, 26, 16, 25, 2, 4, 23, 36, 4, 20, 26, 
		      28, 7, 16, 39, 42, 41, 24, 16, 24, 22, 35, 32, 2, 18, 21, 31, 46, 16, 38, 46, 
		      21, 39, 22, 17, 29, 4, 50, 37, 22, 17, 10, 26, 7, 22, 17, 8, 2, 26, 18, 5, 
		      49, 43, 5, 6, 42, 6, 40, 14, 2, 39, 44, 41, 16, 39, 17, 20, 49, 18, 7, 25, 
		      4, 31, 50, 21, 7, 43, 45, 8, 48, 10, 45, 44, 8, 13, 12, 44, 41, 27, 45, 49, 
		      29, 50, 1, 48, 18, 22, 39, 10, 11, 31, 24, 4, 20, 38, 30, 18, 45, 14, 43, 11, 
		      38, 18, 33, 47, 42, 18, 49, 6, 48, 15, 40, 32, 34, 1, 28, 9, 46, 27, 7, 10, 
		      13, 24, 45, 19, 7, 38, 42, 7, 34, 50, 42, 7, 27, 18, 21, 40, 16, 14, 25, 28, 
		      17, 12, 20, 24, 21, 9, 10, 36, 31, 25, 10, 13, 29, 16, 39, 42, 12, 40, 40, 25, 
		      36, 9, 10, 24, 18, 18, 39, 10, 19, 40, 23, 33, 2, 6, 47, 26, 26, 19, 38, 10, 
		      41, 40, 26, 44, 49, 47, 3, 10, 42, 48, 7, 3, 5, 47, 5, 1, 37, 14, 45, 38, 
		      16, 15, 33, 10, 45, 23, 6, 49, 17, 14, 13, 33, 38, 21, 4, 25, 6, 7, 15, 48, 
		      31, 16, 24, 18, 31, 42, 13, 41, 48, 2, 29, 40, 47, 15, 10, 32, 46, 38, 6, 4, 
		      44, 5, 50, 44, 44, 47, 27, 49, 41, 7, 20, 44, 43, 21, 4, 22, 24, 5, 49, 27, 
		      5, 24, 26, 49, 25, 35, 16, 42, 44, 35, 26, 30, 6, 12, 50, 40, 41, 32, 47, 21, 
		      11, 46, 17, 43, 5, 13, 14, 28, 46, 43, 35, 21, 40, 38, 42, 2, 42, 27, 31, 20, 
		      18, 36, 9, 43, 16, 34, 21, 3, 40, 10, 47, 8, 26, 18, 22, 1, 13, 36, 24, 25, 
		      36, 28, 41, 18, 27, 26, 40, 22, 18, 27, 1, 23, 47, 34, 35, 23, 28, 2, 19, 43, 
		      22, 20, 18, 46, 38, 21, 31, 13, 26, 17, 44, 49, 29, 13, 13, 32, 46, 18, 45, 2, 
		      12, 2, 5, 11, 45, 35, 3, 23, 19, 27, 37, 35, 2, 24, 28, 16, 7, 49, 22, 42, 
		      29, 36, 27, 33, 25, 42, 47, 45, 21, 5, 45, 30, 18, 42, 6, 32, 8, 13, 47, 20, 
		      34, 28, 14, 32, 19, 28, 45, 20, 9, 40, 43, 25, 33, 4, 25, 12, 43, 24, 16, 38, 
		      48, 19, 27, 24, 28, 23, 24, 4, 33, 27, 48, 28, 23, 1, 39, 18, 42, 23, 1, 35, 
		      34, 2, 40, 29, 2, 33, 48, 8, 29, 43, 37, 23, 5, 5, 50, 4, 34, 7, 41, 12, 
		      40, 34, 36, 11, 11, 32, 26, 33, 11, 46, 21, 9, 25, 4, 32, 40, 5, 4, 31, 19, 
		      3, 27, 34, 17, 5, 9, 3, 25, 40, 20, 33, 31, 32, 34, 44, 7, 46, 30, 44, 47, 
		      33, 48, 31, 31, 45, 42, 18, 45, 5, 30, 12, 41, 6, 13, 6, 21, 23, 2, 5, 43, 
		      19, 15, 25, 32, 50, 48, 40, 48, 27, 23, 2, 50, 21, 35, 37, 6, 46, 10, 28, 50, 
		      18, 24, 49, 38, 35, 17, 27, 29, 22, 29, 35, 48, 21, 13, 31, 13, 9, 33, 21, 47, 
		      26, 15, 46, 37, 36, 25, 29, 50, 16, 6, 14, 38, 10, 1, 17, 3, 28, 46, 9, 47, 
		      47, 38, 3, 27, 38, 29, 14, 1, 44, 31, 31, 25, 26, 5, 19, 21, 23, 13, 15, 29, 
		      20, 46, 31, 20, 15, 41, 35, 11, 40, 21, 44, 49, 26, 33, 9, 43, 14, 23, 29, 41, 
		      27, 48, 39, 11, 6, 23, 16, 31, 46, 12, 20, 30, 27, 6, 3, 21, 40, 35, 21, 14, 
		      22, 37, 7, 33, 32, 15, 36, 47, 3, 45, 42, 50, 45, 30, 48, 25, 46, 20, 18, 39, 
		      20, 2, 15, 23, 31, 19, 41, 2, 38, 44, 37, 29, 27, 39, 45, 17, 6, 36, 31, 3, 
		      42, 42, 23, 9, 1, 4, 3, 39, 25, 33, 17, 29, 44, 1, 26, 6, 45, 11, 36, 34, 
		      28, 41, 50, 16, 35, 32, 21, 40, 38, 2, 25, 47, 9, 21, 24, 21, 23, 25, 42, 21, 
		      43, 50, 9, 34, 27, 42, 1, 34, 8, 32, 5, 46, 21, 8, 36, 8, 50, 4, 43, 46, 
		      14, 32, 33, 10, 40, 18, 26, 8, 1, 41, 37, 6, 29, 48, 42, 24, 17, 33, 19, 39, 
		      5, 30, 16, 26, 48, 23, 11, 16, 8, 42, 6, 26, 14, 12, 3, 26, 46, 20, 22, 22, 
		      29, 1, 31, 48, 31, 36, 26, 11, 23, 13, 12, 37, 48, 25, 18, 44, 48, 8, 39, 45, 
		      17, 1, 8, 45, 31, 19, 38, 48, 34, 1, 18, 45, 26, 44, 21, 50, 38, 17, 49, 1, 
		      39, 36, 21, 8, 23, 14, 39, 38, 1, 4, 25, 37, 7, 21, 35, 21, 43, 28, 17, 7, 
		      17, 7, 23, 50, 9, 29, 24, 30, 34, 7, 3, 14, 11, 21, 12, 34, 50, 35, 49, 14, 
		      24, 16, 39, 19, 6, 49, 24, 49, 45, 40, 43, 24, 46, 2, 5, 10, 44, 37, 49, 7, 
		      33, 21, 19, 2, 10, 10, 33, 3, 5, 14, 40, 8, 14, 28, 16, 32, 28, 3, 39, 19, 
		      50, 2, 39, 16, 17, 19, 33, 12, 11, 20, 49, 26, 48, 20, 10, 39, 42, 50, 32, 11, 
		      9, 13, 17, 17, 3, 46, 42, 14, 43, 46, 25, 24, 37, 35, 15, 4, 6, 16, 40, 16, 
		      32, 29, 33, 8, 46, 22, 23, 49, 16, 36, 12, 45, 27, 13, 9, 36, 6, 9, 16, 14, 
		      1, 2, 23, 30, 25, 14, 41, 18, 28, 41, 24, 24, 20, 28, 1, 32, 21, 18, 2, 18, 
		      2, 20, 44, 5, 29, 4, 38, 23, 36, 19, 48, 6, 44, 28, 26, 19, 38, 4, 39, 7, 
		      24, 3, 10, 44, 42, 39, 39, 45, 21, 20, 4, 33, 44, 41, 24, 9, 25, 4, 14, 8, 
		      11, 5, 31, 42, 18, 30, 30, 49, 4, 41, 30, 31, 24, 47, 7, 12, 48, 28, 46, 5, 
		      36, 44, 30, 12, 12, 12, 34, 38, 44, 40, 30, 43, 25, 40, 2, 3, 22, 37, 32, 41, 
		      40, 33, 45, 23, 31, 38, 29, 8, 20, 17, 15, 45, 35, 33, 13, 13, 14, 32, 48, 12, 
		      49, 49, 40, 36, 17, 1, 21, 29, 16, 18, 27, 30, 18, 42, 50, 32, 41, 1, 4, 33, 
		      33, 45, 45, 35, 19, 21, 14, 19, 39, 6, 42, 10, 48, 50, 33, 28, 45, 32, 2, 10, 
		      3, 5, 46, 40, 25, 39, 32, 27, 27, 18, 2, 26, 41, 41, 3, 1, 27, 26, 23, 27, 
		      18, 36, 49, 15, 28, 34, 39, 10, 18, 48, 43, 2, 29, 9, 49, 29, 48, 1, 44, 42, 
		      8, 27, 45, 48, 14, 8, 31, 46, 49, 42, 30, 40, 16, 26, 17, 9, 48, 22, 38, 23, 
		      25, 40, 24, 47, 5, 48, 1, 4, 4, 16, 16, 22, 41, 34, 44, 4, 11, 17, 42, 29, 
		      47, 10, 40, 22, 32, 5, 12, 37, 48, 46, 43, 11, 25, 23, 1, 30, 44, 7, 9, 7, 
		      48, 39, 25, 17, 40, 28, 15, 24, 2, 5, 9, 47, 35, 30, 10, 11, 38, 7, 28, 27, 
		      19, 40, 36, 42, 40, 40, 7, 31, 28, 23, 45, 21, 20, 3, 31, 30, 13, 4, 17, 40, 
		      20, 44, 3, 24, 11, 7, 34, 36, 28, 33, 38, 28, 41, 13, 37, 41, 45, 49, 16, 16, 
		      28, 41, 19, 46, 27, 31, 17, 50, 42, 36, 26, 28, 23, 5, 12, 1, 5, 47, 48, 48, 
		      1, 12, 49, 46, 27, 24, 19, 45, 23, 20, 50, 26, 13, 50, 46, 48, 11, 8, 2, 6, 
		      21, 21, 26, 30, 22, 47, 20, 38, 23, 20, 32, 1, 5, 40, 3, 10, 40, 47, 2, 8, 
		      14, 23, 5, 50, 11, 42, 19, 25, 48, 43, 3, 40, 33, 9, 40, 3, 26, 48, 15, 7, 
		      9, 31, 31, 3, 19, 25, 38, 28, 14, 34, 2, 38, 28, 14, 22, 50, 6, 43, 6, 48, 
		      40, 1, 23, 10, 41, 12, 27, 44, 12, 15, 48, 13, 5, 16, 25, 7, 37, 9, 30, 26, 
		      45, 14, 8, 29, 1, 16, 21, 19, 38, 29, 26, 14, 33, 24, 36, 17, 22, 48, 15, 43, 
		      44, 36, 27, 42, 28, 32, 20, 33, 48, 45, 3, 26, 42, 45, 6, 28, 18, 3, 36, 43, 
		      39, 4, 35, 34, 31, 14, 50, 43, 4, 9, 2, 12, 30, 5, 20, 36, 45, 30, 32, 23, 
		      21, 47, 22, 6, 47, 3, 35, 45, 23, 23, 10, 39, 25, 16, 42, 24, 35, 16, 14, 37, 
		      4, 24, 2, 37, 26, 46, 29, 13, 10, 20, 49, 29, 22, 1, 3, 3, 7, 21, 46, 47, 
		      5, 28, 36, 6, 19, 34, 43, 33, 26, 6, 8, 43, 19, 33, 43, 19, 20, 8, 29, 44, 
		      5, 20, 32, 37, 49, 47, 6, 40, 24, 15, 39, 47, 15, 37, 22, 45, 36, 18, 24, 26, 
		      24, 27, 19, 35, 15, 19, 18, 22, 26, 3, 32, 31, 46, 49, 40, 15, 49, 22, 38, 9 
  };


  
  if( speedcounter++ % speed == 0 ) {
    SDL_LockSurface(originalSurface);
    SDL_LockSurface(renderSurface);


    if (renderSurface->flags & SDL_SRCCOLORKEY) {
      /* Colour key set. */
      for( x = 0; x < originalSurface->w; x++ ) {
	for( y = 0; y < originalSurface->h; y++ ) {
	  srcPixel=getpixel16(originalSurface, x+offsetX, y+offsetY); 

	  if(srcPixel == renderSurface->format->colorkey) {
	    /* Matches colour key, don't bother about this job. */
	    newPixel = srcPixel;
	  } else {
	    SDL_GetRGB( srcPixel, renderSurface->format, &r, &g, &b );
	    rnd = rand[rand_tbl_counter++];
	    if( r+rnd < 255 ) r += rnd; else r -= rnd;
	  
	    rnd = rand[rand_tbl_counter++];
	    if( g+rnd < 255 ) g += rnd; else g -= rnd;
	  
	    rnd = rand[rand_tbl_counter++];
	    if( b+rnd < 255 ) b += rnd; else b -= rnd;
	  
	    newPixel = SDL_MapRGB( renderSurface->format, r, g, b );
	  }

	  putpixel16(renderSurface, x+offsetX, y+offsetY, newPixel);
	  
	  if( rand_tbl_counter > 9997 )
	    rand_tbl_counter = 0;
	}
      }
    } else {
      /* Don't worry about colour key. */
      for( x = 0; x < originalSurface->w; x++ ) {
	for( y = 0; y < originalSurface->h; y++ ) {
	  srcPixel=getpixel16(originalSurface, x+offsetX, y+offsetY); 
	  
	  SDL_GetRGB( srcPixel, renderSurface->format, &r, &g, &b );
	  rnd = rand[rand_tbl_counter++];
	  if( r+rnd < 255 ) r += rnd; else r -= rnd;
	  
	  rnd = rand[rand_tbl_counter++];
	  if( g+rnd < 255 ) g += rnd; else g -= rnd;
	  
	  rnd = rand[rand_tbl_counter++];
	  if( b+rnd < 255 ) b += rnd; else b -= rnd;
	  
	  newPixel = SDL_MapRGB( renderSurface->format, r, g, b );
	  
	  putpixel16(renderSurface, x+offsetX, y+offsetY, newPixel);
	  
	  if( rand_tbl_counter > 9997 )
	    rand_tbl_counter = 0;
	}
      }
    }
    SDL_UnlockSurface(originalSurface);
    SDL_UnlockSurface(renderSurface);
  }

  /* Don't let speedcounter wrap around, even though it would probably be 
     safe */
  if( speedcounter > 100000)
    speedcounter = 0;
  
  return;
}
