/** Implements a very basic noise filter. 
 *
 * Authors: Kristoffer Lawson, setok@fishpool.com
 *
 * $Header: /home/sleber/cvs/liboil/compColorFilter.c,v 1.2 2003/01/08 16:04:55 sleber Exp $
 */


#include <SDL/SDL.h>
#include <tcl.h>
#include "filters.h"
#include "oil.h"
#include "debug.h"

/* Function prototypes. */

static Uint32 getpixel16(SDL_Surface *surface, int x, int y);
static void putpixel16(SDL_Surface *surface, int x, int y, Uint32 colour);
SDL_Surface *compColorFilterCreate(SDL_Surface *originalSurface, 
			       Tcl_HashTable *options,
			       void **filterData);
static void compColorFilterRender(SDL_Surface *originalSurface, 
			    SDL_Surface *renderSurface,
			    int offsetX, int offsetY, 
			    void *filterData);


static Uint32 getpixel16(SDL_Surface *surface, int x, int y) {
  Uint8 *p;
  
  int bpp = surface->format->BytesPerPixel;
  p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

  return *(Uint16 *)p;   
}


static void putpixel16(SDL_Surface *surface, int x, int y, Uint32 colour) {
  Uint16 *bufp;

  bufp = (Uint16 *)surface->pixels + y*surface->pitch/2 + x;
  *bufp = colour;
}


int compColorFilterInit() {
  Oil_AddFilter("compColor", compColorFilterCreate, compColorFilterRender);
  return TCL_OK;
}


SDL_Surface *compColorFilterCreate(SDL_Surface *originalSurface, 
			       Tcl_HashTable *options,
			       void **filterData) {
  return Oil_CreateDisplaySurface16(originalSurface->w, originalSurface->h);
}


static void compColorFilterRender(SDL_Surface *originalSurface, 
			    SDL_Surface *renderSurface, int offsetX, int offsetY,
			    void *filterData) {

  int x, y, rnd;
  static int speedcounter = 0;
  static int speed = 5;         /* Effect speed; 1 = fastest */
  static int rand_tbl_counter = 0;
  Uint32 srcPixel, newPixel;
  Uint8 r, g, b;
  //  SDL_Rect dstrect;
  
  int rand[1000] = { 2, 5, 1, 7, 9, 7, 7, 6, 10, 4, 4, 5, 4, 10, 1, 8, 8, 1, 4, 4, 
		      7, 7, 7, 4, 10, 7, 10, 3, 2, 6, 5, 1, 5, 9, 8, 1, 2, 6, 9, 2, 
		      3, 5, 4, 1, 1, 8, 10, 3, 3, 3, 4, 3, 3, 7, 5, 3, 3, 10, 2, 5, 
		      10, 3, 1, 7, 10, 1, 5, 7, 3, 8, 3, 2, 1, 1, 2, 6, 8, 10, 3, 5, 
		      2, 5, 5, 5, 9, 10, 7, 3, 2, 3, 10, 3, 8, 6, 5, 6, 3, 4, 1, 9, 
		      1, 3, 1, 1, 9, 7, 2, 5, 3, 4, 8, 9, 4, 8, 4, 3, 9, 2, 9, 6, 
		      9, 7, 1, 1, 7, 1, 8, 1, 5, 4, 4, 6, 10, 1, 1, 4, 3, 2, 10, 6, 
		      10, 8, 9, 4, 8, 5, 1, 7, 1, 2, 2, 7, 6, 7, 5, 5, 10, 5, 8, 4, 
		      3, 1, 6, 9, 7, 1, 3, 4, 3, 5, 8, 7, 4, 10, 9, 8, 8, 7, 3, 7, 
		      2, 4, 7, 9, 10, 5, 3, 9, 1, 9, 4, 10, 1, 3, 8, 5, 5, 6, 9, 1, 
		      4, 3, 10, 2, 6, 5, 2, 5, 9, 9, 10, 2, 3, 5, 10, 10, 5, 7, 10, 8, 
		      1, 3, 4, 2, 6, 4, 10, 8, 3, 10, 10, 10, 1, 4, 9, 3, 2, 3, 8, 3, 
		      3, 3, 3, 1, 10, 3, 8, 6, 10, 1, 2, 10, 3, 3, 1, 5, 1, 3, 6, 1, 
		      9, 3, 10, 7, 8, 5, 1, 3, 4, 3, 3, 7, 8, 1, 10, 8, 1, 9, 7, 1, 
		      10, 10, 7, 4, 2, 9, 6, 1, 10, 8, 4, 1, 2, 6, 8, 7, 7, 7, 5, 3, 
		      9, 6, 9, 2, 3, 9, 3, 6, 5, 4, 2, 5, 8, 10, 6, 9, 1, 8, 2, 7, 
		      8, 10, 7, 10, 10, 3, 5, 5, 7, 1, 3, 5, 9, 7, 2, 2, 6, 4, 8, 2, 
		      2, 1, 3, 1, 5, 5, 2, 9, 8, 9, 8, 1, 10, 5, 4, 9, 6, 7, 9, 8, 
		      4, 2, 5, 1, 9, 1, 8, 8, 2, 6, 9, 6, 1, 7, 10, 7, 4, 6, 3, 9, 
		      7, 6, 8, 1, 5, 10, 9, 3, 7, 3, 1, 3, 2, 1, 6, 5, 6, 4, 5, 5, 
		      10, 3, 4, 4, 9, 4, 1, 6, 9, 2, 8, 8, 8, 9, 1, 8, 5, 4, 8, 5, 
		      3, 4, 7, 2, 8, 1, 5, 10, 8, 6, 1, 10, 1, 3, 5, 4, 7, 1, 1, 2, 
		      7, 10, 4, 2, 8, 7, 7, 3, 8, 3, 8, 7, 6, 7, 10, 9, 2, 7, 10, 2, 
		      9, 6, 7, 1, 5, 3, 1, 1, 6, 10, 9, 1, 10, 4, 8, 2, 3, 5, 7, 5, 
		      5, 10, 8, 1, 2, 5, 1, 4, 8, 3, 9, 1, 8, 1, 3, 10, 3, 9, 10, 1, 
		      7, 2, 6, 9, 8, 9, 3, 6, 1, 3, 1, 4, 4, 9, 2, 5, 9, 9, 2, 6, 
		      10, 5, 10, 4, 7, 5, 5, 10, 9, 8, 2, 2, 3, 3, 9, 8, 5, 3, 9, 10, 
		      3, 9, 5, 1, 7, 6, 8, 8, 7, 4, 7, 7, 5, 3, 8, 7, 5, 7, 8, 2, 
		      9, 4, 5, 5, 10, 9, 9, 5, 10, 8, 10, 9, 1, 10, 2, 1, 9, 10, 4, 10, 
		      3, 6, 3, 8, 1, 4, 5, 10, 2, 6, 6, 1, 6, 4, 2, 7, 1, 6, 5, 3, 
		      6, 5, 6, 2, 3, 10, 6, 5, 8, 5, 8, 4, 7, 9, 7, 6, 4, 6, 3, 8, 
		      1, 3, 6, 10, 6, 10, 2, 6, 6, 6, 1, 8, 10, 5, 7, 10, 3, 10, 3, 9, 
		      2, 5, 4, 6, 3, 3, 7, 3, 9, 2, 1, 4, 7, 7, 9, 2, 1, 7, 4, 3, 
		      10, 8, 3, 2, 6, 1, 8, 4, 7, 5, 10, 2, 10, 8, 9, 9, 2, 10, 1, 1, 
		      10, 7, 2, 10, 1, 5, 3, 6, 6, 8, 4, 2, 3, 10, 1, 6, 9, 3, 1, 7, 
		      5, 6, 6, 3, 3, 10, 9, 2, 9, 7, 5, 2, 3, 10, 6, 7, 4, 1, 1, 2, 
		      7, 5, 4, 2, 5, 2, 5, 8, 7, 3, 3, 5, 4, 3, 7, 1, 5, 8, 9, 7, 
		      7, 1, 7, 3, 1, 9, 5, 8, 7, 4, 8, 5, 4, 1, 9, 10, 10, 9, 5, 1, 
		      6, 5, 3, 2, 3, 4, 4, 7, 3, 6, 8, 5, 10, 3, 5, 8, 5, 5, 5, 10, 
		      7, 2, 4, 5, 1, 3, 10, 8, 1, 6, 2, 2, 9, 1, 5, 10, 5, 3, 8, 3, 
		      8, 7, 8, 1, 2, 7, 2, 1, 6, 5, 1, 7, 2, 3, 8, 2, 1, 1, 4, 4, 
		      7, 7, 8, 6, 7, 5, 6, 7, 9, 10, 10, 6, 10, 7, 4, 9, 2, 4, 1, 5, 
		      1, 2, 7, 1, 9, 2, 8, 8, 9, 7, 8, 7, 8, 10, 5, 1, 4, 5, 5, 3, 
		      6, 8, 5, 3, 5, 5, 5, 6, 7, 7, 10, 4, 7, 4, 9, 7, 4, 9, 4, 3, 
		      4, 9, 5, 2, 1, 9, 7, 10, 2, 1, 5, 10, 6, 1, 4, 8, 4, 6, 6, 4, 
		      8, 5, 9, 8, 3, 1, 5, 5, 2, 7, 7, 7, 5, 8, 6, 7, 10, 4, 9, 5, 
		      7, 9, 7, 10, 4, 5, 5, 6, 8, 7, 2, 7, 7, 2, 10, 3, 2, 2, 9, 8, 
		      4, 9, 1, 10, 2, 3, 4, 5, 5, 9, 4, 4, 3, 2, 2, 6, 2, 3, 8, 6, 
		      10, 2, 5, 8, 10, 1, 9, 2, 2, 4, 5, 6, 2, 6, 7, 6, 3, 2, 8, 6, 
		      5, 2, 3, 3, 6, 2, 10, 5, 8, 8, 5, 4, 10, 1, 7, 10, 2, 10, 9, 5 };
  



  if( speedcounter++ % speed == 0 ) {
	  SDL_LockSurface(originalSurface);
	  SDL_LockSurface(renderSurface);

	  for( x = 0; x < originalSurface->w; x++ ) {
		  for( y = 0; y < originalSurface->h; y++ ) {
			  srcPixel=getpixel16(originalSurface, x+offsetX, y+offsetY); 

			  SDL_GetRGB( srcPixel, renderSurface->format, &r, &g, &b );
			  rnd = rand[rand_tbl_counter];

			  if (rnd % 2 == 0 ) {
				  r = 255 - r;
				  g = 255 - g;
				  b = 255 - b;
			  }
			  
			  newPixel = SDL_MapRGB( renderSurface->format, r, g, b );
			  putpixel16(renderSurface, x+offsetX, y+offsetY, newPixel);

			  
		  }
	  }

	  ++rand_tbl_counter;
	  if( rand_tbl_counter > 100 )
		  rand_tbl_counter = 0;

	  SDL_UnlockSurface(originalSurface);
	  SDL_UnlockSurface(renderSurface);
  }

  /* Don't let speedcounter wrap around, even though it would probably be safe */
  if( speedcounter > 100000)
	  speedcounter = 0;

  return;
}
