/* Debugging functionality. Contains several methods by which to write
   various levels of debugging output with printf. This means that debugging
   prints can be written everywhere in the code when debugging (at various
   levels) and just left there. By setting the debug level to 0 none of the
   debug info will be set. These will be in operation only if the DEBUG macro
   is defined. 

   Authors: Kristoffer Lawson, setok@fishpool.com
   $Header: /home/sleber/cvs/liboil/debug.c,v 1.1.1.1 2003/01/06 02:09:05 sleber Exp $ */


#include "baseIncludes.h"

#include "debug.h"


/* Global variables */

int db_level=0;  /* Debug level */
FILE *db_fp;  /* Where to print out debug messages? */
// void ourPrintf(char *fmt, ...)=eprintf;

/* Static variables */

static int Initialised=0;  /* Have we gone through initialisation yet? */
#define LEVELAMOUNT 5
static char *levelMarks[] = {
  "###",
  "***",
  "##",
  "**",
  "*",
};


/** Set file for debug output. */

void setDebugFile(FILE *fp) {
  db_fp = fp;
  Initialised = 1;
  return;
}


/* Print out a debug prefix according to debug 'level'. */

void printDbgPrefix(int level) {
  if(!Initialised) {
    db_fp=stderr;
    Initialised = 1;
  }
  fprintf(db_fp, "%s ", levelMarks[level-1]);
  return;
}


/* Set used printf -style function. */

void setPrintf(int newPrintf(char *fmt, ...)) {
  //ourPrintf=newPrintf;
  return;
}


/* Printfs directly to stderr. */

int eprintf(char *fmt, ...) {
  va_list args;
  char buffer[DB_OUTMAX+1];
  extern int Initialised;
  extern FILE *db_fp;

  va_start(args, fmt);
  if (!Initialised) {
    db_fp=stderr;
    Initialised = 1;
  }
  vsprintf(buffer, fmt, args);
  va_end(args);
  return fprintf(db_fp, "%s\n", buffer);
}


/* Same as errprintf but without newline */

int eprintf_nnl(char *fmt, ...) {
  va_list args;
  char buffer[DB_OUTMAX+1];
  extern int Initialised;
  extern FILE *db_fp;

  va_start(args, fmt);
  if (!Initialised) {
    db_fp=stderr;
    Initialised = 1;
  }
  vsprintf(buffer, fmt, args);
  va_end(args);
  return fprintf(db_fp, "%s", buffer);
}
