/** Draws small random boxes on surface.
 *
 * Authors: Kristoffer Lawson, setok@fishpool.com
 *
 * $Header: /home/sleber/cvs/liboil/filters/boxesFilter.c,v 1.1 2003/02/08 01:29:13 setok Exp $
 */


#include <SDL/SDL.h>
#include <tcl.h>
#include "filters.h"
#include "../oil.h"
#include "../debug.h"


/* Function prototypes. */

SDL_Surface *boxesFilterCreate(SDL_Surface *originalSurface, 
			       Tcl_HashTable *options,
			       void **filterData);
static void boxesFilterRender(SDL_Surface *originalSurface, 
			      SDL_Surface *renderSurface,
			      int offsetX, int offsetY,
			      void *filterData);


int boxesFilterInit() {
  Oil_AddFilter("boxes", boxesFilterCreate, boxesFilterRender);
  return TCL_OK;
}


SDL_Surface *boxesFilterCreate(SDL_Surface *originalSurface, 
			       Tcl_HashTable *options,
			       void **filterData) {
  return Oil_CreateDisplaySurface16(originalSurface->w, originalSurface->h);
}


static void boxesFilterRender(SDL_Surface *originalSurface, 
		       SDL_Surface *renderSurface, int offsetX, int offsetY,
		       void *filterData) {
  Uint32 green;
  int boxIdx;
  SDL_Rect dest;

  green=SDL_MapRGB(renderSurface->format, 0xcc, 0xFF, 0xcc);

  SDL_BlitSurface( originalSurface, NULL, renderSurface, NULL );

  for(boxIdx = 0; boxIdx < 100; boxIdx++) {
    dest.x = rand() % originalSurface->w;
    dest.y = rand() % originalSurface->h;
    dest.w = rand() % 30;
    dest.h = rand() % 30;
    SDL_FillRect( renderSurface, &dest, green );
  }

  return;
}
