/** Kaleidoscope: mirrors in two directions.
 *
 * Authors: Kristoffer Lawson, setok@fishpool.com
 *
 * $Header: /home/sleber/cvs/liboil/filters/kaleido.c,v 1.1 2003/02/03 01:59:38 setok Exp $
 */


#include <SDL/SDL.h>
#include <tcl.h>
#include "filters.h"
#include "../oil.h"
#include "../debug.h"


/* Function prototypes. */

static Uint32 getpixel16(SDL_Surface *surface, int x, int y);
static void putpixel16(SDL_Surface *surface, int x, int y, Uint32 colour);
SDL_Surface *kaleidoCreate(SDL_Surface *originalSurface, 
			       Tcl_HashTable *options,
			       void **filterData);
static void kaleidoRender(SDL_Surface *originalSurface, 
				SDL_Surface *renderSurface,
				int offsetX, int offsetY,
				void *filterData);


static Uint32 getpixel16(SDL_Surface *surface, int x, int y) {
  Uint8 *p;
  
  int bpp = surface->format->BytesPerPixel;
  p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

  return *(Uint16 *)p;   
}


static void putpixel16(SDL_Surface *surface, int x, int y, Uint32 colour) {
  Uint16 *bufp;

  bufp = (Uint16 *)surface->pixels + y*surface->pitch/2 + x;
  *bufp = colour;
}


int kaleidoInit() {
  Oil_AddFilter("kaleido", kaleidoCreate, kaleidoRender);
  return TCL_OK;
}


SDL_Surface *kaleidoCreate(SDL_Surface *originalSurface, 
			   Tcl_HashTable *options,
			   void **filterData) {
  return Oil_CreateDisplaySurface16(originalSurface->w * 2, 
				    originalSurface->h * 2);
}


static void kaleidoRender(SDL_Surface *originalSurface, 
			  SDL_Surface *renderSurface, int offsetX, 
			  int offsetY,
			  void *filterData) {
  int x, y;
  SDL_Rect src, dest;

  /* Blit to original position. */
  SDL_BlitSurface(originalSurface, NULL, renderSurface, NULL);

  /* Mirror on y axis. */
  src.y = 0;
  src.w = 1;
  src.h = originalSurface->h;
  dest.y = 0;
  for(x=0; x < originalSurface->w; x++) {
    src.x = x;
    dest.x = renderSurface->w - x - 1;
    SDL_BlitSurface(renderSurface, &src, renderSurface, &dest);
  }

  /* Mirror on x axis. */
  src.x = 0;
  src.w = renderSurface->w;
  src.h = 1;
  dest.x = 0;
  for(y=0; y < originalSurface->h; y++) {
    src.y = y;
    dest.y = renderSurface->h -y - 1;
    SDL_BlitSurface(renderSurface, &src, renderSurface, &dest);
  }
    
  return;
}
