/** Implements a very basic noise filter. 
 *
 * Authors: Kristoffer Lawson, setok@fishpool.com
 *
 * $Header: /home/sleber/cvs/liboil/flashFilter.c,v 1.2 2003/01/08 16:04:55 sleber Exp $
 */


#include <SDL/SDL.h>
#include <tcl.h>
#include "filters.h"
#include "oil.h"
#include "debug.h"

/* Function prototypes. */

static Uint32 getpixel16(SDL_Surface *surface, int x, int y);
static void putpixel16(SDL_Surface *surface, int x, int y, Uint32 colour);
SDL_Surface *flashFilterCreate(SDL_Surface *originalSurface, 
			       Tcl_HashTable *options,
			       void **filterData);
static void flashFilterRender(SDL_Surface *originalSurface, 
			    SDL_Surface *renderSurface,
			    int offsetX, int offsetY, 
			    void *filterData);


static Uint32 getpixel16(SDL_Surface *surface, int x, int y) {
  Uint8 *p;
  
  int bpp = surface->format->BytesPerPixel;
  p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

  return *(Uint16 *)p;   
}


static void putpixel16(SDL_Surface *surface, int x, int y, Uint32 colour) {
  Uint16 *bufp;

  bufp = (Uint16 *)surface->pixels + y*surface->pitch/2 + x;
  *bufp = colour;
}


int flashFilterInit() {
  Oil_AddFilter("flash", flashFilterCreate, flashFilterRender);
  return TCL_OK;
}


SDL_Surface *flashFilterCreate(SDL_Surface *originalSurface, 
			       Tcl_HashTable *options,
			       void **filterData) {
  return Oil_CreateDisplaySurface16(originalSurface->w, originalSurface->h);
}


static void flashFilterRender(SDL_Surface *originalSurface, 
			    SDL_Surface *renderSurface, int offsetX, int offsetY,
			    void *filterData) {

	//  int dx, dy;
	//  int x, y;
  static int speedcounter = 0;
  static int speed = 6;         /* Flash speed; 1 = fastest */
  static int rand_tbl_counter = 0;
  Uint32 bgcolor; //, srcPixel, newPixel;
  //  Uint8 r, g, b;
  //  SDL_Rect dstrect;
  
  int rand[1000] = { 1, 7, 10, 8, 3, 5, 5, 9, 2, 2, 3, 5, 3, 7, 2, 2, 7, 4, 7, 7, 
		      5, 6, 10, 3, 1, 6, 2, 5, 5, 9, 1, 9, 5, 9, 2, 5, 7, 6, 3, 2, 
		      9, 8, 3, 8, 7, 9, 1, 2, 6, 1, 10, 1, 9, 3, 8, 10, 4, 1, 9, 4, 
		      8, 7, 5, 3, 6, 9, 2, 3, 9, 10, 1, 2, 10, 3, 2, 10, 10, 5, 7, 5, 
		      7, 3, 10, 9, 4, 2, 10, 2, 1, 8, 10, 7, 6, 7, 8, 4, 4, 8, 10, 1, 
		      6, 6, 8, 5, 6, 1, 2, 6, 5, 7, 8, 3, 7, 2, 7, 1, 10, 10, 3, 1, 
		      9, 6, 5, 6, 4, 2, 1, 8, 10, 1, 3, 1, 4, 6, 6, 2, 10, 2, 2, 8, 
		      1, 1, 3, 3, 9, 1, 4, 9, 8, 10, 7, 4, 5, 3, 10, 5, 9, 10, 6, 3, 
		      9, 3, 7, 4, 4, 5, 10, 7, 4, 1, 10, 1, 5, 7, 5, 4, 3, 8, 7, 1, 
		      7, 3, 2, 9, 10, 9, 8, 5, 6, 4, 1, 5, 4, 10, 10, 10, 2, 6, 4, 3, 
		      8, 3, 10, 2, 10, 7, 8, 1, 4, 8, 8, 4, 1, 9, 1, 7, 10, 1, 7, 1, 
		      9, 2, 5, 1, 10, 8, 5, 9, 4, 7, 10, 2, 8, 9, 10, 7, 2, 9, 1, 10, 
		      8, 7, 5, 1, 3, 1, 6, 9, 5, 1, 3, 8, 8, 2, 6, 2, 4, 3, 1, 7, 
		      1, 6, 3, 1, 7, 3, 1, 7, 3, 6, 7, 1, 10, 5, 4, 5, 1, 9, 9, 3, 
		      10, 2, 2, 6, 8, 9, 6, 1, 5, 9, 5, 2, 8, 2, 10, 8, 7, 5, 6, 8, 
		      5, 6, 4, 9, 2, 7, 2, 7, 2, 5, 7, 4, 5, 10, 7, 9, 8, 8, 8, 3, 
		      3, 3, 1, 7, 10, 8, 1, 10, 6, 2, 9, 6, 10, 4, 4, 6, 4, 8, 1, 2, 
		      3, 2, 1, 7, 10, 7, 3, 9, 3, 2, 10, 1, 3, 7, 8, 2, 6, 1, 9, 4, 
		      5, 5, 2, 5, 5, 10, 4, 2, 4, 4, 7, 6, 5, 7, 4, 3, 4, 2, 7, 3, 
		      2, 10, 6, 8, 4, 7, 6, 10, 8, 1, 1, 2, 5, 8, 9, 1, 3, 9, 6, 1, 
		      8, 8, 1, 10, 1, 4, 3, 5, 6, 7, 1, 5, 6, 3, 10, 1, 9, 9, 10, 10, 
		      8, 8, 6, 8, 6, 8, 8, 7, 4, 2, 10, 10, 10, 1, 5, 5, 4, 1, 9, 4, 
		      4, 8, 7, 5, 6, 4, 7, 1, 3, 6, 10, 1, 9, 6, 1, 8, 10, 9, 6, 10, 
		      3, 3, 5, 9, 4, 3, 10, 10, 5, 1, 1, 8, 6, 8, 10, 9, 8, 5, 3, 9, 
		    6, 1, 1, 9, 9, 3, 8, 10, 4, 9, 6, 4, 1, 1, 10, 6, 9, 1, 9, 1,
		    4, 9, 10, 1, 10, 9, 5, 8, 4, 7, 3, 3, 9, 1, 1, 1, 2, 3, 6, 5, 
		      3, 5, 7, 5, 10, 7, 5, 8, 10, 2, 6, 10, 7, 3, 6, 1, 4, 9, 5, 8, 
		      1, 9, 4, 1, 4, 6, 4, 2, 1, 7, 6, 1, 8, 9, 5, 8, 9, 9, 7, 3, 
		      9, 1, 4, 4, 4, 4, 9, 3, 6, 2, 5, 2, 4, 1, 7, 2, 4, 5, 9, 8, 
		      4, 4, 7, 8, 4, 5, 2, 3, 4, 5, 10, 4, 1, 10, 5, 1, 9, 8, 8, 5, 
		      10, 2, 2, 3, 7, 4, 6, 10, 1, 5, 5, 10, 6, 10, 7, 10, 5, 5, 1, 3, 
		      2, 1, 2, 7, 7, 1, 8, 10, 6, 8, 2, 8, 4, 9, 5, 1, 4, 7, 9, 5, 
		      4, 3, 9, 7, 2, 7, 1, 1, 10, 2, 9, 5, 8, 8, 10, 6, 2, 7, 4, 1, 
		      2, 8, 4, 6, 4, 7, 3, 9, 7, 7, 2, 2, 8, 3, 1, 7, 8, 1, 9, 3, 
		      10, 2, 9, 2, 7, 6, 1, 6, 4, 4, 7, 3, 9, 9, 9, 6, 3, 9, 5, 10, 
		      2, 9, 6, 6, 3, 10, 2, 7, 5, 6, 2, 5, 2, 9, 7, 5, 1, 2, 2, 10, 
		      1, 7, 9, 8, 8, 9, 6, 4, 9, 10, 4, 4, 3, 7, 2, 6, 7, 8, 2, 4, 
		      3, 5, 7, 6, 8, 2, 6, 9, 10, 1, 7, 2, 2, 5, 2, 2, 4, 2, 4, 4, 
		      6, 10, 2, 4, 4, 6, 3, 2, 6, 6, 6, 1, 10, 9, 9, 2, 5, 5, 5, 5, 
		      2, 7, 9, 2, 3, 3, 8, 2, 7, 2, 2, 10, 6, 8, 6, 5, 2, 8, 3, 4, 
		      2, 6, 2, 4, 6, 5, 7, 3, 5, 7, 7, 4, 4, 10, 4, 2, 8, 8, 8, 9, 
		      9, 8, 3, 5, 5, 8, 9, 9, 2, 4, 4, 8, 5, 1, 9, 2, 2, 6, 7, 7, 
		      10, 8, 10, 5, 4, 5, 1, 4, 9, 3, 7, 4, 7, 1, 3, 9, 5, 1, 1, 4, 
		      6, 10, 5, 3, 10, 9, 10, 7, 9, 3, 5, 9, 3, 10, 7, 5, 2, 7, 8, 7, 
		      5, 2, 3, 6, 5, 1, 8, 2, 4, 9, 8, 8, 7, 4, 5, 7, 4, 1, 2, 9, 
		      2, 8, 4, 5, 10, 8, 8, 3, 4, 2, 4, 6, 2, 9, 3, 8, 5, 7, 5, 5, 
		      4, 3, 1, 6, 4, 8, 5, 8, 8, 7, 4, 1, 2, 8, 2, 8, 7, 8, 3, 9, 
		      6, 1, 9, 1, 9, 9, 5, 2, 4, 3, 7, 8, 1, 5, 3, 2, 9, 2, 1, 7, 
		      2, 10, 8, 7, 9, 5, 4, 1, 7, 1, 7, 5, 6, 5, 4, 5, 9, 9, 3, 7, 
		      6, 8, 6, 7, 9, 4, 3, 6, 10, 1, 6, 4, 2, 9, 9, 4, 8, 10, 9, 4 };



  if( speedcounter++ % speed == 0 ) {
	  /* Clear the surface */

	  if (rand[rand_tbl_counter++] % 2 == 0 ) {
		  //		  bgcolor = SDL_MapRGB(renderSurface->format, 0, 0, 0);
		  bgcolor = originalSurface->format->colorkey;
		  if (SDL_FillRect( renderSurface, NULL, bgcolor ) != 0 )
			  D2PRINTF(("Error in SDL_FillRect!"));
	  } else {
		  if (SDL_BlitSurface( originalSurface, NULL, renderSurface, NULL ) != 0)
			  D2PRINTF(("Error in SDL_BlitSurface!"));
	  }

	  if( rand_tbl_counter > 999 )
		  rand_tbl_counter = 0;
  }

  /* Don't let speedcounter wrap around, even though it would probably be safe */
  if( speedcounter > 100000)
	  speedcounter = 0;

  return;
}
