/** Implements a very basic noise filter. 
 *
 * Authors: Kristoffer Lawson, setok@fishpool.com
 *
 * $Header: /home/sleber/cvs/liboil/noiseFilter.c,v 1.1.1.1 2003/01/06 02:09:05 sleber Exp $
 */


#include <SDL/SDL.h>
#include <tcl.h>
#include "filters.h"
#include "oil.h"
#include "debug.h"


/* Function prototypes. */

static Uint32 getpixel16(SDL_Surface *surface, int x, int y);
static void putpixel16(SDL_Surface *surface, int x, int y, Uint32 colour);
SDL_Surface *noiseFilterCreate(SDL_Surface *originalSurface, 
			       Tcl_HashTable *options,
			       void **filterData);
static void noiseFilterRender(SDL_Surface *originalSurface, 
			      SDL_Surface *renderSurface,
			      int offsetX, int offsetY,
			      void *filterData);


static Uint32 getpixel16(SDL_Surface *surface, int x, int y) {
  Uint8 *p;
  
  int bpp = surface->format->BytesPerPixel;
  p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

  return *(Uint16 *)p;   
}


static void putpixel16(SDL_Surface *surface, int x, int y, Uint32 colour) {
  Uint16 *bufp;

  bufp = (Uint16 *)surface->pixels + y*surface->pitch/2 + x;
  *bufp = colour;
}


int noiseFilterInit() {
  Oil_AddFilter("noise", noiseFilterCreate, noiseFilterRender);
  return TCL_OK;
}


SDL_Surface *noiseFilterCreate(SDL_Surface *originalSurface, 
			       Tcl_HashTable *options,
			       void **filterData) {
  return Oil_CreateDisplaySurface16(originalSurface->w, originalSurface->h);
}


static void noiseFilterRender(SDL_Surface *originalSurface, 
		       SDL_Surface *renderSurface, int offsetX, int offsetY,
		       void *filterData) {
  int x;
  int y;
  Uint32 srcPixel, green;

  green=SDL_MapRGB(renderSurface->format, 0x00, 0x88, 0);

  SDL_LockSurface(originalSurface);
  SDL_LockSurface(renderSurface);

  /* NOTE: rand() not necessarily terribly fast. */
  for(y=0; y < originalSurface->h; y++) {
    for(x=0; x < (originalSurface->w - 2); x++) {      
      if((rand() % 10) == 1) {
	putpixel16(renderSurface, x+offsetX, y+offsetY, green);
	putpixel16(renderSurface, x+1+offsetX, y+offsetY, green);
	putpixel16(renderSurface, x+2+offsetX, y+offsetY, green);
	x+=2;
      } else {
	srcPixel=getpixel16(originalSurface, x, y); 
	putpixel16(renderSurface, x+offsetX, y+offsetY, srcPixel);
      }
    }
  }
  SDL_UnlockSurface(originalSurface);
  SDL_UnlockSurface(renderSurface);

  return;
}
