#!/usr/bin/perl
#
# Author: Taneli Korhonen, ttkorhon@cc.hut.fi
#
# Generates random integers and prints them so that they are
# easy to copy-paste into a pre-calc array in a C program.
# The output can also be redirected into a file, which then can
# be included in a modern text editor.
#
# Usage: ./rand.pl [min] [max] [amount]
#
# Example: generate 50 random numbers between 1 and 100:
#
#    ./rand.pl 1 100 50
#
# Bugs:
# - prints an extra comma (",") after the last value, which then has to be
# removed by hand when inserting the values into a C program
#

use POSIX;

my $nums_per_row = 20; # Numbers per row

# Check the number of command line arguments
if (scalar @ARGV != 3 ) {
    print "Usage: ./rand.pl [min] [max] [amount]\n";
    exit;
}

my $min = $ARGV[0];
my $max = $ARGV[1]+1; # Doesn't work correctly without the +1
my $count = $ARGV[2];
my $max_rand = $max-$min;
my $i;
my $rowcounter = 0;    # Count printed numbers to see when it's time for a \n

print "{ ";

for ($i = 0; $i < $count; $i++ ) {
    sprintf
    my $num = (int rand $max_rand) + $min;
    
    print "$num, ";

    $rowcounter++;

    if ($rowcounter == $nums_per_row ) {
	print "\n";
	$rowcounter = 0;
    }
}

print " };\n";
