#!/usr/bin/perl
#
# Author: Taneli Korhonen, ttkorhon@cc.hut.fi
#
# Pre-calculates values of sin(x) and prints them so that they are
# easy to copy-paste into a pre-calc array in a C program.
# The output can also be redirected into a file, which then can
# be included in a modern text editor.
#
# Usage: ./sin.pl [amount] [range]
#
# Amount gives the number of values to create. Range gives the maximum
# value for x (multiplied by pi) when calculating the sin(x)'s.
#
# Example: generate 100 values of sin(x) when x goes from 0 to 2*pi:
#
#    ./sin.pl 100 2
#
# Bugs:
# - prints an extra comma (",") after the last value, which then has to be
# removed by hand when inserting the values into a C program
#
use POSIX;

# Check the number of command line arguments
if (scalar @ARGV != 2 ) {
    print "Usage: ./sin.pl [amount] [range]\n";
    print "\n";
    print "Amount:  number of values to create\n";
    print "Range:   x goes from 0 to range*pi\n";
    exit;
}

my $nums_per_row = 10; # Numbers per row

my $amount = $ARGV[0];
my $range = $ARGV[1] * 3.141592;
my $step = $range / $amount;
my $x;
my $rowcounter = 0;

print "{ ";

for ($x = 0; $x < $range; $x += $step ) {
    my $num;
    
    if( $x == 0 ) {
	$num == 1;
    } else {
	$num = sin($x);
    }
    
    printf "%.2f, ", $num;

    $rowcounter++;

    if ($rowcounter == $nums_per_row ) {
	print "\n";
	$rowcounter = 0;
    }

}

print " };\n";
