/** Speckled effect: throws dots around.
 *
 * Authors: Taneli Korhonen, ttkorhon@cc.hut.fi
 *          Kristoffer Lawson, setok@fishpool.com
 *
 * $Header: /home/sleber/cvs/liboil/wobble2Filter.c,v 1.6 2003/02/17 01:24:37 setok Exp $
 */


#include <SDL/SDL.h>
#include <tcl.h>
#include "filters.h"
#include "oil.h"
#include "debug.h"


/* Macros. */

#define RAND_TABLE_SIZE 10000
/* Maximum distance to throw dot. */
#define SCATTER_RANGE 11


/* Types. */

typedef struct {
  int speed;
  int speedcounter;
  int firstRender;
} ScatterData;


/* Variables */

/* Table containing suitable random values. */
static int Rand[RAND_TABLE_SIZE];


/* Function prototypes. */

static Uint32 getpixel16(SDL_Surface *surface, int x, int y);
static void putpixel16(SDL_Surface *surface, int x, int y, Uint32 colour);
SDL_Surface *wobble2FilterCreate(SDL_Surface *originalSurface, 
			       Tcl_HashTable *options,
			       void **filterData);
static void wobble2FilterRender(SDL_Surface *originalSurface, 
			    SDL_Surface *renderSurface,
			    int offsetX, int offsetY, 
			    void *filterData);


static Uint32 getpixel16(SDL_Surface *surface, int x, int y) {
  Uint8 *p;
  
  int bpp = surface->format->BytesPerPixel;
  p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

  return *(Uint16 *)p;   
}


static void putpixel16(SDL_Surface *surface, int x, int y, Uint32 colour) {
  Uint16 *bufp;

  bufp = (Uint16 *)surface->pixels + y*surface->pitch/2 + x;
  *bufp = colour;
}


int wobble2FilterInit() {
  int i;

  Oil_AddFilter("wobble2", wobble2FilterCreate, wobble2FilterRender);

  for(i=0; i < RAND_TABLE_SIZE; i++) {
    Rand[i] = rand() % SCATTER_RANGE;
  }

  return TCL_OK;
}


SDL_Surface *wobble2FilterCreate(SDL_Surface *originalSurface, 
			       Tcl_HashTable *options,
			       void **filterData) {
  ScatterData *scatterData;

  scatterData = malloc(sizeof(ScatterData));
  scatterData->speed = 2;
  scatterData->speedcounter = 2;
  scatterData->firstRender = 1;
  *filterData=scatterData;

  return Oil_CreateDisplaySurface16(originalSurface->w, originalSurface->h);
}


static void wobble2FilterRender(SDL_Surface *originalSurface, 
				SDL_Surface *renderSurface, int offsetX, 
				int offsetY,
				void *filterData) {
  int dx, dy;
  int x, y;
  static int rand_tbl_counter = 0;
  Uint32 srcPixel; //, newPixel;
  ScatterData *scatterData = filterData;
  
  if(scatterData->firstRender) {
    SDL_BlitSurface( originalSurface, NULL, renderSurface, NULL );
    scatterData->firstRender = 0;
  }

  if( scatterData->speedcounter++ % scatterData->speed == 0 ) {
    //    SDL_BlitSurface( originalSurface, NULL, renderSurface, NULL );
    
    SDL_LockSurface(originalSurface);
    SDL_LockSurface(renderSurface);
    
    for( x = 0; x < originalSurface->w; x++ ) {
      for( y = 0; y < originalSurface->h; y++ ) {
	srcPixel=getpixel16(originalSurface, x+offsetX, y+offsetY); 
	
	dx = Rand[rand_tbl_counter++];
	dy = Rand[rand_tbl_counter++];
	  
	if (x+dx >= originalSurface->w)
	  dx = 0;
	if (y+dy >= originalSurface->h)
	  dy = 0;

	// :NOTE: Bug here. If offset is set, can go beyond border. 
	putpixel16(renderSurface, x+dx+offsetX, y+dy+offsetY, srcPixel);
	
	if( rand_tbl_counter > RAND_TABLE_SIZE-3 )
	  rand_tbl_counter = 0;	
      }
    }
    //	  D2PRINTF(("rand counter = %d", rand_tbl_counter ));
    SDL_UnlockSurface(originalSurface);
    SDL_UnlockSurface(renderSurface);
  } 

  scatterData->speedcounter %= 100000;

  return;
}
