/** Implements a very basic wobble filter. 
 *
 * Authors: Taneli Korhonen, ttkorhon@cc.hut.fi
 *
 * $Header: /home/sleber/cvs/liboil/wobble3Filter.c,v 1.3 2003/02/02 01:19:19 setok Exp $
 */


#include <SDL/SDL.h>
#include <tcl.h>
#include "filters.h"
#include "oil.h"
#include "debug.h"

/* Function prototypes. */

static Uint32 getpixel16(SDL_Surface *surface, int x, int y);
static void putpixel16(SDL_Surface *surface, int x, int y, Uint32 colour);
SDL_Surface *wobble3FilterCreate(SDL_Surface *originalSurface, 
			       Tcl_HashTable *options,
			       void **filterData);
static void wobble3FilterRender(SDL_Surface *originalSurface, 
			    SDL_Surface *renderSurface,
			    int offsetX, int offsetY, 
			    void *filterData);


static Uint32 getpixel16(SDL_Surface *surface, int x, int y) {
  Uint8 *p;
  
  int bpp = surface->format->BytesPerPixel;
  p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

  return *(Uint16 *)p;   
}


static void putpixel16(SDL_Surface *surface, int x, int y, Uint32 colour) {
  Uint16 *bufp;

  bufp = (Uint16 *)surface->pixels + y*surface->pitch/2 + x;
  *bufp = colour;
}


int wobble3FilterInit() {
  Oil_AddFilter("wobble3", wobble3FilterCreate, wobble3FilterRender);
  return TCL_OK;
}


SDL_Surface *wobble3FilterCreate(SDL_Surface *originalSurface, 
			       Tcl_HashTable *options,
			       void **filterData) {
  return Oil_CreateDisplaySurface16(originalSurface->w, originalSurface->h);
}


static void wobble3FilterRender(SDL_Surface *originalSurface, 
			    SDL_Surface *renderSurface, int offsetX, int offsetY,
			    void *filterData) {

  int dx, dy;
  int x, y;
  static int speedcounter = 0;
  static int speed = 7;         /* Wobble3 speed; 1 = fastest */
  static int rand_tbl_counter = 0;
  //  static int init_done = 0;
  static int counter = 0, dir = 1;
  Uint32 srcPixel; //, newPixel;
  //  SDL_Rect dstrect;
  
  int rand[10000] =
{ 4, 5, 3, 5, 2, 2, 1, 2, 4, 2, 2, 1, 2, 2, 5, 4, 2, 3, 2, 3, 
5, 4, 1, 1, 1, 3, 4, 3, 1, 1, 4, 1, 3, 2, 4, 3, 3, 1, 1, 1, 
3, 3, 3, 2, 5, 3, 4, 5, 4, 2, 1, 4, 3, 2, 2, 4, 3, 3, 3, 4, 
4, 3, 5, 5, 3, 4, 1, 1, 2, 2, 3, 2, 1, 1, 2, 5, 1, 2, 5, 4, 
1, 2, 3, 3, 5, 1, 3, 1, 2, 4, 1, 5, 3, 2, 5, 3, 4, 5, 2, 4, 
4, 5, 5, 2, 2, 2, 2, 3, 4, 2, 5, 5, 2, 2, 2, 1, 5, 5, 5, 2, 
3, 3, 1, 2, 3, 1, 2, 3, 2, 3, 2, 2, 2, 2, 4, 4, 3, 4, 4, 5, 
4, 1, 3, 5, 4, 3, 2, 1, 2, 2, 5, 2, 2, 4, 2, 1, 1, 1, 4, 2, 
5, 1, 2, 2, 1, 3, 2, 1, 5, 5, 1, 1, 1, 2, 4, 5, 5, 4, 3, 1, 
4, 4, 4, 4, 3, 2, 1, 3, 4, 2, 3, 1, 5, 5, 3, 5, 1, 4, 1, 2, 
5, 3, 4, 4, 1, 5, 1, 5, 2, 4, 1, 4, 1, 2, 3, 2, 5, 4, 3, 3, 
5, 2, 4, 2, 2, 4, 1, 5, 2, 1, 5, 4, 1, 3, 4, 4, 4, 4, 2, 5, 
2, 4, 2, 5, 2, 3, 2, 5, 5, 1, 4, 2, 2, 3, 4, 2, 2, 1, 2, 2, 
5, 2, 2, 2, 3, 3, 2, 3, 4, 2, 2, 1, 4, 3, 5, 5, 3, 3, 4, 1, 
2, 2, 5, 3, 1, 5, 2, 3, 5, 5, 1, 5, 1, 3, 3, 1, 4, 1, 1, 4, 
2, 2, 2, 5, 1, 4, 1, 1, 5, 1, 2, 4, 2, 3, 3, 2, 1, 4, 4, 2, 
4, 5, 3, 3, 4, 3, 3, 3, 4, 3, 5, 4, 2, 2, 2, 1, 4, 5, 5, 4, 
5, 3, 4, 1, 4, 4, 4, 4, 2, 2, 4, 4, 1, 3, 2, 3, 3, 2, 1, 1, 
4, 1, 3, 5, 3, 2, 3, 5, 4, 3, 1, 5, 2, 3, 2, 2, 4, 2, 4, 2, 
3, 2, 3, 1, 3, 1, 5, 3, 5, 5, 3, 2, 1, 2, 4, 3, 2, 1, 2, 3, 
4, 3, 4, 5, 4, 5, 4, 5, 5, 2, 1, 3, 2, 4, 4, 3, 1, 3, 5, 1, 
4, 4, 2, 5, 4, 3, 5, 4, 4, 5, 5, 5, 4, 1, 5, 5, 4, 5, 5, 5, 
5, 3, 3, 2, 1, 1, 5, 5, 3, 5, 1, 5, 2, 2, 2, 2, 5, 5, 4, 4, 
2, 5, 4, 4, 3, 4, 5, 3, 3, 4, 5, 3, 3, 4, 4, 4, 1, 5, 2, 1, 
1, 1, 5, 5, 1, 5, 1, 5, 5, 1, 4, 3, 3, 5, 2, 5, 5, 3, 5, 4, 
2, 5, 1, 4, 2, 3, 2, 4, 1, 3, 2, 1, 3, 1, 2, 4, 2, 4, 1, 5, 
1, 2, 1, 1, 5, 5, 2, 1, 5, 4, 4, 1, 5, 5, 5, 2, 5, 1, 3, 3, 
4, 5, 4, 1, 4, 4, 3, 3, 4, 4, 5, 4, 3, 3, 4, 4, 1, 4, 3, 1, 
4, 1, 3, 5, 1, 5, 4, 1, 2, 1, 1, 5, 3, 5, 5, 1, 2, 3, 1, 4, 
3, 2, 1, 4, 5, 5, 4, 1, 5, 2, 4, 3, 5, 2, 3, 3, 1, 3, 3, 3, 
2, 5, 5, 2, 5, 1, 2, 4, 5, 3, 2, 5, 2, 1, 3, 2, 3, 5, 5, 1, 
2, 1, 5, 4, 3, 1, 3, 5, 1, 1, 2, 3, 3, 2, 4, 2, 3, 3, 5, 3, 
4, 4, 1, 2, 2, 3, 2, 4, 3, 2, 3, 5, 1, 3, 2, 1, 4, 4, 1, 1, 
5, 2, 5, 3, 3, 3, 5, 1, 2, 4, 5, 4, 1, 3, 3, 5, 2, 2, 3, 2, 
2, 1, 1, 2, 4, 2, 4, 3, 4, 4, 4, 2, 1, 4, 5, 2, 2, 3, 3, 3, 
5, 4, 5, 3, 3, 4, 2, 5, 2, 3, 2, 5, 2, 3, 5, 4, 2, 4, 4, 1, 
4, 4, 3, 5, 1, 1, 3, 1, 5, 1, 4, 1, 5, 3, 3, 1, 5, 3, 2, 2, 
3, 1, 3, 1, 3, 1, 4, 1, 1, 3, 3, 3, 3, 5, 5, 2, 5, 2, 4, 2, 
2, 3, 3, 1, 1, 3, 5, 2, 2, 4, 2, 2, 4, 4, 4, 1, 4, 2, 4, 5, 
3, 2, 3, 5, 2, 1, 3, 2, 4, 1, 2, 3, 4, 5, 4, 1, 4, 3, 2, 4, 
3, 3, 5, 3, 2, 5, 5, 4, 1, 5, 2, 4, 3, 2, 1, 1, 1, 2, 5, 4, 
5, 5, 4, 3, 2, 4, 1, 1, 2, 4, 3, 1, 3, 5, 4, 5, 1, 2, 2, 1, 
2, 3, 1, 1, 1, 1, 3, 5, 3, 2, 5, 2, 4, 4, 3, 5, 2, 2, 2, 5, 
3, 1, 3, 5, 1, 5, 1, 4, 2, 1, 4, 5, 3, 1, 2, 2, 1, 3, 1, 3, 
3, 3, 5, 4, 1, 4, 5, 5, 2, 4, 5, 1, 2, 4, 3, 2, 3, 3, 5, 5, 
1, 2, 2, 1, 3, 3, 1, 4, 5, 3, 2, 5, 2, 5, 4, 2, 2, 4, 1, 4, 
3, 4, 1, 5, 5, 3, 4, 2, 4, 2, 5, 5, 1, 5, 3, 3, 3, 5, 3, 4, 
4, 2, 1, 3, 5, 3, 5, 2, 5, 2, 2, 3, 5, 3, 1, 1, 5, 4, 5, 2, 
3, 3, 4, 4, 5, 5, 5, 3, 3, 1, 2, 5, 3, 2, 3, 3, 5, 3, 1, 3, 
4, 1, 3, 3, 4, 2, 5, 1, 1, 4, 5, 4, 1, 5, 1, 1, 1, 5, 1, 5, 
4, 4, 1, 1, 2, 1, 1, 4, 3, 5, 5, 5, 3, 2, 3, 5, 2, 1, 4, 4, 
5, 1, 5, 3, 5, 5, 4, 5, 2, 2, 3, 1, 3, 5, 3, 4, 5, 5, 1, 4, 
5, 5, 4, 1, 1, 1, 2, 2, 5, 4, 3, 1, 5, 2, 5, 3, 5, 5, 1, 1, 
5, 2, 3, 5, 5, 3, 4, 2, 4, 4, 1, 2, 1, 3, 3, 1, 5, 1, 2, 2, 
3, 2, 2, 5, 5, 5, 5, 2, 5, 4, 5, 1, 4, 4, 2, 5, 4, 1, 3, 5, 
2, 1, 2, 1, 2, 2, 5, 2, 5, 2, 1, 5, 1, 5, 3, 2, 1, 5, 3, 3, 
3, 4, 1, 4, 4, 1, 3, 1, 1, 3, 2, 3, 2, 5, 1, 3, 1, 5, 3, 5, 
1, 3, 4, 1, 2, 3, 5, 2, 1, 1, 4, 2, 3, 1, 4, 1, 3, 3, 5, 1, 
2, 1, 4, 4, 5, 2, 3, 2, 1, 2, 4, 3, 4, 5, 4, 3, 4, 2, 5, 4, 
2, 5, 2, 2, 3, 2, 5, 3, 5, 3, 4, 3, 2, 3, 2, 4, 1, 5, 1, 4, 
2, 3, 4, 1, 4, 2, 1, 2, 4, 1, 5, 1, 1, 4, 5, 2, 3, 2, 2, 4, 
4, 5, 3, 5, 2, 4, 5, 2, 3, 2, 1, 5, 1, 1, 2, 4, 2, 5, 5, 5, 
4, 1, 2, 1, 1, 3, 4, 1, 3, 3, 5, 2, 5, 2, 5, 3, 3, 3, 3, 1, 
1, 4, 5, 5, 2, 5, 2, 2, 2, 4, 5, 5, 3, 4, 2, 2, 1, 2, 3, 5, 
3, 2, 5, 1, 5, 4, 3, 2, 1, 3, 5, 2, 2, 5, 2, 5, 5, 5, 1, 5, 
1, 4, 1, 5, 5, 2, 2, 2, 5, 5, 1, 4, 2, 5, 1, 5, 4, 2, 2, 1, 
2, 2, 5, 3, 4, 3, 3, 2, 2, 5, 3, 1, 2, 3, 4, 3, 1, 2, 2, 5, 
5, 4, 3, 4, 3, 1, 4, 4, 3, 2, 1, 1, 5, 3, 2, 4, 2, 1, 3, 3, 
5, 5, 4, 4, 3, 4, 2, 4, 4, 3, 1, 3, 4, 4, 5, 5, 5, 3, 5, 3, 
3, 1, 1, 3, 2, 1, 4, 1, 4, 1, 2, 5, 3, 2, 1, 1, 1, 3, 1, 3, 
1, 4, 2, 5, 3, 4, 3, 3, 2, 3, 5, 5, 4, 5, 3, 1, 1, 2, 3, 4, 
5, 2, 1, 1, 5, 3, 4, 2, 4, 4, 5, 3, 4, 2, 1, 5, 4, 5, 2, 4, 
2, 1, 5, 1, 2, 4, 4, 5, 2, 4, 4, 3, 2, 3, 1, 5, 2, 5, 5, 5, 
4, 5, 2, 2, 5, 5, 4, 2, 2, 3, 3, 1, 1, 3, 5, 2, 1, 2, 1, 3, 
5, 3, 2, 5, 2, 2, 4, 3, 3, 4, 3, 3, 5, 4, 3, 5, 2, 3, 3, 2, 
4, 4, 5, 3, 4, 5, 4, 1, 1, 3, 5, 3, 3, 3, 2, 5, 4, 4, 3, 5, 
1, 3, 1, 2, 1, 5, 2, 3, 2, 2, 4, 2, 2, 3, 2, 4, 1, 2, 1, 5, 
3, 3, 4, 4, 4, 5, 4, 3, 1, 1, 5, 2, 4, 1, 3, 2, 2, 2, 5, 2, 
3, 3, 2, 1, 4, 4, 5, 1, 4, 4, 2, 4, 3, 4, 4, 1, 1, 5, 2, 1, 
3, 3, 3, 4, 1, 5, 2, 5, 4, 1, 5, 3, 1, 1, 1, 1, 4, 4, 1, 2, 
3, 1, 5, 3, 4, 2, 3, 1, 1, 2, 2, 2, 3, 1, 5, 5, 5, 1, 1, 2, 
3, 2, 4, 1, 1, 5, 1, 1, 2, 3, 4, 3, 3, 3, 5, 4, 3, 3, 2, 2, 
1, 3, 5, 1, 4, 3, 5, 5, 1, 1, 1, 4, 3, 1, 2, 4, 4, 3, 2, 2, 
3, 5, 2, 4, 1, 1, 5, 5, 3, 4, 5, 2, 3, 4, 4, 5, 1, 5, 3, 1, 
1, 2, 2, 3, 5, 5, 5, 5, 5, 3, 4, 3, 4, 3, 5, 1, 1, 2, 5, 5, 
2, 2, 3, 4, 4, 4, 1, 2, 1, 2, 2, 1, 4, 1, 4, 5, 3, 1, 4, 5, 
2, 1, 4, 2, 5, 2, 1, 5, 2, 2, 5, 2, 3, 1, 4, 5, 3, 2, 1, 4, 
1, 2, 2, 5, 2, 4, 1, 1, 5, 1, 3, 4, 4, 2, 5, 4, 3, 2, 3, 1, 
5, 1, 5, 4, 3, 5, 4, 1, 1, 4, 1, 1, 1, 1, 4, 2, 1, 5, 3, 3, 
3, 5, 1, 3, 2, 3, 2, 4, 3, 2, 5, 4, 1, 4, 2, 4, 4, 1, 2, 4, 
4, 1, 3, 2, 4, 4, 1, 5, 1, 1, 2, 3, 1, 2, 2, 2, 1, 2, 4, 2, 
5, 2, 5, 3, 4, 1, 1, 3, 4, 4, 1, 4, 3, 3, 5, 4, 5, 3, 5, 2, 
5, 1, 2, 1, 3, 2, 5, 2, 3, 4, 2, 4, 1, 3, 1, 1, 5, 1, 5, 5, 
5, 5, 5, 3, 4, 1, 2, 1, 4, 2, 4, 1, 4, 5, 2, 2, 1, 4, 2, 2, 
3, 4, 1, 1, 1, 5, 4, 4, 2, 3, 3, 5, 4, 4, 2, 5, 2, 1, 2, 2, 
3, 1, 3, 4, 5, 1, 4, 2, 2, 5, 3, 3, 1, 2, 4, 3, 2, 4, 3, 1, 
4, 3, 1, 5, 2, 1, 2, 3, 4, 2, 3, 2, 1, 3, 4, 4, 1, 5, 1, 3, 
1, 2, 5, 4, 3, 5, 3, 2, 1, 2, 4, 5, 5, 3, 3, 5, 2, 4, 1, 1, 
1, 5, 5, 2, 2, 3, 4, 2, 1, 1, 5, 3, 5, 4, 2, 2, 2, 1, 5, 3, 
4, 5, 3, 2, 4, 5, 4, 4, 1, 4, 4, 1, 4, 1, 2, 1, 5, 2, 5, 4, 
5, 2, 1, 1, 3, 2, 1, 4, 2, 4, 5, 2, 2, 3, 5, 2, 2, 4, 4, 3, 
4, 3, 3, 3, 3, 1, 5, 1, 2, 3, 2, 3, 1, 5, 3, 2, 5, 5, 1, 3, 
1, 2, 5, 1, 4, 2, 5, 1, 3, 1, 5, 5, 2, 2, 3, 3, 1, 4, 4, 4, 
2, 1, 3, 4, 5, 3, 1, 1, 4, 4, 4, 3, 2, 3, 2, 2, 1, 2, 5, 2, 
2, 3, 5, 3, 2, 3, 5, 2, 2, 1, 2, 2, 1, 3, 2, 3, 2, 5, 2, 2, 
2, 1, 4, 4, 4, 1, 4, 3, 5, 1, 5, 1, 1, 1, 4, 5, 5, 4, 1, 5, 
1, 1, 4, 1, 3, 2, 2, 3, 5, 2, 5, 1, 5, 3, 2, 1, 4, 5, 3, 2, 
5, 4, 2, 1, 1, 3, 4, 2, 5, 2, 4, 3, 1, 2, 3, 1, 2, 5, 2, 2, 
4, 1, 1, 3, 5, 1, 1, 5, 4, 1, 1, 2, 1, 4, 4, 1, 2, 3, 1, 3, 
5, 5, 3, 1, 1, 2, 5, 2, 5, 2, 3, 1, 5, 1, 2, 3, 2, 2, 4, 4, 
5, 5, 4, 2, 1, 4, 5, 3, 5, 4, 2, 1, 2, 2, 5, 3, 4, 4, 1, 3, 
4, 4, 3, 1, 4, 1, 1, 2, 5, 1, 2, 1, 3, 3, 3, 4, 2, 5, 4, 1, 
1, 1, 4, 1, 3, 3, 4, 5, 5, 1, 1, 4, 3, 5, 4, 2, 4, 1, 1, 5, 
4, 2, 3, 3, 4, 4, 1, 5, 5, 1, 4, 3, 3, 5, 5, 5, 5, 1, 4, 3, 
5, 4, 5, 4, 3, 4, 1, 2, 1, 3, 4, 5, 3, 3, 2, 3, 3, 2, 4, 5, 
3, 3, 5, 2, 2, 5, 2, 4, 4, 1, 5, 3, 1, 5, 4, 3, 2, 4, 5, 1, 
4, 5, 2, 1, 4, 1, 3, 1, 5, 1, 2, 1, 1, 1, 5, 1, 5, 4, 4, 3, 
3, 5, 5, 1, 5, 3, 3, 1, 5, 3, 2, 1, 4, 3, 5, 3, 4, 5, 3, 3, 
2, 5, 4, 1, 4, 1, 2, 4, 2, 5, 4, 3, 1, 3, 1, 4, 3, 3, 5, 2, 
1, 3, 1, 5, 3, 4, 4, 3, 5, 4, 1, 1, 4, 3, 5, 1, 3, 5, 3, 5, 
2, 4, 1, 3, 5, 3, 4, 1, 5, 1, 2, 2, 5, 5, 3, 4, 4, 3, 4, 5, 
4, 3, 1, 4, 1, 3, 4, 3, 2, 5, 2, 2, 4, 3, 1, 3, 2, 1, 2, 3, 
3, 1, 2, 4, 2, 4, 1, 1, 2, 1, 1, 2, 2, 3, 1, 3, 1, 5, 1, 4, 
1, 3, 3, 5, 2, 2, 2, 2, 1, 5, 2, 5, 2, 4, 3, 5, 2, 2, 2, 5, 
1, 5, 1, 3, 2, 1, 2, 1, 2, 2, 2, 1, 4, 4, 4, 4, 4, 1, 1, 2, 
4, 3, 4, 1, 1, 3, 2, 4, 4, 3, 5, 1, 4, 5, 5, 3, 4, 5, 5, 1, 
3, 4, 3, 2, 1, 5, 1, 1, 4, 2, 4, 3, 3, 1, 1, 3, 4, 5, 2, 1, 
4, 2, 3, 5, 3, 5, 2, 3, 2, 1, 1, 1, 1, 1, 2, 1, 5, 2, 5, 3, 
4, 4, 3, 4, 5, 4, 4, 5, 3, 5, 4, 3, 4, 5, 5, 5, 2, 4, 4, 5, 
4, 1, 2, 3, 3, 1, 5, 4, 2, 2, 5, 2, 3, 5, 2, 4, 3, 5, 1, 4, 
5, 3, 5, 3, 2, 2, 5, 4, 5, 2, 2, 3, 4, 3, 3, 2, 3, 4, 2, 1, 
4, 2, 4, 5, 3, 3, 3, 3, 4, 3, 2, 1, 5, 3, 5, 4, 4, 1, 5, 3, 
2, 2, 3, 5, 1, 2, 5, 3, 4, 3, 4, 1, 2, 4, 5, 5, 4, 2, 1, 4, 
2, 1, 5, 3, 3, 2, 1, 3, 3, 4, 2, 4, 5, 4, 2, 4, 1, 1, 2, 2, 
2, 4, 3, 4, 4, 2, 1, 1, 2, 2, 4, 4, 3, 2, 4, 2, 3, 3, 1, 2, 
3, 1, 5, 3, 1, 1, 3, 5, 4, 1, 4, 3, 4, 3, 3, 5, 1, 2, 4, 5, 
4, 4, 5, 1, 2, 4, 1, 3, 5, 3, 5, 2, 1, 4, 1, 3, 4, 5, 5, 1, 
1, 5, 3, 2, 3, 5, 3, 5, 3, 4, 5, 4, 2, 5, 1, 1, 1, 5, 3, 4, 
5, 5, 3, 3, 1, 4, 1, 5, 4, 4, 4, 1, 2, 5, 3, 2, 1, 4, 4, 2, 
4, 1, 5, 3, 2, 3, 1, 5, 1, 2, 2, 5, 3, 5, 4, 5, 1, 1, 3, 3, 
4, 3, 5, 1, 4, 5, 3, 1, 4, 1, 5, 2, 4, 5, 5, 4, 5, 3, 1, 3, 
2, 2, 2, 3, 2, 5, 1, 1, 4, 4, 1, 5, 2, 5, 4, 4, 5, 5, 5, 5, 
4, 5, 5, 3, 1, 1, 5, 3, 2, 3, 3, 2, 3, 4, 4, 4, 1, 1, 3, 5, 
5, 2, 5, 5, 3, 2, 4, 2, 2, 5, 4, 5, 5, 4, 3, 1, 4, 4, 3, 2, 
3, 5, 4, 2, 4, 3, 1, 4, 1, 1, 3, 1, 3, 5, 4, 2, 5, 5, 2, 4, 
2, 4, 2, 1, 1, 1, 5, 2, 2, 1, 5, 4, 3, 4, 2, 1, 5, 5, 1, 4, 
5, 5, 1, 2, 3, 4, 5, 2, 4, 3, 4, 4, 4, 3, 5, 5, 1, 5, 2, 2, 
5, 1, 4, 4, 3, 3, 1, 3, 5, 2, 2, 3, 1, 2, 2, 1, 5, 2, 4, 1, 
4, 2, 4, 4, 1, 4, 3, 3, 1, 4, 1, 3, 4, 5, 2, 2, 2, 4, 5, 4, 
5, 1, 1, 4, 2, 4, 3, 2, 2, 1, 5, 3, 3, 4, 4, 2, 5, 5, 1, 4, 
2, 4, 4, 5, 5, 5, 3, 4, 5, 4, 1, 1, 1, 4, 2, 5, 4, 2, 4, 5, 
3, 4, 4, 3, 5, 1, 3, 2, 4, 4, 3, 3, 1, 3, 4, 3, 3, 2, 5, 1, 
3, 4, 4, 1, 3, 5, 5, 1, 4, 4, 3, 3, 4, 5, 3, 3, 2, 2, 3, 5, 
2, 3, 1, 5, 3, 3, 4, 2, 3, 5, 4, 3, 5, 2, 3, 3, 1, 3, 4, 5, 
5, 1, 3, 4, 4, 3, 4, 5, 2, 2, 5, 3, 1, 1, 3, 4, 4, 1, 3, 5, 
2, 2, 4, 5, 5, 4, 1, 2, 3, 1, 5, 5, 5, 3, 4, 4, 4, 1, 1, 4, 
2, 5, 4, 1, 5, 3, 2, 3, 5, 1, 3, 4, 4, 2, 3, 5, 4, 1, 1, 3, 
2, 1, 4, 3, 5, 4, 2, 5, 3, 1, 5, 3, 5, 5, 1, 1, 5, 1, 5, 2, 
5, 4, 1, 5, 2, 1, 1, 5, 2, 1, 2, 1, 2, 1, 4, 4, 4, 2, 4, 1, 
2, 3, 1, 3, 1, 5, 5, 2, 2, 5, 4, 1, 5, 5, 3, 4, 5, 5, 1, 5, 
5, 4, 1, 1, 1, 1, 2, 1, 1, 5, 4, 1, 3, 4, 2, 4, 4, 3, 4, 1, 
5, 1, 3, 4, 3, 1, 4, 1, 4, 5, 4, 2, 5, 3, 5, 4, 1, 4, 2, 2, 
5, 2, 1, 2, 2, 4, 5, 4, 5, 1, 3, 2, 5, 5, 4, 5, 2, 3, 2, 2, 
1, 2, 1, 4, 4, 4, 2, 4, 5, 4, 3, 4, 2, 1, 3, 5, 4, 4, 3, 5, 
5, 3, 1, 1, 4, 3, 1, 5, 5, 2, 2, 1, 4, 4, 5, 3, 2, 4, 2, 3, 
2, 3, 4, 2, 2, 4, 5, 2, 5, 2, 4, 3, 2, 5, 1, 5, 1, 1, 2, 4, 
3, 4, 2, 5, 3, 5, 3, 5, 3, 4, 1, 3, 1, 1, 1, 2, 3, 1, 3, 5, 
4, 3, 2, 3, 4, 2, 5, 3, 2, 5, 4, 3, 5, 3, 5, 3, 4, 2, 4, 5, 
5, 2, 1, 3, 1, 1, 4, 1, 5, 1, 4, 1, 2, 2, 4, 3, 3, 1, 1, 2, 
5, 3, 3, 3, 4, 4, 2, 5, 1, 2, 2, 5, 3, 2, 2, 1, 1, 2, 2, 1, 
4, 2, 4, 2, 4, 4, 1, 5, 5, 3, 4, 5, 1, 1, 3, 4, 1, 2, 3, 1, 
4, 5, 3, 1, 4, 1, 5, 3, 1, 4, 5, 5, 5, 4, 2, 3, 3, 5, 2, 2, 
3, 1, 3, 5, 2, 3, 2, 1, 4, 5, 3, 1, 5, 3, 1, 5, 4, 2, 4, 2, 
4, 1, 4, 1, 1, 1, 2, 4, 1, 3, 5, 2, 4, 4, 1, 2, 1, 3, 4, 3, 
3, 2, 1, 2, 2, 1, 2, 1, 1, 3, 5, 2, 5, 4, 1, 5, 3, 5, 3, 1, 
1, 4, 3, 2, 5, 4, 5, 3, 5, 3, 3, 4, 3, 4, 4, 4, 1, 2, 4, 2, 
2, 2, 3, 1, 4, 5, 2, 2, 2, 4, 3, 5, 1, 3, 3, 1, 2, 1, 3, 2, 
3, 5, 1, 1, 4, 4, 1, 4, 5, 4, 2, 2, 4, 5, 3, 1, 3, 1, 4, 1, 
5, 1, 5, 3, 3, 5, 5, 5, 3, 2, 1, 2, 2, 5, 4, 2, 4, 3, 5, 5, 
2, 2, 2, 5, 2, 5, 1, 5, 2, 3, 2, 5, 5, 3, 2, 4, 5, 3, 4, 1, 
2, 2, 4, 5, 4, 3, 3, 5, 2, 3, 2, 2, 2, 3, 4, 1, 5, 1, 5, 2, 
3, 2, 2, 3, 5, 4, 4, 1, 2, 5, 1, 1, 5, 3, 3, 2, 4, 3, 2, 1, 
5, 2, 3, 5, 5, 2, 1, 5, 4, 5, 3, 1, 3, 4, 5, 1, 3, 2, 5, 5, 
1, 4, 2, 4, 4, 2, 3, 3, 2, 2, 2, 5, 1, 2, 5, 3, 5, 2, 5, 5, 
5, 5, 4, 3, 4, 5, 5, 5, 4, 4, 2, 4, 2, 3, 2, 1, 1, 5, 5, 2, 
1, 1, 5, 1, 1, 4, 4, 2, 1, 3, 5, 1, 3, 3, 5, 3, 2, 2, 3, 1, 
4, 2, 3, 4, 2, 1, 4, 1, 1, 5, 3, 5, 3, 2, 4, 2, 5, 1, 5, 3, 
4, 5, 1, 5, 5, 2, 2, 2, 2, 1, 1, 2, 1, 2, 4, 4, 4, 4, 4, 5, 
3, 5, 4, 2, 4, 4, 4, 3, 5, 4, 1, 2, 4, 4, 1, 2, 1, 5, 1, 2, 
5, 2, 2, 4, 5, 4, 4, 4, 4, 3, 3, 5, 4, 3, 4, 5, 4, 2, 5, 3, 
2, 4, 4, 4, 2, 4, 1, 1, 4, 1, 1, 3, 1, 5, 5, 2, 2, 3, 5, 4, 
2, 4, 3, 2, 1, 2, 2, 2, 1, 5, 5, 5, 2, 2, 3, 5, 3, 4, 3, 1, 
5, 5, 4, 5, 4, 1, 1, 5, 5, 5, 4, 2, 2, 1, 2, 2, 2, 5, 1, 3, 
1, 5, 5, 2, 1, 3, 2, 4, 2, 3, 5, 1, 3, 2, 5, 1, 5, 5, 5, 3, 
4, 2, 3, 3, 1, 2, 2, 3, 3, 5, 2, 2, 1, 3, 5, 3, 1, 1, 4, 2, 
1, 4, 4, 4, 2, 1, 3, 2, 5, 4, 5, 2, 5, 1, 5, 2, 5, 2, 1, 4, 
5, 3, 2, 3, 2, 4, 2, 4, 5, 5, 3, 2, 2, 5, 1, 2, 4, 5, 5, 1, 
3, 3, 2, 2, 5, 5, 4, 4, 4, 4, 3, 2, 5, 5, 5, 4, 1, 2, 4, 3, 
5, 4, 1, 3, 5, 3, 5, 1, 3, 1, 1, 3, 4, 4, 1, 5, 4, 5, 5, 3, 
4, 3, 3, 5, 1, 2, 3, 4, 3, 5, 1, 2, 3, 2, 3, 2, 5, 4, 2, 2, 
2, 5, 5, 3, 3, 1, 4, 4, 5, 4, 2, 2, 1, 1, 3, 3, 5, 4, 5, 5, 
1, 5, 2, 4, 5, 2, 5, 2, 5, 4, 3, 1, 3, 2, 1, 5, 3, 2, 4, 1, 
1, 1, 2, 3, 3, 1, 4, 5, 4, 5, 1, 4, 2, 5, 3, 4, 3, 5, 4, 5, 
5, 3, 2, 3, 3, 5, 5, 1, 2, 4, 2, 1, 5, 4, 4, 1, 4, 2, 1, 4, 
3, 2, 1, 1, 3, 3, 4, 5, 3, 1, 3, 2, 2, 5, 3, 5, 4, 1, 5, 5, 
4, 1, 4, 2, 4, 1, 5, 1, 1, 1, 1, 3, 2, 2, 5, 1, 5, 2, 5, 5, 
5, 3, 4, 3, 4, 2, 4, 5, 5, 3, 3, 3, 4, 3, 5, 2, 5, 1, 4, 3, 
4, 5, 4, 3, 1, 2, 3, 3, 2, 3, 4, 3, 1, 5, 2, 3, 4, 2, 5, 1, 
2, 2, 1, 3, 4, 4, 1, 1, 3, 5, 3, 5, 5, 5, 4, 5, 4, 3, 4, 3, 
4, 2, 3, 2, 5, 5, 3, 2, 1, 4, 4, 4, 1, 4, 2, 5, 5, 2, 4, 2, 
2, 2, 3, 1, 3, 2, 2, 5, 1, 2, 5, 4, 3, 5, 4, 3, 1, 4, 5, 2, 
5, 4, 3, 3, 3, 1, 5, 5, 3, 1, 3, 2, 1, 3, 1, 3, 2, 2, 5, 1, 
3, 1, 2, 3, 4, 2, 5, 2, 4, 4, 5, 3, 2, 5, 3, 5, 1, 4, 1, 5, 
4, 1, 1, 5, 5, 4, 3, 1, 2, 4, 4, 5, 4, 4, 5, 2, 1, 2, 1, 4, 
5, 2, 2, 5, 3, 3, 5, 3, 3, 3, 1, 1, 5, 4, 4, 4, 1, 5, 3, 2, 
1, 2, 5, 1, 2, 2, 3, 4, 2, 2, 3, 5, 5, 5, 2, 4, 4, 4, 4, 1, 
5, 1, 2, 3, 5, 4, 2, 4, 4, 4, 2, 4, 3, 3, 5, 5, 3, 1, 4, 3, 
1, 3, 5, 3, 4, 3, 2, 3, 2, 3, 4, 4, 3, 4, 2, 5, 1, 4, 3, 3, 
5, 1, 5, 1, 1, 2, 5, 3, 1, 3, 5, 5, 5, 3, 4, 4, 3, 1, 1, 1, 
4, 3, 2, 2, 5, 3, 3, 1, 2, 1, 2, 3, 5, 3, 1, 3, 5, 4, 5, 2, 
3, 2, 5, 5, 2, 2, 4, 5, 5, 2, 3, 1, 4, 4, 4, 3, 2, 1, 1, 5, 
3, 3, 4, 1, 4, 5, 5, 5, 1, 2, 2, 2, 3, 4, 4, 5, 5, 5, 3, 4, 
5, 2, 1, 4, 2, 2, 5, 3, 1, 5, 4, 1, 3, 5, 3, 3, 1, 4, 4, 5, 
4, 4, 2, 1, 3, 2, 4, 5, 3, 5, 1, 1, 5, 3, 1, 1, 5, 5, 4, 1, 
3, 2, 4, 4, 5, 5, 3, 5, 2, 2, 1, 2, 3, 1, 1, 5, 4, 1, 4, 1, 
2, 3, 5, 4, 5, 4, 4, 5, 4, 1, 1, 2, 4, 1, 1, 4, 1, 2, 2, 4, 
1, 5, 1, 5, 1, 5, 1, 2, 5, 3, 3, 1, 4, 2, 3, 4, 3, 3, 3, 3, 
1, 1, 5, 3, 2, 4, 3, 4, 2, 1, 5, 2, 3, 1, 3, 3, 4, 2, 1, 1, 
1, 3, 3, 2, 5, 5, 2, 5, 3, 1, 1, 1, 5, 5, 5, 5, 1, 2, 4, 1, 
5, 2, 2, 4, 4, 5, 1, 5, 4, 5, 3, 4, 2, 2, 3, 5, 3, 1, 3, 2, 
3, 4, 3, 4, 1, 2, 2, 3, 2, 4, 4, 4, 2, 3, 1, 4, 5, 5, 1, 4, 
4, 5, 3, 5, 4, 3, 2, 5, 5, 2, 3, 5, 3, 3, 4, 3, 1, 4, 4, 2, 
3, 2, 5, 5, 1, 2, 5, 1, 1, 1, 1, 1, 2, 1, 4, 4, 1, 5, 4, 3, 
1, 1, 2, 5, 4, 4, 5, 1, 4, 2, 1, 4, 5, 1, 1, 3, 3, 5, 1, 4, 
4, 5, 4, 2, 1, 3, 2, 4, 4, 2, 5, 3, 4, 2, 5, 1, 3, 1, 4, 3, 
3, 4, 4, 1, 4, 4, 4, 2, 1, 1, 5, 1, 4, 2, 1, 5, 4, 4, 1, 1, 
3, 5, 3, 3, 2, 2, 4, 4, 1, 3, 2, 4, 4, 5, 3, 4, 2, 5, 5, 2, 
2, 4, 1, 3, 3, 2, 4, 2, 1, 1, 3, 1, 3, 2, 4, 3, 2, 4, 4, 2, 
3, 4, 2, 4, 1, 4, 2, 2, 2, 1, 2, 3, 3, 4, 2, 4, 5, 3, 3, 3, 
1, 4, 1, 2, 1, 5, 4, 2, 5, 3, 3, 3, 4, 1, 5, 3, 3, 5, 1, 5, 
5, 2, 2, 2, 4, 1, 3, 5, 4, 3, 5, 3, 3, 4, 5, 1, 5, 2, 4, 5, 
2, 4, 5, 5, 4, 1, 3, 4, 3, 3, 1, 2, 3, 2, 3, 1, 2, 3, 3, 4, 
3, 2, 3, 4, 4, 3, 1, 2, 1, 1, 4, 1, 4, 2, 2, 2, 2, 1, 3, 3, 
4, 2, 2, 1, 1, 4, 5, 4, 4, 3, 5, 1, 2, 3, 3, 3, 3, 5, 4, 2, 
2, 3, 3, 2, 2, 2, 3, 2, 4, 4, 5, 3, 2, 2, 4, 2, 2, 1, 2, 4, 
2, 4, 2, 2, 4, 5, 3, 2, 2, 1, 4, 4, 3, 5, 5, 5, 4, 4, 3, 3, 
5, 1, 2, 1, 4, 3, 3, 3, 5, 5, 5, 4, 5, 4, 1, 2, 2, 5, 4, 1, 
3, 5, 5, 5, 5, 3, 4, 3, 5, 4, 1, 2, 1, 5, 4, 5, 1, 5, 5, 4, 
4, 1, 2, 1, 1, 2, 4, 5, 4, 2, 4, 5, 5, 4, 2, 1, 1, 3, 1, 1, 
4, 2, 2, 3, 1, 3, 1, 3, 2, 4, 3, 5, 5, 4, 3, 4, 2, 4, 1, 1, 
4, 1, 3, 1, 3, 3, 3, 3, 5, 2, 1, 1, 2, 4, 4, 1, 2, 3, 5, 1, 
5, 2, 4, 1, 4, 5, 1, 3, 5, 5, 1, 5, 2, 5, 5, 1, 2, 4, 2, 3, 
1, 3, 5, 4, 2, 1, 2, 2, 1, 3, 2, 5, 5, 5, 2, 3, 1, 2, 5, 5, 
2, 5, 3, 5, 1, 3, 3, 3, 5, 5, 2, 2, 4, 1, 3, 1, 2, 1, 4, 3, 
2, 2, 3, 3, 4, 1, 4, 2, 1, 5, 2, 1, 4, 3, 2, 1, 3, 4, 2, 2, 
2, 4, 5, 5, 4, 1, 5, 3, 4, 5, 1, 5, 4, 1, 1, 2, 1, 3, 2, 5, 
2, 4, 1, 2, 3, 2, 4, 1, 3, 2, 1, 1, 4, 5, 5, 4, 5, 5, 3, 2, 
3, 2, 4, 4, 1, 3, 5, 4, 4, 3, 2, 2, 2, 5, 4, 3, 5, 2, 4, 5, 
4, 5, 5, 4, 5, 4, 2, 3, 3, 1, 2, 3, 2, 3, 3, 4, 4, 4, 4, 5, 
4, 3, 2, 5, 4, 2, 5, 2, 4, 5, 3, 3, 4, 1, 1, 5, 4, 3, 1, 5, 
3, 4, 2, 4, 1, 5, 5, 5, 1, 3, 5, 2, 1, 1, 2, 5, 4, 4, 3, 1, 
4, 3, 4, 1, 1, 2, 5, 3, 5, 3, 4, 2, 2, 4, 2, 1, 4, 5, 5, 2, 
1, 4, 4, 1, 1, 4, 4, 4, 5, 2, 4, 1, 2, 1, 5, 3, 5, 1, 1, 3, 
3, 2, 4, 2, 4, 5, 1, 1, 4, 1, 4, 5, 3, 5, 4, 2, 4, 4, 1, 2, 
4, 3, 4, 5, 4, 4, 5, 4, 4, 2, 2, 3, 5, 1, 4, 1, 2, 2, 4, 3, 
5, 3, 4, 5, 5, 3, 2, 1, 1, 3, 3, 4, 5, 3, 2, 1, 3, 2, 5, 2, 
2, 4, 3, 4, 4, 5, 2, 4, 2, 5, 2, 5, 1, 1, 1, 2, 4, 5, 1, 5, 
4, 5, 3, 5, 5, 2, 4, 5, 5, 1, 3, 4, 1, 3, 3, 4, 3, 3, 5, 5, 
4, 4, 3, 5, 2, 4, 2, 2, 2, 1, 1, 4, 2, 4, 4, 5, 2, 4, 5, 1, 
3, 1, 3, 2, 4, 3, 2, 1, 2, 1, 4, 1, 1, 5, 5, 4, 5, 4, 5, 3, 
5, 1, 2, 1, 2, 3, 5, 1, 2, 3, 4, 1, 4, 4, 1, 4, 5, 4, 4, 4, 
2, 5, 5, 1, 2, 4, 5, 2, 4, 1, 5, 3, 1, 5, 4, 5, 5, 1, 3, 4, 
3, 5, 1, 4, 3, 3, 2, 1, 1, 4, 1, 3, 2, 4, 3, 3, 1, 1, 3, 3, 
3, 3, 1, 5, 2, 3, 4, 1, 4, 1, 3, 1, 2, 4, 1, 5, 5, 4, 3, 4, 
1, 4, 2, 2, 4, 4, 2, 2, 1, 3, 5, 1, 4, 5, 2, 2, 3, 4, 5, 1, 
3, 5, 5, 5, 3, 1, 1, 2, 1, 5, 2, 3, 1, 4, 5, 3, 3, 5, 2, 5, 
3, 3, 2, 4, 5, 5, 5, 2, 2, 1, 3, 5, 2, 4, 3, 3, 4, 5, 3, 2, 
4, 2, 1, 1, 5, 4, 5, 3, 5, 4, 4, 4, 4, 1, 5, 4, 5, 2, 1, 5, 
4, 5, 4, 5, 2, 3, 3, 3, 5, 5, 3, 5, 1, 1, 2, 2, 3, 3, 1, 2, 
5, 5, 3, 1, 4, 2, 4, 2, 4, 1, 5, 4, 5, 5, 5, 5, 5, 2, 1, 4, 
4, 4, 3, 5, 3, 1, 2, 3, 5, 4, 2, 1, 5, 2, 4, 3, 1, 5, 2, 3, 
5, 5, 5, 3, 1, 1, 4, 5, 5, 1, 5, 5, 2, 2, 1, 4, 5, 2, 3, 2, 
1, 4, 2, 1, 5, 1, 3, 5, 3, 4, 3, 4, 2, 1, 2, 5, 4, 2, 5, 1, 
2, 5, 2, 5, 3, 5, 1, 3, 3, 2, 4, 1, 2, 4, 4, 4, 5, 4, 3, 2, 
2, 1, 3, 1, 1, 1, 4, 3, 5, 5, 4, 2, 1, 5, 2, 2, 1, 5, 3, 2, 
1, 4, 4, 4, 1, 3, 2, 5, 3, 4, 1, 3, 3, 4, 2, 4, 2, 4, 1, 2, 
4, 3, 5, 2, 4, 5, 3, 5, 5, 2, 2, 1, 5, 2, 2, 2, 1, 3, 4, 1, 
2, 5, 2, 4, 1, 1, 2, 2, 4, 2, 5, 4, 3, 3, 2, 1, 1, 3, 1, 5, 
1, 4, 5, 1, 2, 4, 5, 1, 4, 2, 3, 3, 1, 1, 2, 5, 2, 3, 1, 3, 
5, 3, 5, 1, 5, 4, 2, 3, 5, 4, 1, 1, 5, 2, 1, 4, 1, 5, 2, 4, 
2, 2, 5, 3, 4, 4, 5, 1, 1, 2, 1, 2, 5, 5, 4, 5, 3, 4, 3, 3, 
3, 2, 1, 1, 3, 4, 5, 3, 2, 5, 1, 2, 3, 5, 2, 5, 5, 4, 4, 5, 
1, 4, 4, 3, 4, 5, 4, 3, 5, 1, 2, 3, 2, 3, 2, 4, 2, 4, 1, 1, 
2, 1, 1, 4, 2, 5, 1, 2, 1, 2, 4, 2, 5, 5, 4, 2, 4, 4, 1, 4, 
4, 5, 5, 5, 2, 5, 4, 1, 3, 2, 2, 5, 4, 1, 1, 2, 5, 2, 1, 5, 
4, 4, 4, 4, 4, 1, 2, 2, 1, 4, 1, 4, 5, 1, 3, 3, 5, 1, 5, 3, 
5, 5, 1, 2, 5, 5, 3, 3, 3, 5, 2, 3, 5, 2, 2, 4, 4, 4, 2, 4, 
3, 2, 1, 3, 4, 3, 1, 5, 5, 5, 2, 3, 1, 5, 1, 3, 4, 4, 1, 2, 
2, 2, 5, 5, 1, 1, 5, 3, 2, 4, 4, 5, 4, 2, 1, 1, 1, 5, 3, 5, 
2, 3, 3, 4, 4, 2, 4, 1, 3, 2, 5, 4, 1, 2, 5, 3, 4, 4, 4, 5, 
5, 5, 5, 3, 4, 1, 4, 1, 1, 1, 1, 5, 2, 4, 3, 4, 1, 5, 2, 5, 
3, 3, 3, 3, 4, 2, 4, 1, 4, 2, 3, 5, 3, 2, 3, 2, 1, 1, 4, 1, 
1, 5, 4, 1, 5, 1, 1, 1, 2, 3, 1, 5, 5, 3, 1, 2, 1, 1, 1, 4, 
3, 2, 3, 3, 2, 1, 3, 5, 5, 1, 3, 2, 2, 4, 5, 2, 1, 3, 2, 1, 
5, 5, 2, 2, 2, 4, 1, 5, 5, 1, 3, 3, 3, 1, 5, 5, 3, 5, 3, 1, 
2, 5, 2, 4, 3, 5, 3, 3, 3, 5, 2, 5, 1, 5, 4, 3, 4, 5, 2, 3, 
1, 4, 5, 3, 4, 4, 2, 5, 3, 2, 3, 3, 3, 1, 2, 2, 2, 5, 4, 4, 
2, 1, 4, 3, 3, 1, 5, 2, 1, 3, 1, 2, 1, 1, 3, 4, 2, 3, 1, 2, 
3, 1, 4, 5, 5, 5, 2, 4, 3, 4, 2, 5, 1, 2, 5, 3, 2, 2, 4, 4, 
2, 1, 5, 4, 3, 1, 5, 5, 2, 2, 2, 5, 5, 5, 4, 2, 5, 1, 4, 1, 
1, 1, 4, 3, 4, 1, 2, 4, 5, 2, 2, 2, 4, 5, 4, 1, 1, 4, 5, 1, 
5, 3, 2, 4, 5, 3, 2, 5, 1, 5, 1, 1, 2, 2, 1, 5, 1, 2, 3, 3, 
4, 4, 5, 5, 4, 2, 4, 4, 5, 1, 4, 1, 2, 3, 3, 1, 5, 4, 1, 4, 
4, 1, 4, 5, 3, 3, 4, 2, 2, 1, 2, 1, 5, 5, 5, 5, 5, 3, 1, 1, 
3, 5, 2, 4, 1, 1, 2, 1, 5, 1, 1, 3, 2, 5, 1, 3, 4, 3, 5, 3, 
2, 1, 3, 3, 1, 3, 3, 5, 5, 4, 4, 1, 4, 3, 4, 5, 5, 5, 4, 3, 
2, 2, 2, 5, 5, 2, 4, 4, 4, 3, 5, 2, 2, 5, 5, 3, 5, 3, 3, 2, 
1, 3, 5, 2, 4, 1, 3, 4, 1, 1, 4, 5, 5, 4, 3, 1, 3, 1, 5, 2, 
5, 4, 3, 5, 3, 2, 1, 2, 4, 2, 2, 5, 3, 4, 2, 2, 5, 5, 1, 3, 
4, 5, 3, 4, 1, 4, 2, 4, 2, 5, 1, 5, 4, 1, 3, 2, 2, 5, 3, 1, 
3, 5, 2, 5, 1, 5, 4, 5, 2, 5, 3, 2, 5, 3, 2, 4, 5, 3, 5, 1, 
1, 4, 5, 2, 4, 5, 3, 2, 5, 1, 3, 3, 2, 5, 3, 4, 2, 2, 5, 2, 
5, 5, 3, 5, 1, 3, 5, 2, 1, 2, 1, 1, 3, 3, 3, 4, 2, 4, 5, 4, 
2, 4, 5, 2, 4, 1, 5, 3, 5, 3, 2, 2, 4, 1, 5, 4, 2, 2, 2, 3, 
4, 5, 1, 1, 1, 3, 4, 4, 1, 5, 5, 2, 5, 3, 4, 3, 1, 2, 4, 1, 
2, 3, 5, 4, 1, 3, 1, 2, 5, 1, 3, 4, 2, 3, 1, 3, 4, 4, 5, 4, 
3, 5, 3, 4, 5, 4, 4, 3, 3, 3, 2, 5, 5, 2, 3, 2, 3, 3, 4, 1, 
1, 4, 3, 5, 1, 1, 2, 2, 2, 4, 1, 5, 5, 5, 3, 4, 5, 5, 5, 4, 
2, 5, 4, 3, 3, 2, 2, 4, 2, 4, 1, 4, 2, 2, 4, 1, 4, 2, 3, 1, 
1, 5, 3, 2, 3, 5, 2, 2, 4, 5, 4, 5, 5, 2, 3, 2, 3, 3, 1, 2, 
1, 4, 2, 2, 4, 5, 5, 1, 4, 5, 3, 4, 3, 5, 4, 4, 4, 1, 4, 1, 
1, 1, 4, 5, 2, 2, 4, 3, 4, 2, 5, 3, 5, 5, 2, 1, 2, 5, 4, 5, 
4, 2, 1, 4, 3, 2, 4, 5, 1, 4, 4, 1, 5, 5, 1, 1, 2, 5, 4, 3, 
2, 3, 4, 2, 3, 4, 3, 4, 5, 5, 5, 1, 3, 5, 5, 2, 3, 1, 4, 4, 
4, 3, 4, 5, 2, 1, 4, 3, 2, 2, 4, 5, 4, 1, 5, 3, 2, 5, 1, 4, 
3, 3, 5, 5, 1, 2, 4, 4, 3, 1, 1, 2, 4, 1, 3, 4, 1, 4, 2, 3, 
2, 3, 5, 4, 2, 2, 1, 4, 5, 5, 3, 1, 5, 2, 1, 4, 2, 1, 2, 5, 
2, 1, 3, 4, 2, 4, 1, 4, 2, 2, 5, 4, 4, 2, 3, 4, 1, 3, 5, 1, 
5, 3, 1, 4, 4, 2, 4, 1, 1, 2, 1, 2, 3, 1, 2, 3, 5, 1, 1, 1, 
3, 3, 1, 5, 3, 2, 4, 3, 5, 5, 5, 2, 5, 5, 5, 1, 5, 5, 3, 4, 
4, 5, 1, 3, 5, 4, 4, 4, 4, 2, 4, 1, 5, 1, 1, 3, 1, 5, 2, 3, 
3, 3, 1, 4, 1, 5, 4, 2, 2, 4, 2, 3, 2, 3, 4, 3, 5, 5, 1, 1, 
1, 5, 1, 3, 3, 5, 3, 2, 3, 2, 4, 2, 1, 4, 1, 1, 1, 2, 1, 5, 
1, 1, 1, 2, 1, 5, 4, 3, 4, 5, 4, 2, 1, 2, 1, 5, 3, 5, 5, 4, 
5, 4, 5, 3, 1, 2, 4, 2, 5, 3, 1, 1, 2, 1, 1, 2, 3, 1, 4, 5, 
2, 2, 1, 1, 4, 2, 5, 1, 4, 2, 3, 3, 5, 1, 4, 4, 5, 3, 4, 4, 
1, 2, 5, 1, 2, 4, 4, 5, 2, 4, 5, 1, 2, 2, 4, 5, 5, 5, 2, 3, 
2, 2, 4, 3, 1, 2, 2, 4, 4, 1, 1, 1, 5, 3, 3, 1, 5, 2, 3, 2, 
5, 2, 3, 5, 4, 5, 3, 4, 5, 4, 4, 3, 5, 1, 3, 5, 1, 5, 4, 4, 
3, 4, 1, 3, 1, 2, 5, 3, 1, 1, 3, 4, 3, 3, 1, 3, 4, 5, 2, 4, 
2, 2, 4, 3, 3, 3, 1, 4, 1, 5, 3, 1, 1, 2, 2, 3, 2, 3, 4, 4, 
1, 3, 2, 1, 4, 5, 2, 4, 4, 4, 3, 1, 4, 4, 4, 1, 2, 3, 5, 1, 
2, 4, 5, 1, 4, 4, 2, 4, 2, 3, 2, 1, 2, 4, 3, 5, 1, 5, 3, 2, 
5, 2, 4, 1, 4, 3, 1, 1, 2, 5, 4, 1, 5, 5, 1, 1, 5, 3, 5, 3, 
4, 5, 1, 4, 5, 1, 1, 2, 3, 3, 4, 4, 4, 5, 4, 1, 5, 5, 2, 4, 
2, 4, 4, 3, 3, 1, 1, 2, 4, 4, 4, 1, 2, 3, 3, 5, 1, 5, 2, 1, 
2, 5, 1, 3, 4, 5, 4, 5, 3, 4, 2, 2, 1, 2, 1, 1, 4, 2, 4, 2, 
5, 1, 5, 3, 1, 2, 3, 3, 4, 4, 4, 4, 1, 4, 5, 3, 3, 1, 5, 3, 
4, 5, 2, 2, 2, 4, 4, 5, 1, 1, 1, 5, 3, 1, 4, 4, 5, 5, 1, 4, 
5, 2, 1, 1, 2, 4, 2, 2, 4, 1, 1, 5, 5, 1, 2, 4, 3, 5, 5, 4, 
2, 3, 1, 5, 1, 3, 1, 4, 5, 1, 5, 3, 4, 4, 3, 5, 4, 1, 4, 4, 
2, 2, 2, 2, 1, 1, 3, 4, 3, 2, 5, 2, 3, 3, 4, 2, 5, 3, 2, 3, 
3, 2, 2, 4, 1, 3, 1, 5, 1, 2, 4, 5, 5, 3, 5, 3, 2, 4, 1, 5, 
4, 4, 3, 2, 5, 5, 4, 1, 3, 5, 4, 4, 3, 4, 5, 5, 2, 2, 2, 1, 
1, 1, 1, 4, 1, 1, 2, 4, 3, 3, 2, 3, 5, 4, 4, 4, 4, 1, 2, 4, 
3, 1, 5, 1, 3, 3, 1, 4, 5, 4, 2, 5, 3, 2, 4, 5, 3, 2, 4, 2, 
4, 5, 1, 1, 4, 4, 1, 3, 5, 5, 4, 3, 1, 3, 3, 2, 5, 1, 1, 4, 
3, 2, 1, 3, 1, 2, 3, 5, 2, 3, 4, 2, 4, 3, 3, 3, 1, 3, 2, 5, 
1, 1, 5, 5, 1, 3, 2, 2, 5, 4, 3, 3, 3, 3, 4, 1, 5, 2, 3, 2, 
5, 2, 5, 1, 5, 2, 2, 3, 3, 2, 4, 1, 4, 5, 3, 5, 2, 1, 1, 2, 
3, 1, 5, 1, 5, 5, 2, 3, 2, 5, 4, 4, 2, 2, 5, 1, 4, 1, 4, 5, 
1, 2, 4, 1, 5, 2, 1, 5, 1, 5, 1, 4, 2, 4, 2, 1, 2, 1, 1, 3, 
4, 1, 1, 5, 1, 2, 4, 3, 5, 5, 2, 5, 2, 5, 2, 1, 1, 2, 5, 1, 
1, 1, 4, 3, 3, 4, 3, 5, 3, 1, 3, 4, 1, 5, 3, 3, 2, 3, 4, 4, 
4, 1, 3, 2, 1, 5, 5, 5, 3, 4, 1, 4, 1, 4, 4, 3, 3, 5, 2, 2, 
3, 5, 3, 1, 3, 2, 4, 2, 2, 2, 3, 5, 2, 4, 3, 4, 4, 3, 5, 1, 
3, 2, 4, 4, 2, 4, 5, 2, 4, 2, 2, 2, 5, 2, 5, 1, 2, 5, 4, 1, 
1, 5, 4, 3, 3, 1, 1, 2, 5, 4, 2, 3, 2, 3, 4, 5, 2, 2, 1, 2, 
4, 2, 1, 3, 1, 5, 3, 4, 3, 2, 5, 3, 2, 3, 3, 5, 2, 3, 5, 1, 
2, 5, 5, 3, 5, 2, 4, 4, 2, 2, 4, 1, 4, 5, 3, 3, 3, 5, 2, 3, 
1, 3, 5, 1, 3, 3, 5, 2, 4, 3, 4, 2, 1, 4, 1, 1, 3, 2, 1, 1, 
3, 5, 3, 2, 5, 1, 5, 4, 4, 1, 3, 3, 5, 3, 4, 3, 2, 3, 3, 2, 
5, 2, 2, 5, 1, 4, 4, 5, 2, 3, 1, 3, 3, 4, 5, 4, 4, 1, 2, 3, 
4, 1, 3, 4, 3, 5, 3, 2, 4, 2, 1, 5, 4, 5, 4, 2, 5, 1, 1, 3, 
2, 3, 3, 5, 1, 3, 1, 1, 4, 1, 2, 3, 2, 5, 3, 2, 2, 5, 4, 5, 
3, 4, 5, 3, 5, 3, 5, 5, 4, 5, 4, 2, 1, 3, 2, 1, 3, 5, 4, 2, 
3, 5, 4, 3, 4, 1, 1, 2, 5, 3, 3, 5, 5, 2, 1, 2, 3, 5, 2, 2, 
4, 5, 2, 3, 1, 2, 3, 5, 5, 4, 3, 3, 3, 4, 5, 2, 4, 4, 5, 4, 
3, 2, 3, 2, 5, 1, 1, 2, 3, 3, 4, 3, 2, 5, 5, 2, 2, 5, 1, 5, 
5, 2, 4, 1, 4, 3, 5, 3, 5, 1, 4, 4, 1, 3, 5, 5, 2, 3, 5, 4, 
5, 3, 5, 1, 2, 1, 1, 2, 3, 1, 4, 2, 3, 2, 1, 3, 4, 3, 5, 3, 
1, 5, 2, 4, 1, 1, 1, 4, 3, 3, 2, 3, 5, 4, 2, 3, 3, 2, 1, 5, 
2, 1, 5, 3, 1, 5, 5, 1, 5, 1, 4, 1, 5, 5, 2, 1, 2, 3, 1, 2, 
3, 3, 5, 1, 4, 2, 4, 5, 2, 1, 3, 1, 2, 4, 5, 1, 5, 2, 5, 5, 
4, 4, 2, 3, 2, 3, 1, 1, 1, 2, 3, 3, 3, 3, 4, 1, 2, 4, 2, 2, 
3, 2, 5, 4, 2, 5, 1, 1, 1, 3, 5, 2, 1, 1, 5, 3, 2, 1, 2, 4, 
2, 5, 4, 2, 2, 3, 4, 2, 4, 2, 3, 3, 2, 2, 4, 2, 5, 4, 5, 4, 
3, 1, 4, 1, 1, 3, 3, 2, 2, 5, 5, 4, 2, 3, 3, 3, 2, 3, 5, 5, 
1, 1, 2, 3, 5, 3, 4, 3, 4, 2, 1, 2, 4, 3, 3, 2, 4, 2, 4, 4, 
2, 1, 3, 2, 1, 1, 1, 3, 2, 3, 1, 5, 2, 2, 2, 2, 4, 4, 5, 1, 
4, 4, 2, 3, 3, 1, 3, 5, 4, 3, 1, 5, 5, 1, 3, 4, 3, 5, 3, 4, 
1, 3, 5, 4, 5, 4, 5, 3, 3, 4, 4, 4, 3, 2, 4, 3, 5, 2, 1, 1, 
3, 3, 1, 4, 1, 3, 1, 2, 1, 4, 4, 5, 2, 2, 3, 1, 4, 2, 3, 5, 
5, 4, 4, 3, 1, 3, 5, 2, 2, 2, 1, 5, 3, 4, 2, 1, 1, 4, 5, 3, 
4, 2, 3, 1, 1, 2, 4, 1, 3, 2, 2, 5, 2, 3, 4, 5, 3, 3, 4, 5, 
2, 2, 2, 2, 2, 3, 4, 4, 3, 4, 1, 1, 4, 5, 4, 5, 2, 4, 5, 5, 
1, 2, 2, 4, 5, 1, 2, 3, 1, 4, 2, 1, 1, 3, 1, 3, 4, 4, 1, 3, 
4, 4, 1, 2, 5, 2, 3, 3, 5, 4, 2, 2, 3, 4, 3, 2, 4, 3, 1, 5, 
2, 5, 4, 3, 3, 5, 5, 5, 5, 3, 2, 3, 5, 5, 2, 4, 1, 3, 2, 5, 
4, 5, 2, 5, 5, 2, 2, 1, 2, 5, 5, 3, 5, 1, 4, 5, 2, 3, 5, 4, 
5, 3, 3, 2, 3, 3, 3, 3, 4, 5, 3, 5, 1, 4, 5, 1, 1, 1, 4, 3, 
3, 3, 2, 4, 5, 2, 2, 5, 2, 1, 3, 1, 4, 1, 5, 4, 5, 3, 5, 4, 
5, 4, 2, 4, 2, 1, 3, 2, 4, 5, 3, 4, 1, 4, 5, 4, 4, 3, 3, 1, 
2, 3, 3, 5, 4, 5, 3, 3, 1, 2, 5, 1, 1, 3, 4, 5, 2, 5, 3, 4, 
2, 2, 5, 1, 3, 5, 4, 1, 5, 1, 4, 2, 4, 5, 3, 2, 1, 3, 3, 2, 
4, 1, 4, 4, 5, 1, 1, 1, 1, 2, 2, 5, 2, 5, 3, 3, 1, 4, 4, 2, 
4, 2, 5, 4, 1, 4, 5, 5, 3, 3, 4, 3, 3, 3, 1, 1, 4, 1, 2, 2, 
2, 4, 4, 5, 4, 1, 4, 4, 4, 5, 4, 5, 4, 2, 5, 2, 5, 4, 4, 1, 
2, 3, 5, 4, 4, 4, 5, 1, 4, 5, 4, 1, 4, 3, 2, 2, 3, 2, 1, 5, 
2, 3, 3, 1, 1, 5, 5, 5, 2, 5, 2, 5, 3, 4, 4, 3, 5, 1, 2, 3, 
5, 5, 3, 3, 2, 3, 4, 2, 3, 4, 3, 2, 4, 3, 4, 4, 2, 3, 2, 4, 
2, 5, 2, 3, 5, 1, 4, 3, 3, 1, 5, 3, 5, 4, 5, 5, 2, 1, 4, 5, 
2, 5, 5, 5, 1, 3, 4, 2, 5, 3, 1, 4, 5, 4, 5, 5, 5, 4, 3, 2, 
5, 1, 4, 4, 2, 5, 1, 2, 1, 2, 1, 5, 4, 1, 1, 1, 2, 2, 4, 3, 
2, 4, 2, 1, 4, 2, 5, 2, 5, 2, 2, 4, 3, 3, 4, 2, 1, 3, 5, 1, 
1, 2, 3, 4, 3, 5, 4, 3, 3, 1, 1, 4, 5, 4, 2, 4, 2, 3, 1, 4, 
3, 1, 4, 5, 1, 2, 3, 2, 1, 4, 4, 4, 2, 2, 3, 1, 5, 5, 3, 5, 
1, 5, 5, 2, 5, 2, 3, 1, 5, 2, 5, 3, 1, 4, 2, 2, 2, 2, 1, 1, 
3, 1, 3, 1, 1, 5, 2, 1, 1, 4, 4, 5, 1, 3, 3, 2, 3, 5, 5, 4, 
2, 3, 1, 3, 4, 1, 3, 1, 3, 3, 1, 2, 1, 1, 3, 1, 5, 4, 5, 4, 
3, 2, 5, 5, 4, 3, 2, 5, 2, 4, 5, 1, 1, 4, 4, 5, 1, 2, 5, 1, 
5, 5, 4, 3, 1, 1, 5, 1, 5, 1, 1, 3, 2, 4, 5, 2, 4, 3, 2, 1, 
3, 1, 1, 4, 2, 5, 3, 2, 4, 2, 5, 4, 2, 3, 1, 1, 4, 5, 5, 5, 
1, 4, 3, 4, 2, 5, 2, 4, 3, 3, 5, 3, 5, 1, 2, 2, 2, 5, 4, 4, 
1, 2, 5, 4, 1, 1, 4, 4, 5, 2, 5, 3, 5, 2, 1, 2, 2, 2, 3, 3, 
4, 1, 1, 3, 4, 4, 4, 1, 4, 4, 5, 4, 2, 1, 2, 4, 3, 1, 3, 5, 
5, 3, 5, 2, 3, 1, 2, 3, 4, 2, 2, 3, 2, 4, 4, 4, 5, 1, 1, 5, 
2, 5, 5, 1, 5, 3, 3, 3, 5, 3, 4, 3, 5, 2, 4, 5, 1, 5, 5, 5, 
4, 2, 4, 5, 2, 1, 2, 5, 1, 1, 1, 4, 3, 5, 4, 2, 3, 2, 3, 2, 
5, 1, 5, 4, 2, 3, 2, 1, 2, 5, 3, 3, 2, 1, 3, 1, 3, 4, 5, 4, 
3, 3, 5, 3, 3, 4, 2, 5, 1, 5, 1, 2, 5, 5, 2, 4, 2, 4, 5, 1, 
4, 1, 1, 3, 2, 1, 2, 1, 2, 2, 2, 4, 4, 4, 3, 3, 2, 5, 1, 1, 
1, 5, 1, 2, 3, 4, 3, 5, 5, 5, 4, 2, 4, 3, 4, 5, 3, 1, 2, 3, 
1, 5, 3, 4, 1, 3, 4, 4, 2, 2, 3, 3, 2, 5, 2, 2, 3, 5, 5, 5, 
3, 5, 2, 3, 4, 1, 4, 5, 2, 4, 1, 5, 2, 5, 5, 2, 5, 3, 2, 4, 
5, 5, 1, 2, 3, 2, 3, 2, 5, 2, 1, 3, 4, 3, 2, 1, 3, 4, 1, 3, 
5, 4, 4, 4, 4, 1, 5, 4, 5, 1, 3, 3, 3, 3, 3, 4, 4, 3, 1, 5, 
1, 2, 4, 4, 1, 1, 5, 2, 2, 2, 4, 5, 1, 3, 3, 1, 5, 3, 5, 2, 
3, 2, 5, 4, 1, 3, 1, 3, 3, 2, 4, 4, 5, 3, 2, 5, 4, 5, 5, 4, 
3, 5, 2, 5, 3, 2, 5, 1, 5, 1, 2, 1, 5, 4, 3, 2, 4, 3, 1, 1, 
3, 1, 2, 2, 4, 3, 3, 5, 1, 4, 2, 3, 5, 5, 3, 4, 4, 2, 2, 3, 
4, 1, 3, 1, 5, 3, 4, 5, 2, 3, 3, 4, 5, 3, 4, 3, 5, 1, 2, 5, 
2, 5, 1, 3, 4, 2, 3, 1, 4, 3, 4, 3, 4, 4, 3, 1, 1, 4, 5, 5, 
5, 4, 5, 4, 2, 5, 4, 5, 3, 3, 1, 2, 3, 1, 4, 3, 4, 1, 4, 2, 
4, 5, 5, 4, 2, 2, 1, 1, 2, 3, 2, 3, 5, 2, 3, 1, 3, 4, 1, 3, 
2, 1, 1, 5, 5, 5, 5, 2, 2, 1, 3, 1, 4, 1, 2, 3, 5, 3, 4, 4, 
5, 5, 4, 2, 5, 2, 1, 3, 2, 1, 5, 3, 1, 3, 2, 2, 3, 1, 1, 5, 
1, 4, 2, 3, 4, 1, 3, 5, 2, 1, 5, 1, 5, 2, 3, 5, 2, 2, 1, 3, 
3, 4, 1, 4, 3, 4, 1, 2, 1, 2, 1, 5, 3, 5, 5, 5, 3, 1, 1, 3, 
2, 4, 3, 5, 1, 2, 3, 2, 3, 1, 1, 3, 2, 2, 1, 3, 4, 5, 3, 4, 
1, 4, 1, 4, 2, 3, 5, 2, 3, 1, 4, 1, 2, 3, 5, 4, 4, 3, 1, 5, 
3, 2, 1, 5, 4, 3, 1, 1, 2, 5, 3, 5, 2, 2, 4, 1, 5, 5, 5, 1, 
1, 2, 3, 2, 5, 2, 4, 3, 2, 4, 5, 2, 3, 1, 1, 4, 5, 1, 2, 3, 
3, 2, 5, 3, 5, 3, 5, 5, 2, 2, 3, 4, 1, 4, 2, 1, 4, 2, 4, 1, 
3, 5, 4, 5, 4, 3, 3, 1, 1, 5, 1, 1, 5, 2, 3, 4, 2, 3, 3, 4, 
1, 1, 1, 1, 4, 4, 1, 5, 5, 3, 1, 5, 3, 5, 1, 5, 3, 3, 5, 2, 
5, 1, 5, 4, 1, 5, 4, 5, 2, 5, 5, 5, 1, 4, 3, 3, 4, 3, 2, 2, 
2, 2, 2, 4, 4, 4, 4, 4, 3, 5, 3, 4, 1, 1, 5, 3, 2, 1, 4, 2, 
4, 4, 1, 1, 3, 4, 4, 5, 5, 4, 4, 1, 2, 2, 1, 3, 4, 2, 3, 4, 
4, 3, 5, 5, 4, 5, 1, 2, 1, 5, 2, 5, 1, 4, 5, 4, 1, 4, 3, 1, 
1, 5, 4, 5, 4, 5, 5, 3, 2, 2, 3, 5, 1, 2, 4, 5, 2, 1, 5, 2, 
4, 1, 1, 3, 2, 4, 4, 1, 2, 1, 4, 4, 4, 2, 2, 2, 5, 4, 3, 2, 
4, 2, 5, 5, 3, 1, 5, 3, 4, 1, 2, 1, 4, 5, 2, 3, 1, 5, 5, 2, 
5, 5, 5, 1, 3, 4, 4, 1, 2, 3, 5, 3, 2, 4, 4, 5, 3, 1, 1, 2, 
1, 5, 5, 5, 3, 2, 3, 5, 3, 4, 3, 1, 4, 3, 4, 2, 3, 4, 3, 1, 
1, 1, 5, 1, 2, 3, 3, 2, 4, 1, 4, 1, 2, 5, 4, 3, 5, 2, 3, 3, 
3, 1, 4, 3, 4, 3, 3, 3, 2, 4, 1, 3, 5, 3, 4, 1, 5, 3, 4, 2, 
3, 4, 4, 1, 4, 1, 2, 2, 1, 5, 3, 1, 5, 1, 5, 5, 4, 2, 5, 4, 
2, 5, 5, 5, 1, 3, 3, 4, 3, 4, 4, 1, 2, 1, 4, 4, 2, 1, 5, 2, 
3, 2, 5, 3, 2, 2, 1, 1, 2, 2, 3, 2, 1, 4, 5, 2, 5, 4, 1, 2, 
1, 4, 2, 5, 5, 2, 4, 1, 5, 4, 2, 5, 4, 5, 4, 1, 2, 3, 1, 5, 
2, 5, 3, 2, 2, 5, 3, 4, 3, 3, 3, 2, 5, 5, 5, 3, 3, 5, 4, 1, 
1, 4, 3, 5, 2, 5, 1, 3, 1, 4, 4, 2, 5, 2, 3, 4, 2, 2, 2, 3, 
2, 1, 4, 5, 4, 1, 1, 2, 1, 5, 1, 5, 5, 2, 1, 2, 5, 2, 2, 4, 
1, 3, 2, 2, 5, 3, 2, 4, 4, 1, 1, 1, 2, 3, 5, 4, 1, 2, 3, 3, 
1, 1, 1, 4, 4, 3, 1, 5, 4, 1, 5, 1, 1, 5, 5, 3, 1, 2, 4, 1, 
2, 2, 4, 1, 1, 1, 3, 5, 2, 5, 5, 4, 3, 2, 5, 1, 3, 2, 3, 2, 
5, 4, 4, 3, 2, 5, 4, 1, 4, 2, 2, 2, 5, 1, 1, 1, 5, 3, 1, 1, 
4, 5, 4, 1, 3, 4, 1, 4, 4, 3, 2, 3, 2, 1, 3, 5, 1, 2, 3, 4, 
4, 2, 4, 2, 2, 1, 3, 1, 4, 2, 4, 4, 4, 2, 4, 3, 2, 3, 4, 5, 
3, 4, 5, 5, 2, 3, 1, 5, 2, 4, 1, 3, 5, 4, 5, 2, 5, 4, 2, 5, 
5, 3, 1, 1, 2, 2, 4, 4, 2, 3, 4, 2, 3, 4, 2, 2, 1, 3, 2, 3, 
2, 5, 3, 5, 4, 1, 5, 1, 3, 5, 4, 1, 2, 5, 1, 3, 3, 2, 1, 5, 
1, 5, 1, 2, 2, 3, 2, 3, 2, 1, 1, 4, 1, 3, 5, 1, 2, 3, 5, 1, 
5, 2, 2, 1, 2, 5, 5, 5, 2, 1, 2, 1, 4, 1, 1, 1, 4, 2, 5, 1, 
4, 4, 2, 1, 4, 2, 4, 2, 2, 1, 4, 1, 3, 1, 4, 1, 5, 5, 1, 3, 
2, 5, 4, 3, 4, 3, 3, 5, 5, 3, 2, 1, 5, 3, 4, 1, 3, 2, 2, 2, 
1, 5, 4, 2, 5, 3, 2, 2, 2, 3, 2, 3, 5, 4, 3, 4, 1, 4, 1, 2, 
1, 1, 4, 3, 5, 1, 3, 3, 1, 1, 4, 4, 5, 5, 4, 5, 1, 4, 1, 4, 
4, 4, 1, 1, 1, 5, 4, 3, 3, 1, 4, 1, 2, 3, 5, 3, 1, 4, 1, 3, 
 };
 
 if( speedcounter % speed == 0 ) {
   SDL_BlitSurface( originalSurface, NULL, renderSurface, NULL );
   SDL_LockSurface(originalSurface);
   SDL_LockSurface(renderSurface);

   for( x = 0; x < originalSurface->w; x++ ) {
     for( y = 0; y < originalSurface->h; y++ ) {
       srcPixel=getpixel16(originalSurface, x+offsetX, y+offsetY); 
       
       dx = rand[rand_tbl_counter++] + counter;
       
       dy = rand[rand_tbl_counter++];
       
       if (x+dx >= originalSurface->w)
	 dx = 0;
       if (y+dy >= originalSurface->h)
	 dy = 0;

       putpixel16(renderSurface, x+dx+offsetX, y+dy+offsetY, srcPixel);

       if( rand_tbl_counter > 9998 )
	 rand_tbl_counter = 0;
       
     }
   }
   SDL_UnlockSurface(originalSurface);
   SDL_UnlockSurface(renderSurface);
   
   counter += dir;
	  if( counter == 5 || counter == -5 )
	    dir *= -1;
	  
 }
 
 if( speedcounter > 100000)
   speedcounter = 0;

  return;
}
