/** Exactly same as wobbleFilter.c, except the amount of
 *  wobble is reduced.
 *
 * Authors: Taneli Korhonen, ttkorhon@cc.hut.fi
 *
 * $Header: /home/sleber/cvs/liboil/wobble4Filter.c,v 1.4 2003/02/17 01:24:37 setok Exp $
 */


#include <SDL/SDL.h>
#include <tcl.h>
#include "filters.h"
#include "oil.h"
#include "debug.h"


/* Macros. */

#define RAND_TABLE_SIZE 100
/* Maximum distance to throw dot. */
#define WOBBLE_RANGE 6


/* Types. */

typedef struct {
  int speed;
  int speedcounter;
} WobbleData;
  

/* Variables */

/* Table containing suitable random values. */
static int Rand[RAND_TABLE_SIZE];


/* Function prototypes. */

static Uint32 getpixel16(SDL_Surface *surface, int x, int y);
static void putpixel16(SDL_Surface *surface, int x, int y, Uint32 colour);
SDL_Surface *wobble4FilterCreate(SDL_Surface *originalSurface, 
			       Tcl_HashTable *options,
			       void **filterData);
static void wobble4FilterRender(SDL_Surface *originalSurface, 
			    SDL_Surface *renderSurface,
			    int offsetX, int offsetY, 
			    void *filterData);


static Uint32 getpixel16(SDL_Surface *surface, int x, int y) {
  Uint8 *p;
  
  int bpp = surface->format->BytesPerPixel;
  p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

  return *(Uint16 *)p;   
}


static void putpixel16(SDL_Surface *surface, int x, int y, Uint32 colour) {
  Uint16 *bufp;

  bufp = (Uint16 *)surface->pixels + y*surface->pitch/2 + x;
  *bufp = colour;
}


int wobble4FilterInit() {
  int i;

  Oil_AddFilter("wobble4", wobble4FilterCreate, wobble4FilterRender);

  for(i=0; i < RAND_TABLE_SIZE; i++) {
    Rand[i] = rand() % WOBBLE_RANGE;
  }

  return TCL_OK;
}


SDL_Surface *wobble4FilterCreate(SDL_Surface *originalSurface, 
			       Tcl_HashTable *options,
			       void **filterData) {
  WobbleData *wobbleData;

  wobbleData = malloc(sizeof(WobbleData));
  wobbleData->speed = 5;
  wobbleData->speedcounter = 5;
  *filterData=wobbleData;

  return Oil_CreateDisplaySurface16(originalSurface->w, originalSurface->h);
}


static void wobble4FilterRender(SDL_Surface *originalSurface, 
				SDL_Surface *renderSurface, int offsetX, 
				int offsetY,
				void *filterData) {

  //  int dx, dy;
  // static int speedcounter = 0;
  // static int speed = 5;         /* Wobble speed; 1 = fastest */
  static int rand_tbl_counter = 0;
  Uint32 bgcolor;
  SDL_Rect dstrect;
  WobbleData *wobbleData = filterData;

  /*
  int rand_x[100] = { 2, 5, 1, 3, 3, 3, 2, 5, 5, 5, 2, 3, 4, 3, 2, 3, 4, 5, 3, 5, 
		      3, 2, 4, 1, 1, 4, 1, 2, 1, 1, 4, 5, 4, 3, 5, 3, 4, 5, 5, 3, 
		      1, 2, 2, 3, 3, 1, 5, 5, 3, 4, 3, 5, 3, 4, 5, 2, 1, 3, 2, 5, 
		      1, 5, 3, 4, 4, 4, 2, 1, 1, 4, 4, 4, 4, 1, 4, 4, 3, 1, 4, 1, 
		      2, 3, 4, 5, 1, 3, 5, 2, 2, 3, 1, 5, 1, 1, 5, 1, 3, 3, 1, 2 };
  
  int rand_y[100] = { 5, 3, 3, 2, 5, 2, 5, 1, 4, 4, 2, 2, 4, 1, 5, 2, 1, 2, 4, 5, 
		      4, 1, 2, 2, 2, 2, 2, 4, 5, 2, 5, 5, 2, 2, 3, 5, 3, 1, 3, 4, 
		      5, 1, 5, 4, 5, 2, 1, 2, 2, 1, 3, 5, 5, 1, 4, 2, 4, 1, 3, 1, 
		      5, 3, 3, 1, 5, 2, 2, 5, 4, 3, 2, 5, 1, 3, 4, 3, 5, 4, 2, 1, 
		      2, 4, 1, 5, 2, 5, 5, 2, 5, 3, 4, 4, 1, 5, 3, 3, 4, 4, 1, 1 };
  */


  if( wobbleData->speedcounter++ % wobbleData->speed == 0 ) {
    dstrect.x = Rand[rand_tbl_counter];
    dstrect.y = Rand[rand_tbl_counter+1];
    dstrect.h = originalSurface->h;
    dstrect.w = originalSurface->w;
	  
    rand_tbl_counter++;

    if( rand_tbl_counter == 98 )
      rand_tbl_counter = 0;

    /* Clear the surface */
    bgcolor = renderSurface->format->colorkey;
    if (SDL_FillRect( renderSurface, NULL, bgcolor ) != 0 )
      D2PRINTF(("Error in SDL_FillRect!"));
    
    if (SDL_BlitSurface( originalSurface, NULL, renderSurface, &dstrect ) != 0)
      D2PRINTF(("Error in SDL_BlitSurface!"));
  }

  /* Don't let speedcounter wrap around, even though it would probably be 
     safe */
  wobbleData->speedcounter %= 100000;

  return;
}
