/** Implements a very basic wobble filter. 
 *
 * Authors: Taneli Korhonen, ttkorhon@cc.hut.fi
 *
 * $Header: /home/sleber/cvs/liboil/wobble5Filter.c,v 1.2 2003/01/08 16:04:55 sleber Exp $
 */


#include <SDL/SDL.h>
#include <tcl.h>
#include "filters.h"
#include "oil.h"
#include "debug.h"

/* Function prototypes. */

static Uint32 getpixel16(SDL_Surface *surface, int x, int y);
static void putpixel16(SDL_Surface *surface, int x, int y, Uint32 colour);
SDL_Surface *wobble5FilterCreate(SDL_Surface *originalSurface, 
			       Tcl_HashTable *options,
			       void **filterData);
static void wobble5FilterRender(SDL_Surface *originalSurface, 
			    SDL_Surface *renderSurface,
			    int offsetX, int offsetY, 
			    void *filterData);


static Uint32 getpixel16(SDL_Surface *surface, int x, int y) {
  Uint8 *p;
  
  int bpp = surface->format->BytesPerPixel;
  p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

  return *(Uint16 *)p;   
}


static void putpixel16(SDL_Surface *surface, int x, int y, Uint32 colour) {
  Uint16 *bufp;

  bufp = (Uint16 *)surface->pixels + y*surface->pitch/2 + x;
  *bufp = colour;
}


int wobble5FilterInit() {
  Oil_AddFilter("wobble5", wobble5FilterCreate, wobble5FilterRender);
  return TCL_OK;
}


SDL_Surface *wobble5FilterCreate(SDL_Surface *originalSurface, 
			       Tcl_HashTable *options,
			       void **filterData) {
  return Oil_CreateDisplaySurface16(originalSurface->w, originalSurface->h);
}


static void wobble5FilterRender(SDL_Surface *originalSurface, 
			    SDL_Surface *renderSurface, int offsetX, int offsetY,
			    void *filterData) {

	//  int dx, dy;
  static int speedcounter = 0;
  static int speed = 5;         /* Wobble speed; 1 = fastest */
  static int rand_tbl_counter = 0;
  Uint32 bgcolor;
  SDL_Rect dstrect;
  
  int rand_x[500] = { 20, 17, 18, 15, 2, 9, 19, 7, 12, 20, 12, 18, 14, 9, 2, 1, 16, 18, 5, 8, 
		      17, 2, 15, 20, 11, 9, 5, 1, 5, 17, 12, 8, 15, 2, 12, 13, 12, 2, 11, 20, 
		      18, 18, 2, 11, 6, 12, 2, 6, 2, 20, 6, 13, 13, 4, 9, 15, 20, 17, 17, 4, 
		      14, 11, 20, 13, 6, 10, 17, 13, 15, 17, 14, 11, 19, 4, 16, 19, 20, 14, 9, 10, 
		      11, 11, 2, 6, 7, 14, 18, 2, 9, 14, 19, 2, 16, 9, 15, 9, 3, 7, 16, 7, 
		      14, 10, 20, 12, 15, 12, 14, 17, 20, 9, 1, 1, 19, 4, 5, 8, 11, 5, 3, 17, 
		      3, 11, 11, 9, 14, 9, 6, 6, 14, 8, 20, 7, 11, 18, 18, 16, 7, 3, 11, 20, 
		      10, 20, 18, 8, 3, 5, 13, 17, 6, 16, 19, 1, 13, 20, 1, 9, 8, 14, 15, 9, 
		      9, 8, 16, 8, 4, 8, 20, 9, 6, 12, 6, 20, 9, 15, 12, 10, 8, 20, 17, 2, 
		      12, 20, 3, 4, 20, 13, 3, 10, 7, 20, 11, 2, 9, 16, 17, 12, 19, 19, 7, 5, 
		      13, 9, 12, 19, 20, 3, 19, 2, 3, 9, 2, 7, 2, 13, 19, 8, 18, 13, 17, 14, 
		      12, 11, 17, 4, 6, 10, 17, 12, 18, 16, 14, 2, 15, 19, 14, 10, 4, 10, 15, 19, 
		      17, 13, 9, 11, 9, 12, 1, 4, 15, 14, 8, 18, 15, 7, 14, 11, 19, 16, 17, 17, 
		      2, 6, 5, 4, 9, 7, 7, 2, 9, 6, 19, 6, 4, 3, 2, 8, 8, 3, 16, 10, 
		      14, 11, 3, 9, 1, 1, 16, 10, 17, 6, 5, 17, 15, 18, 19, 18, 2, 10, 16, 2, 
		      2, 11, 10, 8, 12, 18, 4, 12, 3, 3, 12, 4, 6, 3, 5, 20, 9, 12, 2, 1, 
		      9, 6, 2, 9, 18, 2, 15, 17, 3, 20, 8, 8, 12, 17, 16, 17, 14, 9, 18, 13, 
		      10, 2, 10, 14, 13, 18, 11, 20, 15, 1, 4, 11, 7, 15, 5, 6, 9, 4, 12, 11, 
		      8, 13, 13, 20, 4, 6, 3, 4, 14, 17, 16, 17, 16, 2, 11, 3, 9, 1, 17, 6, 
		      17, 10, 10, 10, 5, 4, 19, 4, 7, 7, 11, 4, 19, 12, 6, 15, 2, 7, 5, 19, 
		      10, 11, 5, 4, 18, 13, 15, 15, 5, 19, 20, 8, 18, 2, 13, 11, 18, 14, 15, 9, 
		      8, 13, 8, 10, 15, 3, 15, 8, 17, 1, 16, 9, 1, 15, 17, 5, 5, 18, 12, 16, 
		      1, 6, 15, 1, 9, 18, 13, 20, 20, 4, 8, 7, 2, 17, 13, 19, 15, 6, 9, 11, 
		      10, 15, 18, 10, 7, 9, 1, 12, 4, 4, 8, 10, 10, 9, 13, 7, 3, 20, 12, 16, 
		      5, 12, 16, 16, 20, 13, 16, 15, 14, 8, 15, 4, 20, 1, 11, 12, 13, 9, 7, 6 };


  
  int rand_y[500] = { 11, 14, 11, 1, 2, 12, 19, 8, 18, 7, 14, 14, 3, 10, 11, 19, 7, 5, 15, 2, 
		      16, 4, 8, 4, 8, 11, 20, 15, 8, 8, 2, 3, 5, 9, 10, 8, 6, 16, 11, 6, 
		      11, 9, 4, 12, 1, 16, 20, 6, 3, 11, 10, 16, 3, 4, 14, 20, 12, 8, 3, 14, 
		      1, 16, 17, 6, 5, 17, 10, 15, 10, 12, 3, 6, 20, 15, 13, 14, 6, 19, 12, 15, 
		      20, 18, 3, 2, 17, 7, 10, 1, 2, 5, 14, 5, 9, 17, 14, 8, 13, 18, 8, 16, 
		      1, 20, 15, 15, 17, 7, 6, 11, 8, 20, 10, 7, 18, 14, 14, 4, 11, 19, 12, 8, 
		      5, 1, 13, 1, 4, 2, 14, 5, 12, 19, 17, 11, 1, 1, 15, 3, 19, 14, 14, 4, 
		      20, 2, 8, 14, 19, 11, 19, 13, 12, 11, 19, 17, 19, 10, 2, 17, 14, 4, 1, 18, 
		      8, 1, 18, 8, 16, 20, 3, 4, 14, 10, 3, 15, 10, 9, 8, 3, 14, 4, 2, 13, 
		      4, 2, 3, 16, 10, 5, 16, 12, 1, 6, 18, 11, 17, 1, 13, 11, 16, 8, 3, 2, 
		      18, 5, 7, 3, 14, 7, 6, 3, 1, 13, 8, 13, 7, 10, 16, 13, 1, 5, 12, 8, 
		      16, 11, 13, 11, 11, 13, 12, 17, 10, 5, 2, 17, 19, 17, 19, 5, 5, 11, 5, 8, 
		      17, 20, 11, 2, 2, 12, 8, 9, 8, 16, 19, 9, 3, 20, 2, 5, 10, 1, 4, 10, 
		      8, 3, 8, 16, 11, 3, 16, 17, 4, 16, 8, 9, 5, 8, 19, 7, 11, 10, 19, 3, 
		      5, 2, 2, 15, 4, 19, 6, 2, 1, 10, 17, 18, 3, 15, 15, 16, 4, 6, 16, 4, 
		      13, 5, 7, 6, 4, 8, 11, 8, 10, 12, 14, 15, 17, 6, 12, 18, 11, 16, 4, 6, 
		      15, 14, 3, 4, 7, 12, 8, 10, 8, 19, 10, 5, 18, 11, 13, 7, 3, 18, 18, 1, 
		      7, 15, 8, 4, 15, 19, 5, 11, 19, 2, 17, 17, 9, 19, 3, 12, 11, 14, 3, 9, 
		      4, 1, 3, 7, 20, 8, 7, 11, 18, 2, 7, 7, 20, 5, 4, 9, 10, 3, 10, 1, 
		      3, 2, 12, 3, 20, 16, 14, 16, 7, 17, 10, 8, 18, 8, 16, 3, 4, 18, 12, 16, 
		      20, 7, 17, 7, 1, 15, 18, 18, 6, 18, 3, 17, 5, 16, 17, 1, 10, 5, 12, 6, 
		      6, 11, 7, 17, 15, 19, 16, 7, 19, 9, 17, 11, 6, 17, 18, 19, 15, 2, 4, 6, 
		      3, 7, 13, 19, 17, 1, 13, 8, 19, 17, 8, 18, 12, 16, 13, 8, 1, 1, 20, 1, 
		      7, 12, 15, 12, 16, 20, 14, 10, 4, 7, 6, 12, 9, 4, 17, 16, 13, 13, 19, 1, 
		      9, 2, 8, 6, 15, 8, 20, 14, 20, 5, 14, 13, 11, 18, 4, 20, 9, 3, 14, 18 };



  if( speedcounter++ % speed == 0 ) {
	  if( rand_x[rand_tbl_counter] < 9 ) {
		  
		  dstrect.x = rand_x[rand_tbl_counter];
		  dstrect.y = rand_y[rand_tbl_counter];
		  dstrect.h = originalSurface->h;
		  dstrect.w = originalSurface->w;

		  /* Clear the surface */
		  //		  bgcolor = SDL_MapRGB(renderSurface->format, 0, 0, 0);
		  bgcolor = originalSurface->format->colorkey;
		  if (SDL_FillRect( renderSurface, NULL, bgcolor ) != 0 )
			  D2PRINTF(("Error in SDL_FillRect!"));
		  
	  }		  
	  if (SDL_BlitSurface( originalSurface, NULL, renderSurface, &dstrect ) != 0)
		  D2PRINTF(("Error in SDL_BlitSurface!"));
	  rand_tbl_counter++;

	  if( rand_tbl_counter == 499 )
		  rand_tbl_counter = 0;

  }

  /* Don't let speedcounter wrap around, even though it would probably be safe */
  if( speedcounter > 100000)
	  speedcounter = 0;

  return;
}
