/** Implements a very basic noise filter. 
 *
 * Authors: Kristoffer Lawson, setok@fishpool.com
 *
 * $Header: /home/sleber/cvs/liboil/wobbleFilter.c,v 1.5 2003/02/17 01:24:37 setok Exp $
 */


#include <SDL/SDL.h>
#include <tcl.h>
#include "filters.h"
#include "oil.h"
#include "debug.h"


/* Macros. */

#define RAND_TABLE_SIZE 100
/* Maximum distance to throw dot. */
#define WOBBLE_RANGE 11


/* Types. */

typedef struct {
  int speed;
  int speedcounter;
} WobbleData;
  

/* Variables */

/* Table containing suitable random values. */
static int Rand[RAND_TABLE_SIZE];


/* Function prototypes. */

static Uint32 getpixel16(SDL_Surface *surface, int x, int y);
static void putpixel16(SDL_Surface *surface, int x, int y, Uint32 colour);
SDL_Surface *wobbleFilterCreate(SDL_Surface *originalSurface, 
			       Tcl_HashTable *options,
			       void **filterData);
static void wobbleFilterRender(SDL_Surface *originalSurface, 
			    SDL_Surface *renderSurface,
			    int offsetX, int offsetY, 
			    void *filterData);


static Uint32 getpixel16(SDL_Surface *surface, int x, int y) {
  Uint8 *p;
  
  int bpp = surface->format->BytesPerPixel;
  p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

  return *(Uint16 *)p;   
}


static void putpixel16(SDL_Surface *surface, int x, int y, Uint32 colour) {
  Uint16 *bufp;

  bufp = (Uint16 *)surface->pixels + y*surface->pitch/2 + x;
  *bufp = colour;
}


int wobbleFilterInit() {
  int i;

  Oil_AddFilter("wobble", wobbleFilterCreate, wobbleFilterRender);

  for(i=0; i < RAND_TABLE_SIZE; i++) {
    Rand[i] = rand() % WOBBLE_RANGE;
  }

  return TCL_OK;
}


SDL_Surface *wobbleFilterCreate(SDL_Surface *originalSurface, 
			       Tcl_HashTable *options,
			       void **filterData) {
  WobbleData *wobbleData;

  wobbleData = malloc(sizeof(WobbleData));
  wobbleData->speed = 2;
  wobbleData->speedcounter = 2;
  *filterData=wobbleData;

  return Oil_CreateDisplaySurface16(originalSurface->w, originalSurface->h);
}


static void wobbleFilterRender(SDL_Surface *originalSurface, 
			       SDL_Surface *renderSurface, int offsetX, 
			       int offsetY,
			       void *filterData) {

  static int rand_tbl_counter = 0;
  Uint32 bgcolor;
  SDL_Rect dstrect;
  WobbleData *wobbleData = filterData;

  if( wobbleData->speedcounter++ % wobbleData->speed == 0 ) {
    dstrect.x = Rand[rand_tbl_counter];
    dstrect.y = Rand[rand_tbl_counter+1];
    dstrect.h = originalSurface->h;
    dstrect.w = originalSurface->w;
	  
    rand_tbl_counter++;
    
    if( rand_tbl_counter == 98 )
      rand_tbl_counter = 0;
    
    /* Clear the surface */
    //	  black = SDL_MapRGB(renderSurface->format, 0, 0, 0);
    bgcolor = renderSurface->format->colorkey;
    if (SDL_FillRect( renderSurface, NULL, bgcolor ) != 0 )
      D2PRINTF(("Error in SDL_FillRect!"));
    
    if (SDL_BlitSurface( originalSurface, NULL, renderSurface, &dstrect ) != 0)
      D2PRINTF(("Error in SDL_BlitSurface!"));
  }

  /* Don't let speedcounter wrap around, even though it would probably be 
     safe */
  wobbleData->speedcounter %= 100000;

  return;
}
