/** Scrolls the object in X direction
 *
 * Authors: Taneli Korhonen, ttkorhon@cc.hut.fi
 *
 */


#include <SDL/SDL.h>
#include <tcl.h>
#include "filters.h"
#include "oil.h"
#include "debug.h"


/* Types. */

typedef struct {
  int speed;
  int speedcounter;
  int step;
  int xOffset;
} ScrollData;


/* Function prototypes. */

static Uint32 getpixel16(SDL_Surface *surface, int x, int y);
static void putpixel16(SDL_Surface *surface, int x, int y, Uint32 colour);
SDL_Surface *xScrollFilterCreate(SDL_Surface *originalSurface, 
			       Tcl_HashTable *options,
			       void **filterData);
static void xScrollFilterRender(SDL_Surface *originalSurface, 
			    SDL_Surface *renderSurface,
			    int offsetX, int offsetY, 
			    void *filterData);


static Uint32 getpixel16(SDL_Surface *surface, int x, int y) {
  Uint8 *p;
  
  int bpp = surface->format->BytesPerPixel;
  p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

  return *(Uint16 *)p;   
}


static void putpixel16(SDL_Surface *surface, int x, int y, Uint32 colour) {
  Uint16 *bufp;

  bufp = (Uint16 *)surface->pixels + y*surface->pitch/2 + x;
  *bufp = colour;
}


int xScrollFilterInit() {
  Oil_AddFilter("xScroll", xScrollFilterCreate, xScrollFilterRender);
  return TCL_OK;
}


SDL_Surface *xScrollFilterCreate(SDL_Surface *originalSurface, 
			       Tcl_HashTable *options,
			       void **filterData) {
  ScrollData *scrollData;

  scrollData = malloc(sizeof(ScrollData));
  scrollData->speed = 2;
  scrollData->speedcounter = 0;
  scrollData->step = 5;
  scrollData->xOffset = 0;
  *filterData=scrollData;

  return Oil_CreateDisplaySurface16(originalSurface->w, originalSurface->h);
}


static void xScrollFilterRender(SDL_Surface *originalSurface, 
				SDL_Surface *renderSurface, int offsetX, 
				int offsetY,
				void *filterData) {
  Uint32 bgcolor;
  SDL_Rect srcrect, dstrect;
  ScrollData *scrollData = filterData;

  if(scrollData->speedcounter++ % scrollData->speed == 0 ) {
    /* Clear the surface */
    bgcolor = originalSurface->format->colorkey;
	  
    if (SDL_FillRect( renderSurface, NULL, bgcolor ) != 0 )
      D2PRINTF(("Error in SDL_FillRect!"));
	
    /* Copy the right side of the final image. */
    srcrect.x = 0;
    srcrect.y = 0;
    srcrect.w = originalSurface->w - scrollData->xOffset;
    srcrect.h = originalSurface->h;

    dstrect.x = scrollData->xOffset;
    dstrect.y = srcrect.y;
    dstrect.w = srcrect.w;
    dstrect.h = srcrect.h;

    if (SDL_BlitSurface( originalSurface, &srcrect, renderSurface, 
			 &dstrect ) != 0)
      D2PRINTF(("Error in SDL_BlitSurface!"));

    /* Copy left part of final image (wrapped from right to left). */
    srcrect.x = originalSurface->w - scrollData->xOffset;
    srcrect.y = 0;
    srcrect.w = scrollData->xOffset;
    srcrect.h = originalSurface->h;

    dstrect.x = 0;
    dstrect.y = srcrect.y;
    dstrect.w = srcrect.w;
    dstrect.h = srcrect.h;

    if (SDL_BlitSurface( originalSurface, &srcrect, renderSurface, 
			 &dstrect ) != 0)
      D2PRINTF(("Error in SDL_BlitSurface!"));
	  
    scrollData->xOffset += scrollData->step;
    if( scrollData->xOffset >= originalSurface->w )
      scrollData->xOffset = 0;
  }

  return;
}
