/** Splits the object in half by moving every second line away from the object
 *
 * Authors: Taneli Korhonen, ttkorhon@cc.hut.fi
 *
 */


#include <SDL/SDL.h>
#include <tcl.h>
#include "filters.h"
#include "oil.h"
#include "debug.h"

/* Function prototypes. */

static Uint32 getpixel16(SDL_Surface *surface, int x, int y);
static void putpixel16(SDL_Surface *surface, int x, int y, Uint32 colour);
SDL_Surface *xSplit2FilterCreate(SDL_Surface *originalSurface, 
			       Tcl_HashTable *options,
			       void **filterData);
static void xSplit2FilterRender(SDL_Surface *originalSurface, 
			    SDL_Surface *renderSurface,
			    int offsetX, int offsetY, 
			    void *filterData);


static Uint32 getpixel16(SDL_Surface *surface, int x, int y) {
  Uint8 *p;
  
  int bpp = surface->format->BytesPerPixel;
  p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

  return *(Uint16 *)p;   
}


static void putpixel16(SDL_Surface *surface, int x, int y, Uint32 colour) {
  Uint16 *bufp;

  bufp = (Uint16 *)surface->pixels + y*surface->pitch/2 + x;
  *bufp = colour;
}


int xSplit2FilterInit() {
  Oil_AddFilter("xSplit2", xSplit2FilterCreate, xSplit2FilterRender);
  return TCL_OK;
}


SDL_Surface *xSplit2FilterCreate(SDL_Surface *originalSurface, 
			       Tcl_HashTable *options,
			       void **filterData) {
  return Oil_CreateDisplaySurface16(originalSurface->w, originalSurface->h);
}


static void xSplit2FilterRender(SDL_Surface *originalSurface, 
			    SDL_Surface *renderSurface, int offsetX, int offsetY,
			    void *filterData) {

  int y;
  //  int max_x_offset = originalSurface->w;
  //  static int dir = 1;
  //  static int current_x_offset = 0;
  static int speedcounter = 0;
  static int speed = 2; /* Speed; 1 = fastest */
  static int rand_tbl_counter = 0;
  Uint32 bgcolor;
  SDL_Rect srcrect, dstrect;

  int rand[500] = { 20, 22, 25, 1, 3, 27, 11, 13, 4, 13, 13, 3, 29, 26, 21, 11, 27, 12, 26, 17, 
		    26, 16, 9, 23, 3, 9, 16, 18, 26, 5, 25, 13, 2, 6, 17, 15, 27, 21, 7, 19, 
		    10, 30, 15, 28, 22, 14, 4, 17, 9, 2, 25, 19, 22, 9, 8, 30, 22, 26, 27, 4, 
		    17, 15, 22, 4, 27, 18, 27, 14, 24, 29, 17, 27, 4, 29, 6, 2, 19, 7, 9, 24, 
		    29, 26, 19, 16, 23, 14, 15, 28, 23, 20, 24, 12, 23, 23, 15, 24, 22, 21, 5, 5, 
		    19, 13, 13, 11, 22, 9, 25, 15, 20, 14, 17, 18, 1, 12, 11, 5, 11, 26, 21, 4, 
		    28, 25, 24, 14, 21, 20, 8, 4, 18, 16, 20, 11, 15, 15, 28, 19, 28, 24, 3, 26, 
		    8, 13, 24, 28, 6, 8, 26, 27, 30, 8, 20, 27, 23, 29, 16, 22, 24, 20, 11, 11, 
		    3, 12, 1, 16, 5, 10, 2, 15, 14, 15, 7, 1, 7, 1, 2, 29, 2, 22, 7, 4, 
		    19, 14, 21, 14, 22, 5, 6, 2, 20, 14, 28, 29, 24, 1, 20, 28, 27, 12, 21, 19, 
		    6, 22, 25, 27, 28, 26, 5, 2, 28, 6, 23, 13, 9, 28, 8, 3, 16, 19, 1, 19, 
		    25, 17, 9, 25, 14, 24, 15, 25, 19, 28, 25, 15, 20, 26, 16, 13, 9, 24, 8, 30, 
		    19, 19, 30, 22, 17, 23, 14, 5, 1, 16, 26, 26, 27, 24, 6, 2, 22, 8, 11, 28, 
		    13, 21, 18, 6, 11, 2, 28, 26, 29, 13, 9, 22, 19, 28, 2, 2, 16, 29, 24, 24, 
		    11, 21, 6, 4, 24, 4, 15, 27, 29, 18, 26, 11, 8, 4, 5, 18, 27, 13, 21, 25, 
		    1, 9, 20, 17, 21, 27, 20, 6, 11, 18, 6, 18, 18, 16, 23, 20, 9, 10, 28, 21, 
		    9, 9, 27, 5, 8, 7, 10, 12, 14, 7, 30, 3, 8, 5, 20, 22, 12, 17, 30, 29, 
		    9, 27, 25, 7, 22, 15, 18, 28, 27, 25, 7, 20, 14, 30, 15, 21, 1, 4, 10, 15, 
		    4, 20, 24, 27, 22, 19, 2, 27, 8, 25, 3, 18, 15, 1, 6, 17, 30, 12, 4, 9, 
		    15, 16, 13, 20, 22, 8, 20, 26, 21, 16, 3, 3, 20, 17, 18, 28, 27, 7, 15, 5, 
		    28, 15, 11, 15, 10, 13, 15, 3, 24, 17, 9, 12, 22, 28, 15, 7, 28, 29, 7, 21, 
		    20, 14, 21, 13, 7, 16, 10, 5, 2, 19, 4, 2, 25, 23, 25, 5, 3, 8, 22, 24, 
		    24, 5, 14, 15, 7, 25, 30, 1, 11, 30, 20, 21, 19, 4, 15, 29, 15, 10, 6, 5, 
		    18, 8, 30, 27, 24, 29, 2, 21, 17, 20, 9, 4, 14, 20, 12, 22, 5, 24, 9, 25, 
		    26, 2, 5, 10, 25, 6, 7, 11, 15, 17, 29, 30, 6, 9, 14, 9, 9, 24, 29, 29 };


  if( speedcounter++ % speed == 0 ) {
	  /* Clear the surface */
	  //	  bgcolor = SDL_MapRGB(renderSurface->format, 0, 0, 0);
	  bgcolor = originalSurface->format->colorkey;
	  if (SDL_FillRect( renderSurface, NULL, bgcolor ) != 0 )
		  D2PRINTF(("Error in SDL_FillRect!"));
	  
	  if( rand[rand_tbl_counter] < 15 ) {
		  
		  for( y = 0; y < originalSurface->h; y++ ) {
			  srcrect.x = 0;
			  srcrect.y = y;
			  srcrect.h = 1;
			  srcrect.w = originalSurface->w;
			  
			  if( y % 2 == 0 )
				  dstrect.x = rand[rand_tbl_counter];
			  else
				  dstrect.x = 0;
			  
			  dstrect.y = srcrect.y;
			  dstrect.h = originalSurface->h;
			  dstrect.w = originalSurface->w;
			  
			  if (SDL_BlitSurface( originalSurface, &srcrect, renderSurface, &dstrect ) != 0)
				  D2PRINTF(("Error in SDL_BlitSurface!"));
		  }
	  } else {
		  if (SDL_BlitSurface( originalSurface, NULL, renderSurface, NULL ) != 0)
			  D2PRINTF(("Error in SDL_BlitSurface!"));
	  }

	  rand_tbl_counter++;

	  if( rand_tbl_counter == 499 )
		  rand_tbl_counter = 0;

  }

  return;
}
