/** Splits the object in half by moving every second line away from the object
 *
 * Authors: Taneli Korhonen, ttkorhon@cc.hut.fi
 *
 */


#include <SDL/SDL.h>
#include <tcl.h>
#include "filters.h"
#include "oil.h"
#include "debug.h"

/* Function prototypes. */

static Uint32 getpixel16(SDL_Surface *surface, int x, int y);
static void putpixel16(SDL_Surface *surface, int x, int y, Uint32 colour);
SDL_Surface *xSplitFilterCreate(SDL_Surface *originalSurface, 
			       Tcl_HashTable *options,
			       void **filterData);
static void xSplitFilterRender(SDL_Surface *originalSurface, 
			    SDL_Surface *renderSurface,
			    int offsetX, int offsetY, 
			    void *filterData);


static Uint32 getpixel16(SDL_Surface *surface, int x, int y) {
  Uint8 *p;
  
  int bpp = surface->format->BytesPerPixel;
  p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

  return *(Uint16 *)p;   
}


static void putpixel16(SDL_Surface *surface, int x, int y, Uint32 colour) {
  Uint16 *bufp;

  bufp = (Uint16 *)surface->pixels + y*surface->pitch/2 + x;
  *bufp = colour;
}


int xSplitFilterInit() {
  Oil_AddFilter("xSplit", xSplitFilterCreate, xSplitFilterRender);
  return TCL_OK;
}


SDL_Surface *xSplitFilterCreate(SDL_Surface *originalSurface, 
			       Tcl_HashTable *options,
			       void **filterData) {
  return Oil_CreateDisplaySurface16(originalSurface->w, originalSurface->h);
}


static void xSplitFilterRender(SDL_Surface *originalSurface, 
			    SDL_Surface *renderSurface, int offsetX, int offsetY,
			    void *filterData) {

  int y;
  int max_x_offset = originalSurface->w;
  static int dir = 1;
  static int current_x_offset = 0;
  static int speedcounter = 0;
  static int speed = 2; /* Speed; 1 = fastest */
  Uint32 bgcolor;
  SDL_Rect srcrect, dstrect;

  if( speedcounter++ % speed == 0 ) {

	  //	  bgcolor = SDL_MapRGB(renderSurface->format, 0x00, 0x00, 0x00);
	  bgcolor = originalSurface->format->colorkey;
	  
	  /* Clear the surface */
	  if (SDL_FillRect( renderSurface, NULL, bgcolor ) != 0 )
		  D2PRINTF(("Error in SDL_FillRect!"));
	  
	  for( y = 0; y < originalSurface->h; y++ ) {
		  srcrect.x = 0;
		  srcrect.y = y;
		  srcrect.h = 1;
		  srcrect.w = originalSurface->w;
		  
		  if( y % 2 == 0 ) 
			  dstrect.x = current_x_offset;
		  else
			  dstrect.x = srcrect.x;
		  
		  dstrect.y = srcrect.y;
		  dstrect.h = srcrect.h;
		  dstrect.w = srcrect.w;
		  
		  if (SDL_BlitSurface( originalSurface, &srcrect, renderSurface, &dstrect ) != 0)
			  D2PRINTF(("Error in SDL_BlitSurface!"));
		  
	  }
	  
	  current_x_offset += dir;
	  
	  if( (current_x_offset == max_x_offset) || (current_x_offset < 0) )
		  dir *= -1;
  }
  return;
}
