/** Implements a very basic yFlag filter. 
 *
 * Authors: Taneli Korhonen, ttkorhon@cc.hut.fi
 *
 * $Header: /home/sleber/cvs/liboil/yFlagFilter.c,v 1.2 2003/01/08 16:04:56 sleber Exp $
 */


#include <SDL/SDL.h>
#include <tcl.h>
#include "filters.h"
#include "oil.h"
#include "debug.h"

/* Function prototypes. */

static Uint32 getpixel16(SDL_Surface *surface, int x, int y);
static void putpixel16(SDL_Surface *surface, int x, int y, Uint32 colour);
SDL_Surface *yFlagFilterCreate(SDL_Surface *originalSurface, 
			       Tcl_HashTable *options,
			       void **filterData);
static void yFlagFilterRender(SDL_Surface *originalSurface, 
			    SDL_Surface *renderSurface,
			    int offsetX, int offsetY, 
			    void *filterData);


static Uint32 getpixel16(SDL_Surface *surface, int x, int y) {
  Uint8 *p;
  
  int bpp = surface->format->BytesPerPixel;
  p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

  return *(Uint16 *)p;   
}


static void putpixel16(SDL_Surface *surface, int x, int y, Uint32 colour) {
  Uint16 *bufp;

  bufp = (Uint16 *)surface->pixels + y*surface->pitch/2 + x;
  *bufp = colour;
}


int yFlagFilterInit() {
  Oil_AddFilter("yFlag", yFlagFilterCreate, yFlagFilterRender);
  return TCL_OK;
}


SDL_Surface *yFlagFilterCreate(SDL_Surface *originalSurface, 
			       Tcl_HashTable *options,
			       void **filterData) {
  return Oil_CreateDisplaySurface16(originalSurface->w, originalSurface->h);
}


#define STRETCH_AMOUNT 0.03
static void yFlagFilterRender(SDL_Surface *originalSurface, 
			    SDL_Surface *renderSurface, int offsetX, int offsetY,
			    void *filterData) {
	int y; //, x;

	//  static double xZoom = 1, yZoom = 1;
	//  static int zoomdiff = -1;
  static int speedcounter = 0, speed = 5;
  static int dir = 1;
  static float xos = 0;
  static int sin_c = 0;

  Uint32 bgcolor; //, srcPixel
  SDL_Rect srcrect, dstrect;

  float sin[787] = { 0.00, 0.00, 0.01, 0.01, 0.02, 0.02, 0.02, 0.03, 0.03, 0.04, 
		     0.04, 0.04, 0.05, 0.05, 0.06, 0.06, 0.06, 0.07, 0.07, 0.08, 
		     0.08, 0.08, 0.09, 0.09, 0.10, 0.10, 0.10, 0.11, 0.11, 0.12, 
		     0.12, 0.12, 0.13, 0.13, 0.14, 0.14, 0.14, 0.15, 0.15, 0.16, 
		     0.16, 0.16, 0.17, 0.17, 0.18, 0.18, 0.18, 0.19, 0.19, 0.19, 
		     0.20, 0.20, 0.21, 0.21, 0.21, 0.22, 0.22, 0.23, 0.23, 0.23, 
		     0.24, 0.24, 0.25, 0.25, 0.25, 0.26, 0.26, 0.26, 0.27, 0.27, 
		     0.28, 0.28, 0.28, 0.29, 0.29, 0.30, 0.30, 0.30, 0.31, 0.31, 
		     0.31, 0.32, 0.32, 0.33, 0.33, 0.33, 0.34, 0.34, 0.34, 0.35, 
		     0.35, 0.36, 0.36, 0.36, 0.37, 0.37, 0.37, 0.38, 0.38, 0.39, 
		     0.39, 0.39, 0.40, 0.40, 0.40, 0.41, 0.41, 0.42, 0.42, 0.42, 
		     0.43, 0.43, 0.43, 0.44, 0.44, 0.44, 0.45, 0.45, 0.45, 0.46, 
		     0.46, 0.47, 0.47, 0.47, 0.48, 0.48, 0.48, 0.49, 0.49, 0.49, 
		     0.50, 0.50, 0.50, 0.51, 0.51, 0.51, 0.52, 0.52, 0.52, 0.53, 
		     0.53, 0.53, 0.54, 0.54, 0.54, 0.55, 0.55, 0.55, 0.56, 0.56, 
		     0.56, 0.57, 0.57, 0.57, 0.58, 0.58, 0.58, 0.59, 0.59, 0.59, 
		     0.60, 0.60, 0.60, 0.61, 0.61, 0.61, 0.62, 0.62, 0.62, 0.63, 
		     0.63, 0.63, 0.63, 0.64, 0.64, 0.64, 0.65, 0.65, 0.65, 0.66, 
		     0.66, 0.66, 0.67, 0.67, 0.67, 0.67, 0.68, 0.68, 0.68, 0.69, 
		     0.69, 0.69, 0.69, 0.70, 0.70, 0.70, 0.71, 0.71, 0.71, 0.71, 
		     0.72, 0.72, 0.72, 0.73, 0.73, 0.73, 0.73, 0.74, 0.74, 0.74, 
		     0.74, 0.75, 0.75, 0.75, 0.76, 0.76, 0.76, 0.76, 0.77, 0.77, 
		     0.77, 0.77, 0.78, 0.78, 0.78, 0.78, 0.79, 0.79, 0.79, 0.79, 
		     0.80, 0.80, 0.80, 0.80, 0.81, 0.81, 0.81, 0.81, 0.81, 0.82, 
		     0.82, 0.82, 0.82, 0.83, 0.83, 0.83, 0.83, 0.83, 0.84, 0.84, 
		     0.84, 0.84, 0.85, 0.85, 0.85, 0.85, 0.85, 0.86, 0.86, 0.86, 
		     0.86, 0.86, 0.87, 0.87, 0.87, 0.87, 0.87, 0.88, 0.88, 0.88, 
		     0.88, 0.88, 0.89, 0.89, 0.89, 0.89, 0.89, 0.89, 0.90, 0.90, 
		     0.90, 0.90, 0.90, 0.91, 0.91, 0.91, 0.91, 0.91, 0.91, 0.92, 
		     0.92, 0.92, 0.92, 0.92, 0.92, 0.92, 0.93, 0.93, 0.93, 0.93, 
		     0.93, 0.93, 0.93, 0.94, 0.94, 0.94, 0.94, 0.94, 0.94, 0.94, 
		     0.95, 0.95, 0.95, 0.95, 0.95, 0.95, 0.95, 0.95, 0.96, 0.96, 
		     0.96, 0.96, 0.96, 0.96, 0.96, 0.96, 0.96, 0.97, 0.97, 0.97, 
		     0.97, 0.97, 0.97, 0.97, 0.97, 0.97, 0.97, 0.98, 0.98, 0.98, 
		     0.98, 0.98, 0.98, 0.98, 0.98, 0.98, 0.98, 0.98, 0.98, 0.98, 
		     0.99, 0.99, 0.99, 0.99, 0.99, 0.99, 0.99, 0.99, 0.99, 0.99, 
		     0.99, 0.99, 0.99, 0.99, 0.99, 0.99, 0.99, 0.99, 1.00, 1.00, 
		     1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 
		     1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 
		     1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 
		     1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 
		     1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 0.99, 0.99, 
		     0.99, 0.99, 0.99, 0.99, 0.99, 0.99, 0.99, 0.99, 0.99, 0.99, 
		     0.99, 0.99, 0.99, 0.99, 0.99, 0.99, 0.99, 0.98, 0.98, 0.98, 
		     0.98, 0.98, 0.98, 0.98, 0.98, 0.98, 0.98, 0.98, 0.98, 0.97, 
		     0.97, 0.97, 0.97, 0.97, 0.97, 0.97, 0.97, 0.97, 0.97, 0.97, 
		     0.96, 0.96, 0.96, 0.96, 0.96, 0.96, 0.96, 0.96, 0.95, 0.95, 
		     0.95, 0.95, 0.95, 0.95, 0.95, 0.95, 0.94, 0.94, 0.94, 0.94, 
		     0.94, 0.94, 0.94, 0.94, 0.93, 0.93, 0.93, 0.93, 0.93, 0.93, 
		     0.93, 0.92, 0.92, 0.92, 0.92, 0.92, 0.92, 0.91, 0.91, 0.91, 
		     0.91, 0.91, 0.91, 0.90, 0.90, 0.90, 0.90, 0.90, 0.90, 0.89, 
		     0.89, 0.89, 0.89, 0.89, 0.88, 0.88, 0.88, 0.88, 0.88, 0.88, 
		     0.87, 0.87, 0.87, 0.87, 0.87, 0.86, 0.86, 0.86, 0.86, 0.86, 
		     0.85, 0.85, 0.85, 0.85, 0.84, 0.84, 0.84, 0.84, 0.84, 0.83, 
		     0.83, 0.83, 0.83, 0.82, 0.82, 0.82, 0.82, 0.82, 0.81, 0.81, 
		     0.81, 0.81, 0.80, 0.80, 0.80, 0.80, 0.79, 0.79, 0.79, 0.79, 
		     0.78, 0.78, 0.78, 0.78, 0.77, 0.77, 0.77, 0.77, 0.76, 0.76, 
		     0.76, 0.76, 0.75, 0.75, 0.75, 0.75, 0.74, 0.74, 0.74, 0.73, 
		     0.73, 0.73, 0.73, 0.72, 0.72, 0.72, 0.72, 0.71, 0.71, 0.71, 
		     0.70, 0.70, 0.70, 0.70, 0.69, 0.69, 0.69, 0.68, 0.68, 0.68, 
		     0.68, 0.67, 0.67, 0.67, 0.66, 0.66, 0.66, 0.65, 0.65, 0.65, 
		     0.65, 0.64, 0.64, 0.64, 0.63, 0.63, 0.63, 0.62, 0.62, 0.62, 
		     0.61, 0.61, 0.61, 0.60, 0.60, 0.60, 0.60, 0.59, 0.59, 0.59, 
		     0.58, 0.58, 0.58, 0.57, 0.57, 0.57, 0.56, 0.56, 0.56, 0.55, 
		     0.55, 0.55, 0.54, 0.54, 0.54, 0.53, 0.53, 0.53, 0.52, 0.52, 
		     0.52, 0.51, 0.51, 0.51, 0.50, 0.50, 0.49, 0.49, 0.49, 0.48, 
		     0.48, 0.48, 0.47, 0.47, 0.47, 0.46, 0.46, 0.46, 0.45, 0.45, 
		     0.45, 0.44, 0.44, 0.43, 0.43, 0.43, 0.42, 0.42, 0.42, 0.41, 
		     0.41, 0.41, 0.40, 0.40, 0.39, 0.39, 0.39, 0.38, 0.38, 0.38, 
		     0.37, 0.37, 0.36, 0.36, 0.36, 0.35, 0.35, 0.35, 0.34, 0.34, 
		     0.33, 0.33, 0.33, 0.32, 0.32, 0.32, 0.31, 0.31, 0.30, 0.30, 
		     0.30, 0.29, 0.29, 0.29, 0.28, 0.28, 0.27, 0.27, 0.27, 0.26, 
		     0.26, 0.25, 0.25, 0.25, 0.24, 0.24, 0.24, 0.23, 0.23, 0.22, 
		     0.22, 0.22, 0.21, 0.21, 0.20, 0.20, 0.20, 0.19, 0.19, 0.18, 
		     0.18, 0.18, 0.17, 0.17, 0.16, 0.16, 0.16, 0.15, 0.15, 0.15, 
		     0.14, 0.14, 0.13, 0.13, 0.13, 0.12, 0.12, 0.11, 0.11, 0.11, 
		     0.10, 0.10, 0.09, 0.09, 0.09, 0.08, 0.08, 0.07, 0.07, 0.07, 
		     0.06, 0.06, 0.05, 0.05, 0.05, 0.04, 0.04, 0.03, 0.03, 0.03, 
		     0.02, 0.02, 0.01, 0.01, 0.01, 0.00, 0.00 };


  if( speedcounter++ % speed == 0 ) {
	  
	  //          bgcolor = SDL_MapRGB(renderSurface->format, 0x00, 0x00, 0x00);
          
	  bgcolor = originalSurface->format->colorkey;
          /* Clear the surface */
          if (SDL_FillRect( renderSurface, NULL, bgcolor ) != 0 )
                  D2PRINTF(("Error in SDL_FillRect!"));
          
          for( y = 0; y < originalSurface->h; y++  ) {
                  srcrect.x = 0;
                  srcrect.y = y;
                  srcrect.h = 1;
                  srcrect.w = originalSurface->w;
                  
		  dstrect.x = (int)xos;
                  dstrect.y = srcrect.y;
                  dstrect.h = srcrect.h;
                  dstrect.w = srcrect.w;
                  
		  xos += dir * sin[sin_c] / 2;

		  sin_c += 13;
		  if( sin_c > 786 ) {
			  dir *= -1;
			  sin_c = 0;
		  }


	  if (SDL_BlitSurface( originalSurface, &srcrect, renderSurface, &dstrect ) != 0)
		  D2PRINTF(("Error in SDL_BlitSurface!"));
          }

  }       
  return;
}
