/** Scrolls the object in Y direction
 *
 * Authors: Taneli Korhonen, ttkorhon@cc.hut.fi
 *
 */


#include <SDL/SDL.h>
#include <tcl.h>
#include "filters.h"
#include "oil.h"
#include "debug.h"

/* Function prototypes. */

static Uint32 getpixel16(SDL_Surface *surface, int x, int y);
static void putpixel16(SDL_Surface *surface, int x, int y, Uint32 colour);
SDL_Surface *yScrollFilterCreate(SDL_Surface *originalSurface, 
			       Tcl_HashTable *options,
			       void **filterData);
static void yScrollFilterRender(SDL_Surface *originalSurface, 
			    SDL_Surface *renderSurface,
			    int offsetX, int offsetY, 
			    void *filterData);


static Uint32 getpixel16(SDL_Surface *surface, int x, int y) {
  Uint8 *p;
  
  int bpp = surface->format->BytesPerPixel;
  p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

  return *(Uint16 *)p;   
}


static void putpixel16(SDL_Surface *surface, int x, int y, Uint32 colour) {
  Uint16 *bufp;

  bufp = (Uint16 *)surface->pixels + y*surface->pitch/2 + x;
  *bufp = colour;
}


int yScrollFilterInit() {
  Oil_AddFilter("yScroll", yScrollFilterCreate, yScrollFilterRender);
  return TCL_OK;
}


SDL_Surface *yScrollFilterCreate(SDL_Surface *originalSurface, 
			       Tcl_HashTable *options,
			       void **filterData) {
  return Oil_CreateDisplaySurface16(originalSurface->w, originalSurface->h);
}


static void yScrollFilterRender(SDL_Surface *originalSurface, 
			    SDL_Surface *renderSurface, int offsetX, int offsetY,
			    void *filterData) {

	//  static int dir = 1;
  static int y_offset = 0;
  static int speedcounter = 0;
  static int speed = 2; /* Speed; 1 = fastest */
  static int step = 5;
  Uint32 bgcolor;
  SDL_Rect srcrect, dstrect;

  if( speedcounter++ % speed == 0 ) {

    /* Copy what is to be right side of picture. */
	  srcrect.x = 0;
	  srcrect.y = 0;
	  srcrect.w = originalSurface->w;
	  srcrect.h = originalSurface->h - y_offset;

	  dstrect.x = srcrect.x;
	  dstrect.y = y_offset;
	  dstrect.w = srcrect.w;
	  dstrect.h = srcrect.h;

	  if (SDL_BlitSurface( originalSurface, &srcrect, renderSurface, &dstrect ) != 0)
		  D2PRINTF(("Error in SDL_BlitSurface!"));

	  /* And left side. */
	  srcrect.x = 0;
	  srcrect.y = originalSurface->h - y_offset;
	  srcrect.w = originalSurface->w;
	  srcrect.h = y_offset;

	  dstrect.x = srcrect.x;
	  dstrect.y = 0;
	  dstrect.w = srcrect.w;
	  dstrect.h = srcrect.h;

	  if (SDL_BlitSurface( originalSurface, &srcrect, renderSurface, &dstrect ) != 0)
		  D2PRINTF(("Error in SDL_BlitSurface!"));
	  
	  /* Pitisi viel tarkistaa, kopioituuko yksi rivi kahteen kertaan */

	  y_offset += step;

	  if( y_offset >= originalSurface->h )
		  y_offset = 0;
  }
  return;
}
