/** Splits the object in half by moving every second line away from the object
 *
 * Authors: Taneli Korhonen, ttkorhon@cc.hut.fi
 *
 */


#include <SDL/SDL.h>
#include <tcl.h>
#include "filters.h"
#include "oil.h"
#include "debug.h"

/* Function prototypes. */

static Uint32 getpixel16(SDL_Surface *surface, int x, int y);
static void putpixel16(SDL_Surface *surface, int x, int y, Uint32 colour);
SDL_Surface *ySplit2FilterCreate(SDL_Surface *originalSurface, 
			       Tcl_HashTable *options,
			       void **filterData);
static void ySplit2FilterRender(SDL_Surface *originalSurface, 
			    SDL_Surface *renderSurface,
			    int offsetX, int offsetY, 
			    void *filterData);


static Uint32 getpixel16(SDL_Surface *surface, int x, int y) {
  Uint8 *p;
  
  int bpp = surface->format->BytesPerPixel;
  p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

  return *(Uint16 *)p;   
}


static void putpixel16(SDL_Surface *surface, int x, int y, Uint32 colour) {
  Uint16 *bufp;

  bufp = (Uint16 *)surface->pixels + y*surface->pitch/2 + x;
  *bufp = colour;
}


int ySplit2FilterInit() {
  Oil_AddFilter("ySplit2", ySplit2FilterCreate, ySplit2FilterRender);
  return TCL_OK;
}


SDL_Surface *ySplit2FilterCreate(SDL_Surface *originalSurface, 
			       Tcl_HashTable *options,
			       void **filterData) {
  return Oil_CreateDisplaySurface16(originalSurface->w, originalSurface->h);
}


static void ySplit2FilterRender(SDL_Surface *originalSurface, 
			    SDL_Surface *renderSurface, int offsetX, int offsetY,
			    void *filterData) {

  int x;
  //  int max_x_offset = originalSurface->w;
  //  static int dir = 1;
  //  static int current_x_offset = 0;
  static int speedcounter = 0;
  static int speed = 2; /* Speed; 1 = fastest */
  static int rand_tbl_counter = 0;
  Uint32 bgcolor;
  SDL_Rect srcrect, dstrect;

  int rand[500] = { 16, 3, 3, 28, 25, 14, 24, 6, 4, 14, 3, 13, 15, 16, 16, 15, 29, 26, 3, 11, 
		    22, 27, 26, 8, 17, 22, 7, 12, 26, 17, 12, 20, 1, 24, 28, 14, 9, 22, 21, 10, 
		    22, 1, 15, 30, 3, 22, 9, 27, 5, 9, 10, 30, 14, 19, 16, 6, 1, 8, 18, 15, 
		    22, 18, 6, 8, 12, 30, 26, 12, 3, 16, 24, 30, 27, 7, 11, 16, 12, 14, 15, 6, 
		    4, 24, 21, 10, 1, 30, 1, 1, 19, 17, 5, 23, 19, 27, 6, 2, 6, 23, 16, 7, 
		    14, 2, 2, 18, 10, 7, 13, 2, 21, 4, 1, 15, 27, 10, 27, 8, 6, 14, 15, 9, 
		    14, 10, 13, 4, 30, 12, 20, 2, 13, 20, 8, 13, 23, 14, 14, 12, 4, 20, 20, 23, 
		    22, 9, 11, 25, 29, 11, 11, 13, 18, 23, 28, 18, 13, 30, 22, 5, 13, 5, 27, 6, 
		    11, 2, 21, 11, 23, 3, 21, 7, 10, 17, 18, 6, 7, 20, 30, 20, 21, 9, 19, 26, 
		    8, 2, 25, 27, 2, 30, 27, 5, 18, 24, 27, 30, 7, 30, 2, 20, 27, 29, 6, 6, 
		    25, 24, 24, 17, 20, 30, 1, 6, 28, 4, 6, 7, 3, 16, 19, 19, 7, 3, 20, 6, 
		    27, 4, 21, 24, 10, 20, 1, 13, 25, 6, 13, 16, 5, 16, 26, 12, 22, 18, 23, 10, 
		    29, 22, 23, 27, 16, 11, 10, 16, 18, 18, 7, 30, 20, 21, 22, 16, 19, 20, 30, 10, 
		    13, 28, 26, 13, 20, 5, 6, 26, 23, 30, 7, 13, 17, 9, 16, 20, 10, 3, 8, 20, 
		    4, 20, 11, 18, 15, 17, 7, 19, 24, 22, 2, 2, 8, 5, 25, 12, 22, 11, 5, 29, 
		    14, 30, 21, 23, 20, 11, 4, 14, 6, 18, 28, 25, 16, 15, 28, 7, 22, 27, 11, 30, 
		    3, 5, 12, 17, 29, 30, 7, 30, 2, 12, 15, 5, 3, 29, 9, 24, 29, 30, 14, 21, 
		    24, 10, 30, 8, 11, 22, 28, 22, 6, 30, 2, 23, 7, 26, 11, 11, 15, 30, 7, 11, 
		    7, 22, 17, 21, 1, 17, 13, 25, 26, 15, 8, 16, 20, 4, 13, 24, 9, 19, 21, 9, 
		    4, 29, 15, 5, 10, 3, 29, 30, 6, 17, 18, 14, 11, 29, 7, 25, 27, 20, 10, 28, 
		    27, 16, 14, 20, 7, 9, 18, 10, 1, 15, 26, 19, 8, 24, 18, 11, 2, 18, 1, 15, 
		    19, 30, 5, 14, 28, 25, 17, 10, 5, 22, 16, 1, 4, 14, 14, 8, 28, 15, 22, 15, 
		    9, 23, 16, 17, 9, 10, 23, 4, 9, 22, 27, 21, 14, 1, 24, 10, 17, 15, 26, 28, 
		    4, 9, 12, 6, 30, 4, 5, 14, 13, 15, 5, 28, 21, 15, 25, 3, 8, 13, 5, 20, 
		    13, 27, 13, 26, 24, 6, 21, 13, 11, 11, 25, 10, 10, 2, 14, 6, 12, 7, 18, 18 };


  if( speedcounter++ % speed == 0 ) {
	  /* Clear the surface */
	  //	  bgcolor = SDL_MapRGB(renderSurface->format, 0, 0, 0);
	  bgcolor = originalSurface->format->colorkey;
	  if (SDL_FillRect( renderSurface, NULL, bgcolor ) != 0 )
		  D2PRINTF(("Error in SDL_FillRect!"));
	  
	  if( rand[rand_tbl_counter] < 15 ) {
		  
		  for( x = 0; x < originalSurface->w; x++ ) {
			  srcrect.x = x;
			  srcrect.y = 0;
			  srcrect.h = originalSurface->h;
			  srcrect.w = 1;
			  
			  if( x % 2 == 0 )
				  dstrect.y = rand[rand_tbl_counter];
			  else
				  dstrect.y = 0;
			  
			  dstrect.x = srcrect.x;
			  dstrect.h = originalSurface->h;
			  dstrect.w = originalSurface->w;
			  
			  if (SDL_BlitSurface( originalSurface, &srcrect, renderSurface, &dstrect ) != 0)
				  D2PRINTF(("Error in SDL_BlitSurface!"));
		  }
	  } else {
		  if (SDL_BlitSurface( originalSurface, NULL, renderSurface, NULL ) != 0)
			  D2PRINTF(("Error in SDL_BlitSurface!"));
	  }

	  rand_tbl_counter++;

	  if( rand_tbl_counter == 499 )
		  rand_tbl_counter = 0;

  }
  return;
}
