/** Implements a very basic yStretch filter. 
 *
 * Authors: Taneli Korhonen, ttkorhon@cc.hut.fi
 *
 * $Header: /home/sleber/cvs/liboil/yStretchFilter.c,v 1.2 2003/01/08 16:04:56 sleber Exp $
 */


#include <SDL/SDL.h>
#include <tcl.h>
#include "filters.h"
#include "oil.h"
#include "debug.h"

/* Function prototypes. */

static Uint32 getpixel16(SDL_Surface *surface, int x, int y);
static void putpixel16(SDL_Surface *surface, int x, int y, Uint32 colour);
SDL_Surface *yStretchFilterCreate(SDL_Surface *originalSurface, 
			       Tcl_HashTable *options,
			       void **filterData);
static void yStretchFilterRender(SDL_Surface *originalSurface, 
			    SDL_Surface *renderSurface,
			    int offsetX, int offsetY, 
			    void *filterData);


static Uint32 getpixel16(SDL_Surface *surface, int x, int y) {
  Uint8 *p;
  
  int bpp = surface->format->BytesPerPixel;
  p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

  return *(Uint16 *)p;   
}


static void putpixel16(SDL_Surface *surface, int x, int y, Uint32 colour) {
  Uint16 *bufp;

  bufp = (Uint16 *)surface->pixels + y*surface->pitch/2 + x;
  *bufp = colour;
}


int yStretchFilterInit() {
  Oil_AddFilter("yStretch", yStretchFilterCreate, yStretchFilterRender);
  return TCL_OK;
}


SDL_Surface *yStretchFilterCreate(SDL_Surface *originalSurface, 
			       Tcl_HashTable *options,
			       void **filterData) {
  return Oil_CreateDisplaySurface16(originalSurface->w, originalSurface->h);
}


#define STRETCH_AMOUNT 0.03
static void yStretchFilterRender(SDL_Surface *originalSurface, 
			    SDL_Surface *renderSurface, int offsetX, int offsetY,
			    void *filterData) {
  int y;

  //  static double xZoom = 1, yZoom = 1;
  //  static int zoomdiff = -1;
  static int speedcounter = 0, speed = 3;
  static int dir = 1;
  static float yos = 0;
  static int sin_c = 0;

  Uint32 bgcolor; //, srcPixel
  SDL_Rect srcrect, dstrect;

  float sin[33] = { 0.00, 0.10, 0.20, 0.30, 0.39, 0.48, 0.56, 0.64, 0.72, 0.78, 
		    0.84, 0.89, 0.93, 0.96, 0.99, 1.00, 1.00, 0.99, 0.97, 0.95, 
		    0.91, 0.86, 0.81, 0.75, 0.68, 0.60, 0.52, 0.43, 0.33, 0.24, 
		    0.14, 0.04, 0.00 };

  if( speedcounter++ % speed == 0 ) {

	  //          bgcolor = SDL_MapRGB(renderSurface->format, 0x00, 0x00, 0x00);
	  bgcolor = originalSurface->format->colorkey;
          
          /* Clear the surface */
          if (SDL_FillRect( renderSurface, NULL, bgcolor ) != 0 )
                  D2PRINTF(("Error in SDL_FillRect!"));
          
          for( y = 0; y < originalSurface->h; y++ ) {
                  srcrect.x = 0;
                  srcrect.y = y;
                  srcrect.h = 1;
                  srcrect.w = originalSurface->w;
                  
                  dstrect.x = srcrect.x;
		  dstrect.y = (int)srcrect.y*yos;
                  dstrect.h = srcrect.h;
                  dstrect.w = srcrect.w;
                  
                  if (SDL_BlitSurface( originalSurface, &srcrect, renderSurface, &dstrect ) != 0)
                          D2PRINTF(("Error in SDL_BlitSurface!"));
                  
          }
	  yos += dir * sin[sin_c]/10;

	  ++sin_c;
	  if( sin_c > 32 ) {
		  dir *= -1;
		  sin_c = 0;
	  }
  }       
  return;
}
